/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TSampleSQryDB2.cpp
 * Description    : Connect to DB2 Datastore
 *                  Create query string
 *                  specify to retrieve folder and docs.
 *                  Create a parametric query object and submit query
 *                  get the results in a collection.
 *                  display documents.
 *                  recursively display folder and its document/
 *                  folder content retrieving each document/folder
 *                  as it goes.
 *                  Disconnect from DB2 Datastore
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDB2.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <DKSQLQuery.hpp>
#include <dkCollection.hpp>
#include <DKFolder.hpp>
#include <DKParts.hpp>

void processResults(dkCollection* results);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   //-------------------------------------------------------------- 
   // Checking for input parameters
   //--------------------------------------------------------------   
   if (argc < 3)
   {
       cout<<"Invoke Syntax: "<<endl;
       cout<<"  SampleSQryDB2 <database> <userid> <pw>" << endl;
       return;
   }
 
   // ### get input parameters
   DKString database = DKString(argv[1]);
   DKString userid = DKString(argv[2]);
   DKString pw = DKString(argv[3]);
   DKDatastoreDB2 dsDB2; // must be outside try/catch block
   try {
         dkQuery* pQry;
         DKAny any;
         DKResults* pResults;
         DKNVPair par[2];
         DKString strMax = "5";
         any = strMax;
         par[0].set(DK_CM_PARM_MAX_RESULTS, any);
         par[1].setName(DK_CM_PARM_END);

         cout << "connecting to datastore" << endl;
         dsDB2.connect(database,userid,pw);
         cout << "datastore connected " << database << " userid - " << userid << endl;
         DKString cmd = "SELECT * FROM EMPLOYEE";
         cout << "query string " << cmd << endl;
         cout << "create query" << endl;
         pQry = dsDB2.createQuery(cmd,DK_CM_SQL_QL_TYPE);
         cout << "executing query" << endl;
         pQry->execute(par);
         cout << "query executed" << endl;
         cout << "get query results" << endl;
         any = pQry->result(); 
         pResults = (DKResults*)((dkCollection*) any);
         
         processResults(pResults);

         dsDB2.commit();
         dsDB2.disconnect();
         cout << "datastore disconnected" << endl;
         delete pQry;
         delete pResults;
    }
    catch(DKException &exc)
    {
         cout << "Error id " << exc.errorId() << endl;
         for(unsigned long i = 0; i < exc.textCount(); i++) {
             cout << "Error text: " << exc.text(i) << endl;
         }
         for (unsigned long j=0; j < exc.locationCount(); j++) {
              const DKExceptionLocation* p = exc.locationAtIndex(j);
              cout << "Filename: "   << p->fileName()     << endl;
              cout << "Function: "   << p->functionName() << endl;
              cout << "LineNumber: " << p->lineNumber()   << endl;
         }
         cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;
}

void processResults(dkCollection* results)
{
   ushort itemType;
   DKDDO* item;
   ushort i;
   DKString dataName;
   DKAny any;

   dkIterator* iter = results->createIterator();
   DKAny *element;

   while(iter->more()) {
      element = iter->next();
      item = (DKDDO*) element->value();
      if (!item) continue;
//      item->retrieve();  //Only valid if Primary Key exists for Table
      itemType = item->getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
      switch(itemType) {
        case DK_CM_ITEM:
	    {
             cout << " Item found, " 
                  << "   number of data item " << item->dataCount() << endl;
             for (i = 1; i <= item->dataCount(); i++) {
                 dataName = item->getDataName(i);
                 cout << "    data-item[" << i << "] = "
                      << dataName  << ", \t\t value "
                      << item->getData(i) << endl;
                 }   
             }
             break;
        default:
             cout << " Item is not recognized " << endl;
      }     
   }
   delete iter;
}
