/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TDatastorePoolDJ.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKClobDJ.hpp>
#include <DKDatastoreDJ.hpp>
#include <DKDatastorePool.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKResults.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKPid.hpp>
#include <DKParametricQuery.hpp>
#include <dkCollection.hpp>
#include <DKFolder.hpp>
#include <DKParts.hpp>
#include <DKEnvironment.hpp>

void processResults(dkCollection* results);

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);
   DKString schema = ""; 
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TDatastorePoolDJ <database> <userid> <pw>" << endl;
     return;
    }
    else if (argc == 5)
    {
      schema = DKString(argv[4]);
    }
    // ### get input parameters
    DKString database = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKString factoryDJLName = "libcmbdjfac815";

   try {
      DKDatastorePool* dsrDJ = new DKDatastorePool("DJ", factoryDJLName);
      dsrDJ->setDatastoreName(database);
      dsrDJ->setMaxPoolSize(10);
      dsrDJ->setMinPoolSize(3);
      dsrDJ->setTimeOut(1); 

      dsrDJ->initConnections(userid, pw, 2);
      //dsrDJ->setTimeOut(1); 
      cout<<" #getMaxPoolSize =  " << dsrDJ->getMaxPoolSize()<<endl;
      cout<<" #getMinPoolSize =  " << dsrDJ->getMinPoolSize()<<endl;
      cout<<" #getTimeOut=  " << dsrDJ->getTimeOut()<<endl;

      dkDatastore *dsDJ1, *dsDJ2, *dsDJ3, *dsDJ4;;
      dsDJ1 = dsrDJ->getConnection(userid, pw);  //new ds1
      cout<<" #getConnection, dsDJ1(ds1) =  " << dsDJ1 << endl;
      dsDJ2 = dsrDJ->getConnection(userid, pw);  // new ds2
      cout<<" #getConnection, dsDJ2(ds2) =  " << dsDJ2 << endl;

      DKDatastoreSession *dsIdDJ1, *dsIdDJ2, *dsIdDJ3, *dsIdDJ4;
      dsIdDJ1 = dsrDJ->getConnection(userid, pw, 0);        //new dsid1
      cout<<" #getConnection, dsIdDJ1(dsid1)="<< dsIdDJ1->getDatastore()
        <<", sessionId=" << dsIdDJ1->getId()<<endl;
      dsIdDJ2 = dsrDJ->getConnection(userid, pw, 0);        //new dsid2
      cout<<" #getConnection, dsIdDJ2(dsid2)="<<dsIdDJ2->getDatastore()
        <<", sessionId=" << dsIdDJ2->getId()<<endl;

      dsrDJ->returnConnection(dsIdDJ2);                     //dsid2 in _freeIdPool
      cout<<" #returnConnection, dsIdDJ2(dsid2) =  " << 
        dsIdDJ2->getDatastore() << ", sessionId=" << dsIdDJ2->getId()<<endl;
      dsrDJ->returnConnection(dsDJ2);                       //ds2 in _freePool
      cout<<" #returnConnection, dsDJ2(ds2) =  " << dsDJ2 <<endl; 

      dsIdDJ3 = dsrDJ->getConnection(userid, pw, 1); //ds2 in _usedPool
      cout<<" #getConnection, dsIdDJ3(ds2)=" <<
        dsIdDJ3->getDatastore() << ", sessionId=" << dsIdDJ3->getId()<<endl;
      dsIdDJ4 = dsrDJ->getConnection(userid, pw, 5); //new dsid3  
      cout<<" #getConnection, dsIdDJ4(dsid3)=" <<
        dsIdDJ4->getDatastore() << ", sessionId=" << dsIdDJ4->getId() <<endl;


      dsIdDJ4 = dsrDJ->getConnection(userid, pw, 6); // new dsid4
      cout<< " #getConnection, dsIdDJ4(dsid4)=" <<
        dsIdDJ4->getDatastore() << ", sessionId=" << dsIdDJ4->getId()<<endl;

      dsDJ1 = dsrDJ->getConnection(userid, pw);  // new ds3
      cout<<" #getConnection, dsDJ1(ds3) =  " << dsDJ1 <<endl;
      dsDJ2 = dsrDJ->getConnection(userid, pw);  // new ds4
      cout<<" #getConnection, dsDJ2(ds4) =  " << dsDJ2 <<endl;
 
      dsDJ3 = dsrDJ->getConnection(userid, pw);  // new ds5
      cout<< " #getConnection, dsDJ3(ds5) =  " << dsDJ3 <<endl;
      dsDJ4 = dsrDJ->getConnection(userid, pw);  // new ds6
      cout<<" #getConnection, dsDJ4(ds6) =  " << dsDJ4 <<endl;


      dsIdDJ3 = dsrDJ->getConnection(userid, pw, 9); // dsid2 
      cout<<" #getConnection, dsIdDJ3(dsid2)=" <<
      dsIdDJ3->getDatastore() << ", sessionId=" << dsIdDJ3->getId()<<endl;


/**
      // **** use dsDJ2 to do query
      cout<<"||**** start: use dsDJ2 to do query ****||"<<endl;
         dkQuery* pQry;
         DKAny any;
         DKString cmd; 
         DKResults* pResults;
         DKNVPair par[2];
         DKString strMax = "5";
         any = strMax;
         par[0].set(DK_CM_PARM_MAX_RESULTS, any);
         par[1].setName(DK_CM_PARM_END);

         if (schema == "") 
          cmd = "SELECT * FROM EMPLOYEE";
         else
          cmd = "SELECT * FROM " + schema + ".EMPLOYEE";  
    
         cout << "query string " << cmd << endl;
         cout << "create query" << endl;
         pQry = dsDJ2->createQuery(cmd,DK_CM_SQL_QL_TYPE);
         cout << "executing query" << endl;
         pQry->execute(par);
         cout << "query executed" << endl;
         cout << "get query results" << endl;
         any = pQry->result();
         pResults = (DKResults*)((dkCollection*) any);

         processResults(pResults);

         dsDJ2->commit();
         delete pQry;
         delete pResults;
      cout<<"||**** end: use dsDJ2 to do query ****||"<<endl;
      // ****end of use dsDJ2 to do query 

**/
      dsrDJ->returnConnection(dsDJ2);
      cout<<" #returnConnection, dsDJ2(ds4) =  "<< dsDJ2 <<endl;
      dsDJ4 = dsrDJ->getConnection(userid, pw);
      cout<<" #getConnection, dsDJ4 =  "<< dsDJ4 <<endl;
      cout<<"    dsDJ4 == dsDJ2 =  " <<(dsDJ4 == dsDJ2)<<endl;
/**
      // **** use dsDJ4 to retrieve XDO  
      cout<<"||**** start: use dsDJ4 to retrieve XDO ****||"<<endl;
       DKString table;                           //from SAMPLE database of DJ
       DKString column = "RESUME";               //blob column 
       DKString predicate = "EMPNO = '000130' and RESUME_FORMAT = 'ascii'" ; //primary key of exist clob
       if (schema == "") 
         table = "EMP_RESUME"; 
       else
         table = schema + ".EMP_RESUME"; 
       DKClobDJ* axdo = new DKClobDJ((DKDatastoreDJ*)dsDJ4);
       DKPidXDODJ*  apid = new DKPidXDODJ;
       apid->setTable(table);
       apid->setColumn(column);
       apid->setDataPredicate(predicate);   //exist blob
       axdo->setPidObject(apid);
       cout<<"table= "<<axdo->getTable()<<endl;
       cout<<"column="<<((DKPidXDODJ*)(axdo->getPid()))->getColumn()<<endl;
       cout<<"dataPredicate="<<((DKPidXDODJ*)(axdo->getPid()))->getDataPredicate()<<endl;
       cout<<"about to do retrieve()..."<<endl;
       axdo->retrieve("connV4.ascii");
       cout<<"retrieve successfully..."<<endl;
       cout<<"about to do open()"<<endl;
       axdo->setInstanceOpenHandler("notepad ", TRUE); //use notepad  in NT
       axdo->open(); 
       dsDJ4->commit();
       delete axdo;
       delete apid;   
      cout<<"||**** end: use dsDJ4 to retrieve XDO ****||"<<endl;
      // ****end of use dsDJ4 to retrieve XDO
**/
      dsrDJ->returnConnection(dsDJ1);
      cout<<" #returnConnection, dsDJ1(ds3) =  " << dsDJ1 <<endl;
      dsrDJ->returnConnection(dsDJ3);
      cout<<" #returnConnection, dsDJ3(ds5) =  " << dsDJ3 <<endl;
      dsrDJ->returnConnection(dsIdDJ4); //dsid4 in _freeIdPool
      cout<<" #returnConnection, dsIdDJ4(dsid4) =  " <<
        dsIdDJ4->getDatastore() << ", sessionId=" << dsIdDJ4->getId() <<endl;
     
/***
      cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
     Sleep(120050);     
#elif AIX
     sleep(120050);
#endif    
      cout<<"**** end sleep for 120050 mini sec*******"<<endl;
***/

      dsDJ2 = (DKDatastoreDJ*)dsrDJ->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsDJ2(ds7) =  " << dsDJ2<<endl;
      cout<<" #call delete dsrDJ..." <<endl;
      delete dsrDJ;  //take a while for all 4 in usedPool to be disconnected
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}

void processResults(dkCollection* results)
{
   ushort itemType;
   DKDDO* item;
   ushort i;
   DKString dataName;
   DKAny any;

   dkIterator* iter = results->createIterator();
   DKAny *element;

   while(iter->more()) {
      element = iter->next();
      item = (DKDDO*) element->value();
      if (!item) continue;
//      item->retrieve();  //Only valid if Primary Key exists for Table
      itemType = item->getPropertyByName(DK_CM_PROPERTY_ITEM_TYPE);
      switch(itemType) {
        case DK_CM_DOCUMENT:
      {
             cout << " Item is a document, "
                  << "   number of data item " << item->dataCount() << endl;
             for (i = 1; i <= item->dataCount(); i++) {
                 dataName = item->getDataName(i);
                 cout << "    data-item[" << i << "] = "
                      << dataName  << ", \t\t value "
                      << item->getData(i) << endl;
                 }
             }
             break;
        default:
             cout << " Item is not recognized " << endl;
      }
   }
   delete iter;
}
