/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/******************************************************************************
 * Filename       : LoadSampleTSQBICDL.cpp
 *
 * (NOTE: This loader loads the same data as LoadSampleTSQBIC.cpp does, except
 *  it loads data directly from file to have better performance and be able to
 *  handle large object files.)
 *
 * Description    : This sample loader using java class library's API to
 * load sample data into the Digital Library with parts to be indexed by 
 * Text Search and QBIC.  You should run it with the provided input data file 
 * "sampleQBIC.dat". The physical files	for this loader are packaged in
 * "samples.jar".  You should use command "jar -xvf samples.jar" to unpack it.  
 *
 * reminder: The "samples.jar" is in:
 *           x:\cmbroot\samples\java\dl	- for NT
 *           /usr/lpp/cmb/samples/java/dl - for AIX
 * 
 * To compile:
 *     in NT system, enter:
 *        nmake -f LoadSampleTSQBICDL.mak debug=1  (debug=1 is optional)
 *     The compiled .exe will be in \db_bin subdirectory) 
 *
 *     in AIX system, enter:
 *        make -f LoadSampleTSQBICDL.mak
 * 
 * To run it as default, this loader assumes following are already defined: 
 * 
 *    Index Class Name - "DLSAMPLE" (defined with the key fields in step 4)
 *    Text search server(search service name) - "TM"
 *    Text search index - "TMINDEX"
 *    QBIC DBName(database name) - "SAMPLEDB"
 *    QBIC Catalog - "SAMPLECAT"
 *    QBIC Host Alias of QBIC server - "qbicsrv"
 * 
 * Invoke Syntax: 
 *    (in NT system at  x:\cmbroot\samples\cpp\dl directory or
 *     in AIX system at /usr/lpp/cmb/samples/cpp\dl    enter following command: )
 *
 *    LoadSampleTSQBICDL <inputDataFile> <outputLog> <userid> <password> <libraryServer>
 *    example:  LoadSampleTSQBICDL sampleQBIC.dat qbic.log frnadmin password LIBSRVRN 
 *
 *    where:  
 *      <inputDataFile>: a input control file, see sampleQBIC.dat for example. 
 *      <outputLog>: the log file to store the process results, it can be used
 *                   for debug purpose, you can delete it later.  
 *      <userid>: the userid you use to logon the library server.
 *      <password>: the password to logon the library server.
 *      <libraryServer>: the library server name
 *
 * ### NOTE: ###
 *    This loader also allow you to invoke with "indexClassName" as the input 
 *    parameter.  Please be aware that your index class's attribute names and
 *    input data file should match. For example you invoke as:    
 *
 *    java LoadSampleTSQBICDL sampleQBIC.dat my.log frnadmin password LIBSRVRN MYINDEX
 *
 *    The index class name "DLSAMPLE" will be replaced with "MYINDEX". 
 * 
 ******************************************************************************
 * Following is the instruction to customize the loader to run your own data 
 ******************************************************************************
 * This loader can be modified to work with any index class defined with		
 * different attribute types(key fields).  Rules to follow:
 *
 *  1. The attribute name in the inputDataFile(ie. sampleQBIC.dat) should
 *     exactly match with the index class's attribute name(case sensitive). 
 *     for example: 
 *
 *     index class attribute names     in sampleQBIC.dat file
 *     ---------------------------   ----------------------
 *      DLSEARCH_Author              DLSEARCH_Author=Janet Hedges
 *      DLSEARCH_Publisher           DLSEARCH_Publisher=IBM
 *
 *     index class with your           your <inputDataFile> 
 *     defined attribute names         (ie. mine.dat)
 *     ---------------------------   ----------------------
 *      Department                   Department=U6M
 *      Manager                      Manager=John Smith
 *
 *  2. In your inputDataFile, the prefix for the parts to be added should be 
 *     "xdo", "smi", "html", "thmb" or "qbic".  The "xdo" is for a regular part,
 *     "smi" is for the part to be indexed by Text Search, "html" is for the 
 *     html part, "thmb" is for the thumbnail part and "qbic" is for the part
 *     to be indexed by the QBIC. They are case sensitive too.
 *
 *  3. The inputDataFile should use "end=" to represent the ending of an item.
 *
 *  4. Examples of index class's attributes and sampleQBIC.dat
 *
 *     index class attr. name     character length min - max
 *     ----------------------    ----------------------------
 *      DLSEARCH_Author            1 - 50
 *      DLSEARCH_Publisher         1 - 50
 *      DLSEARCH_DocType           1 - 32
 *      DLSEARCH_Date              1 - 50
 *      DLSEARCH_Title             1 - 100
 *      DLSEARCH_Subject           1 - 100
 *      DLSEARCH_Authors           1 - 100
 *      DLSEARCH_Editors           1 - 50
 *      DLSEARCH_Source            1 - 100
 *      DLSEARCH_Volume            1 - 20
 *      DLSEARCH_Issue             1 - 20
 *      DLSEARCH_Relation          1 - 64
 *
 *    (all above key field names are defined with key field type as
 *     "Variable character" and character type as "Extended alphanumeric", the
 *     character length min - max are shown as above)
 *
 *     the sampleQBIC.dat file: (shown as following - case sensitive)
 *    -----------------------------------------------------------
 *   ########################################################
 *   DLSEARCH_Author=Anonymous
 *   DLSEARCH_Authors=
 *   DLSEARCH_Subject=Contents
 *   DLSEARCH_Editors=
 *   DLSEARCH_Publisher=IBM
 *   DLSEARCH_Date=1994
 *   DLSEARCH_Source=Software Quarterly
 *   DLSEARCH_Volume=1
 *   DLSEARCH_Issue=1
 *   DLSEARCH_Title=Contents of Software Quarterly Volume 1 Issue 1
 *   html=../../java/dl/vol11/contents.htm
 *   thmb=../../java/dl/images/321smt.gif
 *   xdo=../../java/dl/images/sqcloud0.gif
 *   xdo=../../java/dl/images/sqclouda.gif
 *   xdo=../../java/dl/images/choice.gif
 *   qbic=../../java/dl/images/321sm.gif
 *   xdo=../../java/dl/images/sqhome.gif
 *   xdo=../../java/dl/images/tellsq.gif
 *   xdo=../../java/dl/images/getsq.gif
 *   xdo=../../java/dl/images/software.gif
 *   xdo=../../java/dl/vol11/contents.txt
 *   smi=../../java/dl/vol11/contents.dat
 *   end=end
 *   ########################################################
 *
 *   Note: about the prefix - smi, qbic, html, thmb
 *     smi - for the part to be indexed by the Text Search, ie. contents.dat
 *     qbic - for the part to be indexed by QBIC, ie. 321sm.gif
 *     html - for the html part, ie. contents.htm
 *     thmb - for the thumbnail part, ie. 321smt.gif
 *
 *  5. This loader requires the DL indexClass("DLSAMPLE"), Text
 *     search server("TM"), Text search index("TMINDEX"), QBIC 
 *     database("SAMPLEDB"), QBIC catalog("SAMPLECAT") and QBIC server("qbicsrv")
 *     have been defined first.  The "DLSAMPLE" has attribute names defined as 4.,
 *     and all the field types are "variable character".
 *
 *     If you have different index class name, different
 *     text search server(search service name) or text search index,
 *     different database name, catalog name and host alias name for QBIC,
 *     please modify the following variables:
 *
 *     indexClassName = "DLSAMPLE",    //a DL index class name
 *     searchEngineTS = "SM",          //Text search engine name - SM 
 *     searchServerTS = "TM",          //Text search server name    
 *     searchIndexTS = "TMINDEX",      //Text search index name
 *     searchInfo = "ENU",             //ENU for english, FRA for french... 
 *     searchEngineQBIC = "QBIC",      //QBIC engine name - QBIC
 *     QBICDataBase = "SAMPLEDB",      //QBIC DataBase name
 *     QBICCatalog = "SAMPLECAT",      //QBIC catalog name
 *     searchServerQBIC = "qbicsrv";   //QBIC server name
 *
 *  6. If you want to use this loader to load data without part to be indexed	
 *     by Text Search, please comment out the lines indicated by the 
 *     |**** no text search by comment out following line ****| &  
 *     |**** end of comment out for no text search ***********|
 *     In this case, you don't need to define the TM search index name. 
 *
 *     Also, if you want to use this loader to load data without part to be 
 *     indexed by QBIC, please comment out the lines indicated by the 
 *     |**** no image search by comment out following line ****| &  
 *     |**** end of comment out for no image search ***********|
 *     In this case, you don't need to define the QBIC search index name.
 *
 *  7. If physical files are in different directories then file path in
 *     sampleQBIC.dat should be modified.
 * 
 *****************************************************************************/  
#include <iostream.h>
#include <fstream.h>
#include <stdio.h>
#include <string.h>
#include <DKConstantDL.h>
#include <DKDatastoreDL.hpp>
#include <DKDatastoreTS.hpp>
#include <DKDatastoreQBIC.hpp>
#include <DKIndexInfoTS.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKParametricQuery.hpp>
#include <DKAny.hpp>
#include <DKResults.hpp>
#include <DKFolder.hpp>
#include <DKParts.hpp>
#include <DKDDO.hpp>
#include <dkXDO.hpp>
#include <DKAttrDefDL.hpp>
#include <DKBlobDL.hpp>
#include <dkResultSetCursor.hpp>
#include <DKEnvironment.hpp>
#include <DKSearchEngineInfoDL.hpp>
#define  MAXXDO   50
#define  MAXLEN   1024

 DKString 
      indexClassName = "DLSAMPLE",    //a DL index class name
      searchEngineTS = "SM",          //TextMiner engine name - SM 
      searchServerTS = "TM",          //TextMiner search server name    
      searchIndexTS = "TMINDEX",      //TextMiner search index name
      searchInfo = "ENU",             //ENU for english, FRA for french... 
      searchEngineQBIC = "QBIC",      //QBIC engine name - QBIC
      QBICDataBase = "SAMPLEDB",      //QBIC DataBase name
      QBICCatalog = "SAMPLECAT",      //QBIC catalog name
      searchServerQBIC = "QBICSRV";   //QBIC server name

 DKString  tocItemId = ""; //for DLSEARCH_Relation attr. value
 short docDataId = -1,
       itemDataId = -1,
       partsDataId = -1;
 int xdoFCount = 0;
 DKString xdoPathNames[MAXXDO];
 DKDatastoreDL *dsDL = 0;
 FILE *out; 
 FILE *in;
 DKString  searchIndexTM, searchIndexQBIC;
 int       smiPartId, notePartId, qbicPartId;  

 DKDDO* createDDO(DKString indexClassName);
 int    findContentClass(DKString fullFileName);
 void   addXDO(DKString fileName, int CClass, int partId, DKString itemId);
 DKString  prepareLine(DKString fullFileName, int partId);
 void   verifySetUp(DKString userId, DKString passWord);
 unsigned short  translateAttrTypeToDataItemType(short attrType);

 void main(int argc, char *argv[])
 {

    try
    {
      // Indicate this is a Console Application on Windows NT/95 for error
      // message code page conversion to OEM code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment::setSubSystem(DK_CM_SS_CONSOLE);
      DKDatastoreDL myDSDL;
      dsDL = &myDSDL;

      DKBoolean  smiSrv = FALSE,          //flag for calling wakeUpService 
                 qbicSrv = FALSE;         //flag for calling invokeSearchEngine


      DKString noteName = "notepart.txt", //for generated notepart file name
               itemId;                    //itemId for XDO add 
      
      smiPartId = 1;                      //partId for TM search indexed part
      notePartId = 2;                     //partId for notepart
      qbicPartId = 4;                     //partId for QBIC search indexed part
 
      int htmlPartId = 3,                 //partId for html part 
          qbicPartId = 4,                 //partId for QBIC search indexed part
          thmbPartId = 5,                 //partId for thumbnail part
          partId = 10;                    //partId for regular xdo part

      DKBoolean tocFlag = FALSE,          // flag to show item is a toc item
              xdoFlag = FALSE,            // flag to signal part is exist
              smiFlag = FALSE,            // flag to signal TM search part is exist 
              qbicFlag = FALSE,           // flag to signal QBIC search part is exist
              htmlFlag = FALSE,           // flag to signal html part is exist 
              thmbFlag = FALSE,           // flag to signal thumbnail part is exist
              reset = FALSE;              // reset tocItemId for new Toc
           
      DKString  smiName = "",             //filepathname of TM search part
                qbicName = "",            //filepathname of QBIC search part
                htmlName = "",            //filepathname of html part
                thmbName = "";            //filepathname of thumbnail part

      DKString tmpString;

      //*** TextMiner search service name-search index name for API call     
      searchIndexTM = searchServerTS + "-" + searchIndexTS;
      //*** QBIC DataBase name-Catalog name-server Name for API call      
      searchIndexQBIC = QBICDataBase + "-" + 
                        QBICCatalog + "-" + searchServerQBIC;
      int CClass; 
      short dataId, dataCount;
      short i;
      //-------------------------------------------------------------- 
      // Checking for input parameters
      //--------------------------------------------------------------   
      if (argc <6)
      {
       cout<<"Invoke Syntax: "<<endl;
       cout<<"  LoadSampleTSQBICDL <inputDataFile> <outputLogFile> ";
       cout<<"<userid> <password> <libraryServer>"<<endl;
       return;
      }
      
      // ### get input parameters
      DKString inputFile  = DKString(argv[1]);
      DKString outputFile = DKString(argv[2]);
      DKString userid     = DKString(argv[3]);
      DKString password   = DKString(argv[4]);
      DKString libSrv     = DKString(argv[5]);
      
      //----------------------------------------------------------------- 
      // support to run with user's own DL index class - at your own risk
      //----------------------------------------------------------------- 
      if (argc > 6)
      {
        indexClassName = DKString(argv[6]);       
      }
      out = fopen(outputFile, "w");
      if (out == NULL)
      {
        cout<<"can not open "<<outputFile<<" for write !"<<endl;
        return;
      } 

      //### for writing to log
      tmpString = "You enter: LoadSampleTSQBICDL ";
      tmpString = tmpString + inputFile;
      tmpString = tmpString + " ";
      tmpString = tmpString + outputFile; 
      tmpString = tmpString + " ";
      tmpString = tmpString + userid;
      tmpString = tmpString + " ";
      tmpString = tmpString + password; 
      tmpString = tmpString + " ";
      tmpString = tmpString + libSrv;
      if (argc > 6)
      {
        tmpString = tmpString + " "; 
        tmpString = tmpString + indexClassName;              
      }
      tmpString = tmpString + "\n";
      fputs((char*)tmpString, out);               

      in = fopen(inputFile, "r");
      if (in == NULL)
      {
         cout<<"can not open input data file: "<<inputFile<<endl;
         return;
      } 

      //-------------------------------------------------------------- 
      // verify the TextMiner "TMINDEX" and QBIC dataBase & catalog 
      // are exist
      //-------------------------------------------------------------- 
      verifySetUp(userid, password);
      
      //-------------------------------------------------------------- 
      // connecting to data store
      //-------------------------------------------------------------- 
      cout<<"Connecting datastore, please wait..."<<endl;
      dsDL->connect(libSrv, userid, password);
      cout<<"datastore connected"<<endl;
      cout<<"Loading "<<inputFile<<",  please wait.... "<<endl;
      DKAny anyOpt = DK_FALSE;
      dsDL->setOption(DK_DL_OPT_WAKEUPSRV, anyOpt);
      
      DKDDO *ddo = createDDO(indexClassName);

      //-------------------------------------------------------------- 
      // start to read input data file
      //-------------------------------------------------------------- 
      DKString line, name, value;
      char thisLine[MAXLEN];
      DKPid pid;
      DKAny anyValue;
      while (fgets(thisLine, MAXLEN, in) != NULL)
      {       
       int n, s;
       name = value = "";
       line = thisLine;
       line.strip();
       if (((char*)line)[0] == '#')
          continue;
       n = line.indexOf("=", 0);
       if (n == 0)
          continue;
       s = line.length();
       name = line.subString(0, n-1);
       value = line.subString(n+1, s-n);
       name.strip();
       value.strip(); 
       if (value == "")
          continue;

       //-------------------------------------
       // read and store the attribute value
       //-------------------------------------
       dataId = ddo->dataId(name);
       if (dataId != 0)
       {
         anyValue = value; 
         ddo->setData(dataId, anyValue);
         continue;
       }  
       if (tocItemId != "") 
         ddo->setData(itemDataId, tocItemId);
       //-------------------------------------
       // read and store the xdo FilePathName
       //-------------------------------------
       if ((name == "xdo") && (value != ""))
       {
         xdoFlag = TRUE;
         xdoPathNames[xdoFCount++] = value;         
       }
       else if ((name == "smi") && (value != ""))
       {
         smiName = value;
         smiFlag = TRUE;
         smiSrv = TRUE; 
       }
       else if ((name == "qbic") && (value != ""))
       {
         qbicName = value;
         qbicFlag = TRUE;
         qbicSrv = TRUE;
       }
       else if ((name == "html") && (value != ""))
       {
         htmlName = value;
         htmlFlag = TRUE;
       }
       else if ((name == "thmb") && (value != ""))
       {
         thmbName = value;
         thmbFlag = TRUE;
       }
       else
       if (name == "end")
       {
         //------------------------------------------
         // start to add DDO for an item(Document) 
         //------------------------------------------
         if (value == "toc")
             tocFlag = TRUE;
         if (value == "endToc")
             reset = TRUE;

         cout<<"start to create a DDO..."<<endl;
         //### for writing to log 
         dataCount = ddo->dataCount();   
         tmpString = " dataCount = ";
         tmpString = tmpString + DKString(dataCount);
         tmpString = tmpString + "\n"; 
         fputs((char*)tmpString, out);
         ddo->add();
         //------------------------------------------
         // start to add XDO with DDO's itemId
         //------------------------------------------
         itemId = (ddo->getPid()).getId();
         if (smiFlag)
            addXDO(smiName, DK_DL_CC_ASCII, smiPartId, itemId);
         if (htmlFlag)
            addXDO(htmlName, DK_DL_CC_HTML, htmlPartId, itemId);
         if (qbicFlag)
         {
           CClass = findContentClass(qbicName);
           addXDO(qbicName, CClass, qbicPartId, itemId); 
         }
         if (thmbFlag)
         {
           CClass = findContentClass(thmbName);
           addXDO(thmbName, CClass, thmbPartId, itemId);
         }
         if (xdoFlag)
         {
           partId = 10;
           for (i = 0; i < xdoFCount; i++)
           {
             CClass = findContentClass(xdoPathNames[i]); 
             addXDO(xdoPathNames[i], CClass, partId++, itemId);
           }
         }
         //-------------------------------------
         // create the notepart file
         //-------------------------------------
         FILE *note;
         note = fopen(noteName, "w");         
         if (smiFlag)
         {
           //### for writing to file and log
           tmpString = prepareLine(smiName, smiPartId);
           tmpString = tmpString + "\n";
           fputs((char*)tmpString, note); 
           fputs((char*)tmpString, out);  
    	 }
         if (htmlFlag)
         {
           //### for writing to file and log
           tmpString = prepareLine(htmlName, htmlPartId);
           tmpString = tmpString + "\n";
           fputs((char*)tmpString, note); 
           fputs((char*)tmpString, out);  
    	 }
         if (qbicFlag)
         {
           //### for writing to file and log
           tmpString = prepareLine(qbicName, qbicPartId);
           tmpString = tmpString + "\n";
           fputs((char*)tmpString, note); 
           fputs((char*)tmpString, out);  
    	 }
         if (thmbFlag)
         {
           //### for writing to file and log
           tmpString = prepareLine(thmbName, thmbPartId);
           tmpString = tmpString + "\n";
           fputs((char*)tmpString, note); 
           fputs((char*)tmpString, out);  
    	 }
         if (xdoFlag)
         {
           partId = 10;
           for (i = 0; i < xdoFCount; i++)
           {
             //### for writing to file and log
             tmpString = prepareLine(xdoPathNames[i], partId);
             tmpString = tmpString + "\n";
             fputs((char*)tmpString, note); 
             fputs((char*)tmpString, out); 
             partId = partId + 1;

           }
         }
         fclose(note);

         //### add notepart xdo
         addXDO(noteName, DK_DL_CC_ASCII, notePartId, itemId);
         
         //### for writing to log
         tmpString = "Add success with itemId = ";
         tmpString = tmpString + (ddo->getPid()).getId();
         tmpString = tmpString + "\n";
         fputs((char*)tmpString, out);
         fputs("===================================================\n", out);
 
         //### if toc, save created itemId for DLSEARCH_relation value of latter items
         if (tocFlag)
         {
           tocItemId = (ddo->getPid()).getId();
           tocFlag = FALSE;
         }
         if (reset)
         {
           tocItemId = "";
           reset = FALSE;
         }
         //### for writing to log
         tmpString = "DLSEARCH_relation = ";
         tmpString = tmpString + tocItemId;
         tmpString = tmpString + "\n";
         fputs((char*)tmpString, out); 

         //### reset the flags
         smiFlag = xdoFlag = qbicFlag = htmlFlag = thmbFlag = FALSE;
         xdoFCount = 0; 

         //### reset attrValue to null
         for (dataId = 1; dataId < dataCount; dataId++)
         {
           if (dataId != docDataId) 
               ddo->setNull(dataId);
         }
         // ### reset ddo itemid to blank
         pid = ddo->getPid();
         pid.setPrimaryId("");
         ddo->setPid(pid);     
       } //if is "end"
      }  //end of read: while
      fclose(in);
      if (smiSrv)
      {
/**** no text search by comment out following line ****/
      //### for writing to log
        DKString tmpA = "about to call wakeUpService(";
        tmpA = tmpA + searchEngineTS;
        tmpA = tmpA + ")...\n"; 
        DKString tmpB= "invokeSearchEngine(";
        tmpB = tmpB + searchEngineTS;
        tmpB = tmpB + "..) done. \n";

        cout<<tmpA;
        fputs((char*)tmpA, out);

        dsDL->invokeSearchEngine(searchEngineTS, searchIndexTM);


        cout<<tmpB;
        fputs((char*)tmpB, out);
/**** end of comment out for no text search ***********/
      } 
      if (qbicSrv)
      {
/**** no image search by comment out following line ****/
        anyOpt = DK_DL_SRCHINDEX_ONLINE;
        dsDL->setOption(DK_DL_OPT_SEARCHINDEX, anyOpt);
        //### for writing to log
        DKString tmpA2 = "about to call invokeSearchEngine(";
        tmpA2 = tmpA2 + searchEngineQBIC;
        tmpA2 = tmpA2 + ")...\n"; 
        DKString tmpB2= "invokeSearchEngine(";
        tmpB2 = tmpB2 + searchEngineQBIC;
        tmpB2 = tmpB2 + ") done. \n";

        cout<<tmpA2;
        fputs((char*)tmpA2, out);

        dsDL->invokeSearchEngine(searchEngineQBIC, searchIndexQBIC);

        cout<<tmpB2;
        fputs((char*)tmpB2, out);
/**** end of comment out for no image search ***********/
      }

      fclose(out);
      //delete ddo;
      cout<<"Finish loading "<< inputFile <<endl;
      dsDL->disconnect();
      cout<<"datastore disconnected. Done...."<<endl;
      delete dsDL;

    } // end of try
    catch(DKException &exc)
    {        
      if(dsDL) delete dsDL;
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
   }   
 }

 //**********************************************************
 // createDDO - create a ddo to be reuse for each document
 //**********************************************************
 DKDDO*  createDDO(DKString indexClassName)
 {
     // setup ddo
     DKPid pid;
     pid.setObjectType(indexClassName);        
     DKDDO *ddo = new DKDDO(dsDL, pid);
     ddo->addProperty(DK_CM_PROPERTY_ITEM_TYPE, DK_CM_DOCUMENT);
  
     // get attribute collection
     DKSequentialCollection  *collection = 0;
     dkIterator *iterator = 0;
     DKAttrDefDL *attrDef = 0; 
     DKAny *pA = 0;
     collection = (DKSequentialCollection*)((dkCollection*)
                   dsDL->listEntityAttrs(indexClassName));
     iterator = collection->createIterator();
     int i = 0;
     short dataId;
     DKString attrName,
              html = "html";
     //-------------------------------------------------------------- 
     // obtains data for setting up attrbuteList for ddo
     //-------------------------------------------------------------- 
     unsigned short attrType = 0;
     while (iterator->more() == TRUE)
     {
        pA = iterator->next();
        attrDef = (DKAttrDefDL*) pA->value();
        attrName = attrDef->getName();
        dataId = ddo->addData(attrName);
        ddo->addDataProperty(dataId, DK_CM_PROPERTY_ATTRIBUTE_ID,
                             attrDef->getId());
        attrType = attrDef->getType();
        attrType = translateAttrTypeToDataItemType(attrType);
        ddo->addDataProperty(dataId, DK_CM_PROPERTY_TYPE,
                             attrType);
        ddo->addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE,
                             attrDef->isNullable());
        ddo->addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT,
                               DK_CM_ATTRIBUTE_READWRITE);
        if (attrName == "DLSEARCH_DocType")
        { 
           docDataId = dataId;
           DKAny a = html;      
           ddo->setData(dataId, a);
        }
        else if (attrName == "DLSEARCH_Relation")
        {
           itemDataId = dataId;        
           ddo->setNull(dataId);
        }
        else 
           ddo->setNull(dataId);

        delete attrDef;
     }
     delete iterator;
     delete collection;

     partsDataId = ddo->addData(DKPARTS);     
     ddo->addDataProperty(partsDataId, DK_CM_PROPERTY_TYPE,
              DK_CM_DATAITEM_TYPE_COLLECTION_XDO);
     return ddo;
 }
 
 //**********************************************************
 // findContentClass - find content class from file extension
 //**********************************************************
 int  findContentClass(DKString fullFileName)
 {
     DKString suffix = "";   
     int n, pos, s, CClass;
     n = pos = s = 0;
     s = fullFileName.length();
     n = fullFileName.indexOf(".", 0);
     while (n != 0)
     {
       pos = n;
       n = fullFileName.indexOf(".", n+1);
     }
     if (pos > 0)
       suffix = fullFileName.subString(pos+1, s-pos);
     if (suffix != "")
     {
        if ((suffix == "htm") || 
	    (suffix == "html")) 
                  CClass = DK_DL_CC_HTML;
	    else if (suffix == "gif")     
                  CClass = DK_DL_CC_GIF;
	    else if (suffix == "jpg")
                  CClass = DK_DL_CC_JPG;
	    else if (suffix == "avi")
                  CClass = DK_DL_CC_AVI;
	    else if (suffix == "wav")
                  CClass = DK_DL_CC_WAV;
	    else if (suffix == "mid")
                  CClass = DK_DL_CC_MIDI;
        else if (suffix == "bmp")
                  CClass = DK_DL_CC_WINV3_BMP;
	    else  CClass = DK_DL_CC_ASCII;           
     }
     else
        CClass = DK_DL_CC_ASCII; 
     return CClass;
 } 
 //*****************************************************
 // addXDO - add an xdo
 //*****************************************************
 void  addXDO(DKString fileName, int CClass, int partId, DKString itemId)
 {
      DKBlobDL  *axdo = new DKBlobDL(dsDL);
      DKPidXDODL apid;
      apid.setPrimaryId(itemId);
      apid.setPartId(partId);
      axdo->setPidObject(&apid);
      axdo->setContentClass(CClass);
      if ((CClass == DK_DL_CC_ASCII) && (partId == notePartId))
      {
        axdo->setAffiliatedType(DK_DL_NOTE);
      }
      else
        axdo->setAffiliatedType(DK_DL_BASE);
/**** no text search by comment out following line ****/
      if (partId == smiPartId)
      {
        DKSearchEngineInfoDL aSrchEng;
        aSrchEng.setSearchEngine(searchEngineTS);
        aSrchEng.setSearchIndex(searchIndexTM);
        aSrchEng.setSearchInfo(searchInfo);
        axdo->setExtension("DKSearchEngineInfoDL", (dkExtension*)&aSrchEng);
      }
/**** end of comment out for no text search ***********/
/**** no image search by comment out following line ****/
      if (partId == qbicPartId)
      {
        DKSearchEngineInfoDL aSrchEng;
        aSrchEng.setSearchEngine(searchEngineQBIC);
        aSrchEng.setSearchIndex(searchIndexQBIC);
        aSrchEng.setSearchInfo(searchInfo);
        axdo->setExtension("DKSearchEngineInfoDL", (dkExtension*)&aSrchEng);
      }
/**** end of comment out for no image search ***********/

      //add from buffer
      axdo->setContentFromClientFile(fileName);
      axdo->add();        


      //### for writing to log
      DKString tmpStr = "add xdo part: ";
      tmpStr = tmpStr + fileName;
      tmpStr = tmpStr + "\n";
      fputs((char*)tmpStr, out);
      delete axdo;
 } 

 //**********************************************************
 // prepareLine - prepare a line record for the notepart file
 //**********************************************************
 DKString  prepareLine(DKString fullFileName, int partId)
 {
     DKString suffix = "";   
     int m, n, mpos, npos, s;
     m = n = mpos = npos = s = -1;
     s = fullFileName.length();
     m = fullFileName.indexOf('\\');
     while (m != 0)
     {
       mpos = m;
       m = fullFileName.indexOf('\\', m+1);
     }
     n = fullFileName.indexOf('/');
     while (n != 0)
     {
       npos = n;
       n = fullFileName.indexOf('/', n+1);
     }
     if ((npos == -1) && (mpos == -1))
       suffix = fullFileName + ":" + DKString(partId);
     else if (mpos == -1)
          suffix = fullFileName.subString(npos+1, s-npos) + 
                 ":" + DKString(partId);
     else 
          suffix = fullFileName.subString(mpos+1, s-mpos) + 
                 ":" + DKString(partId);
     return suffix; 
 }
 //**********************************************************
 // verifySetUp - verify the TextMiner index "TMINDEX" and
 // the QBIC Database and QBIC catalog are defined
 //**********************************************************
 void  verifySetUp(DKString userId, DKString passWord)
 {
     cout<<"verifying: "<<searchIndexTS<<endl;
     DKDatastoreTS dsTS;
     try {
         DKIndexInfoTS *pIndxInfo = 0;
         cout<<"connecting to datastoreTS... "<<endl;
         dsTS.connect(searchServerTS, "", ""); 
         cout<<"datastoreTS connected. " <<endl;
         pIndxInfo = dsTS.getIndexInformation(searchIndexTS);
         cout<<"Index Name =" <<pIndxInfo->getIndexName()<<endl;
         if (pIndxInfo != 0)
            delete pIndxInfo;
         dsTS.disconnect();
         cout<<"datastoreTS disconnected."<<endl<<endl;
     }
      catch(DKException &exc)
      {        
         cout << "Error id " << exc.errorId() << endl;
         for(unsigned long i = 0; i< exc.textCount();i++)
         {
            cout << "Error text: " << exc.text(i) << endl;
         }
         for (unsigned long g=0; g< exc.locationCount();g++)
         {
            const DKExceptionLocation* p = exc.locationAtIndex(g);
            cout << "Filename: " << p->fileName() << endl;
            cout << "Function: " << p->functionName() << endl;
            cout << "LineNumber: " << p->lineNumber() << endl;
         }
         cout << "Exception Class Name: " << exc.name() << endl;
      }   

     cout<<"verifying QBIC database/catalog: " <<QBICDataBase<<" "<<QBICCatalog<<endl;
     DKDatastoreQBIC dsQBIC;
     try {
         cout<<"connecting to datastoreQBIC... "<<endl;
         dsQBIC.connect(searchServerQBIC, userId, passWord);
         cout<<"datastoreQBIC connected. "<<endl;
         cout<<"open catalog " << QBICCatalog <<endl;
         dsQBIC.openCatalog(QBICDataBase, QBICCatalog);
         cout<<"catalog opened successfully. "<<endl;
         dsQBIC.closeCatalog();
         dsQBIC.disconnect();
         cout<<"datastoreQBIC disconnected. "<<endl;
     }
      catch(DKException &exc)
      {        
         cout << "Error id " << exc.errorId() << endl;
         for(unsigned long i = 0; i< exc.textCount();i++)
         {
            cout << "Error text: " << exc.text(i) << endl;
         }
         for (unsigned long g=0; g< exc.locationCount();g++)
         {
            const DKExceptionLocation* p = exc.locationAtIndex(g);
            cout << "Filename: " << p->fileName() << endl;
            cout << "Function: " << p->functionName() << endl;
            cout << "LineNumber: " << p->lineNumber() << endl;
         }
         cout << "Exception Class Name: " << exc.name() << endl;
      }   

     return;
 }

  //**********************************************************
  // translateAttrTypeToDataItemType - change attribute type to
  //   ddo dataitem type 
  //**********************************************************
 unsigned short  translateAttrTypeToDataItemType(short attrType)
 {
   unsigned short rAttrType = DK_CM_DATAITEM_TYPE_UNDEFINED;
   switch(attrType)
   {
    case DK_CM_VSTRING:
    case DK_CM_FSTRING:
    case DK_CM_DECIMAL:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_STRING;
    }
    break;
    case DK_CM_SHORT:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_SHORT;
    }
    break;
    case DK_CM_LONG:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_LONG;
    }
    break;
    case DK_CM_DATE:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_DATE;
    }
    break;
    case DK_CM_TIME:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIME;
    }
    break;
    case DK_CM_TIMESTAMP:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIMESTAMP;
    }
    break;
   }
   return rAttrType;
 }
