/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
                         
//*****************************************************************************
//
// Filename : TCreateDelEntQBIC.CPP/MAK
// Description : Connect/create db,cat,feature/Disconnect to QBIC Datastore
//
//*****************************************************************************

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreQBIC.hpp>
#include <DKDatastoreDefQBIC.hpp>
#include <DKDatabaseDefQBIC.hpp>
#include <DKCatalogDefQBIC.hpp>
#include <DKFeatureDefQBIC.hpp>
#include <DKException.hpp> 
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
    cout << "begin..." << endl;
    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 6)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TCreateDelEntQBIC <qbicSrv> <userid> <pw> <qbicdb> <qbiccat>" << endl;
     return;
    }
    // ### get input parameters
    DKString qbicSrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
    DKString qbicdb = DKString(argv[4]);
    DKString qbiccat = DKString(argv[5]);
    DKDatastoreQBIC dsQBIC; // must be outside try/catch block
    try {
    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    DKString str; 
    DKDatastoreDefQBIC* dsDef = 0;
    DKDatabaseDefQBIC* db = 0;
    DKCatalogDefQBIC* cat = 0;
    DKFeatureDefQBIC* feat = 0;
    cout << "Datastore QBIC created" << endl;
    cout << "connecting to datastore" << endl;
    dsQBIC.connect(qbicSrv, userid, pw);
    cout << "datastore connected" << endl;
    if (dsQBIC.isConnected())
    {
     cout << "isConnected() returns = TRUE" << endl;
    }
    else
      cout << "isConnected() returns FALSE" << endl;

    cout << "db name " << qbicdb << " cat name " << qbiccat << endl;
    dsDef = (DKDatastoreDefQBIC*)dsQBIC.datastoreDef();
    cout << "create database " << endl;
    db = new DKDatabaseDefQBIC(&dsQBIC);
    db->setName(qbicdb);
    cout << "add database " << qbicdb << endl;
    dsDef->add(db);
    cout << "create catalog " << endl;
    cat = new DKCatalogDefQBIC(&dsQBIC);
    cat->setName(qbiccat);
    cout << "add catalog " << qbiccat << " to database " << qbicdb << endl;
    db->add(cat);
    cout << "create feature " << endl;
    feat = new DKFeatureDefQBIC(&dsQBIC);
    feat->setName(DK_QBIC_FEATURE_QBCOLORFEATURE);
     
    cout << "add feature QbColorFeatureClass to catalog " << qbiccat << \
       " - database " << qbicdb << endl;

    cat->add(feat);

    cout << "get the db " << qbicdb << endl;
   
    db = (DKDatabaseDefQBIC*)dsDef->getEntity(qbicdb);

    cout << "get the cat " << qbiccat << endl;
 
    cat = (DKCatalogDefQBIC*)db->getSubEntity(qbiccat);

    cout << "get the feature QbColorFeatureClass" << endl;

    feat = (DKFeatureDefQBIC*)cat->getAttr(DK_QBIC_FEATURE_QBCOLORFEATURE);

    cout << "delete the feature QbColorFeatureClass" << endl;

    feat->del();
   
    cout << "delete the catalog" << endl;

    cat->del();

    cout << "delete the database" << endl;

    db->del();

    dsQBIC.disconnect();
    cout << "datastore disconnected" << endl;
    }
 
    catch(DKException &exc)
    {
      cout << "Error id "     << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }

    cout << "done ..." << endl;
    
}

