/************************************************************************/
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/

/*
 * Filename       : TIndexingTS.cpp
 * Description    : Start the indexing process for a text search index
 *
 */


#include <iostream.h>
#include <stdio.h>                
#include <DKConstantDL.h>
#include <DKDatastoreTS.hpp>
#include <DKDatastoreDefTS.hpp>
#include <DKDatastoreAdminTS.hpp>
#include <DKIndexFuncStatusTS.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 3)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TIndexingTS <srchSrv> <srchIndex>" << endl;
     return;
    }
    // ### get input parameters
    DKString srchSrv = DKString(argv[1]);
    DKString srchIndex = DKString(argv[2]);
    DKDatastoreTS dsTS; // must be outside try/catch block
   try {
      DKDatastoreDefTS* dsDef = 0;
      DKDatastoreAdminTS* dsAdmin = 0;
      DKIndexFuncStatusTS* pIndexFuncStatus = 0;
      cout << "Datastore TS created" << endl;
      cout << "connecting to datastore" << endl;
      dsTS.connect(srchSrv,"","");
      cout << "datastore connected " << srchSrv << endl;
      dsDef = (DKDatastoreDefTS*)dsTS.datastoreDef();
      dsAdmin = (DKDatastoreAdminTS*)dsDef->datastoreAdmin();
      cout << "start updating index " << srchIndex << endl;
      dsAdmin->startUpdateIndex(srchIndex);
      cout << "**** get index status ****" << endl;
      pIndexFuncStatus = dsAdmin->getIndexFunctionStatus(srchIndex,
                              DK_TS_INDEX_FUNCID_INDEX_DOCUMENTS);
      cout << "***** index status *****" << endl;
      cout << "*isCompleted " << pIndexFuncStatus->isCompleted() << endl; 
      cout << "*getEnabledId " << pIndexFuncStatus->getEnabledId() << endl; 
      cout << "*getReasonCode " << pIndexFuncStatus->getReasonCode() << endl; 
      cout << "*getFuncStopped " << pIndexFuncStatus->getFunctionStopped() << endl; 
      cout << "*getStartedLast " << pIndexFuncStatus->getStartedLast() << endl; 
      cout << "*getEndedLast " << pIndexFuncStatus->getEndedLast() << endl; 
      cout << "*getStartedExecution " << pIndexFuncStatus->getStartedExecution() << endl; 
      cout << "*getScheduledDocs " << pIndexFuncStatus->getScheduledDocs() << endl; 
      cout << "*getDocsInPrimaryIndex " << pIndexFuncStatus->getDocsInPrimaryIndex() << endl; 
      cout << "*getDocsInSecondaryIndex " << pIndexFuncStatus->getDocsInSecondaryIndex() << endl; 
      cout << "*getDocMessages " << pIndexFuncStatus->getDocMessages() << endl; 
      if (pIndexFuncStatus->isCompleted() == TRUE)
      {
       cout << "***********indexing completed****************" << endl; 
      }
      if (pIndexFuncStatus->getReasonCode() != 0)
      {
       dsAdmin->setIndexFunctionStatus(srchIndex,
    	 DK_TS_INDEX_FUNCID_INDEX_DOCUMENTS,DK_TS_INDEX_ACTID_RESET);
      } 
      delete pIndexFuncStatus;
      dsTS.disconnect();
      cout << "datastore disconnected" << endl;
   }
   catch(DKException &exc)
   {
      cout << "Error id" << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text:" << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;
}

