/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Attribute Definitions, if it does not exist, null is returned.  
       In this sample, if null is detected, no action will take place for that particular 
       Attribute definition because it was not defined.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKDatastoreICM.hpp>
#include <DKAttrDefICM.hpp>
#include <DKConstant.h>
#include "SConnectDisconnectICM.hpp"
#include "SAttributeDefinitionDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SAttributeDefinitionDeletionICM
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Delete Attribute Definitions, Insurance Model Attributes, and other Sample
 *                    Attributes.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Attribute Definitions Using it's del() Method.
 *                    Deleting Attribute Definitions Using the Datastore Definition's del() Method.
 *                    Deletes Attributes Created by SAttributeDefinitionCreationICM
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SAttributeDefinitionDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Attributes should already be defined.  If not, Sample
 *                    SAttributeDefinitionCreationICM should be run first to create the 
 *                    Attribute.  This sample will still complete, but only those that exist 
 *                    will be deleted.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SAttributeDefinitionDeletionICM.hpp
 *                    SAttributeDefinitionDeletionICM_launch.cpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SAttributeDefinitionDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                       << endl;
            cout << "  SAttributeDefinitionDeletionICM <database> <userName> <password>"      << endl;
            cout << "  *** Some parameters not specified, using defaults..."        << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "================================================"     << endl;
        cout << "IBM Information Integrator for Content      v" << ver << endl;
        cout << "Sample Program:  SAttributeDefinitionDeletionICM"     << endl;
        cout << "------------------------------------------------"     << endl;
        cout << " Database: " << database                              << endl;
        cout << " UserName: " << userName                              << endl;
        cout << "================================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //----------------------------------------------------------------
            // Retrieve the S_varchar Attribute Definition
            //----------------------------------------------------------------
            // For more information about the defined attributes, please consult
            // the SAttributeDefinitionCreationICM sample.
            cout << "Retrieving the Attribute Definition of 'S_varchar'..." << endl;

                // First, we need acces to the Datastore Definition object.
                DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKAttrDefICM* attrDef = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_varchar"); // Retrieve the definition.
                
            cout << "Retrieved the Attribute Definition of 'S_varchar'." << endl;

            //----------------------------------------------------------------
            // Delete the S_varchar Attribute Definition in the Datastore Definition.
            //----------------------------------------------------------------
            // The attribute definition can be deleted using its own del() method or
            // by using the Datastore Definition's del() funciton.
            cout << "Deleting the Attribute Definition 'S_varchar' in Datastore Definition..." << endl;

                if(attrDef!=NULL){   // if it is not already deleted,
                    attrDef->del();  // Delete the attribute definition
                    delete(attrDef); // Free up Memory
                }

            cout << "Deleted the Attribute Definition 'S_varchar' in Datastore Definition." << endl;

            //----------------------------------------------------------------
            // Retrieve & Delete the S_blob Attribute Definition
            //----------------------------------------------------------------
            // This method will use the datastore definition to delete the attribute.
            cout << "Deleting the Attribute Definition 'S_blob'..." << endl;

                attrDef = (DKAttrDefICM *) dsDefICM->retrieveAttr("S_blob"); // Retrieve the definition.
                if(attrDef!=NULL){          // if it is not already deleted,
                    dsDefICM->del(attrDef); // Delete the attribute definition
                    delete(attrDef);        // Free up Memory
                }

            cout << "Deleted the Attribute Definition 'S_blob'." << endl;

            //----------------------------------------------------------------
            // Delete the rest of the attributes created in the creation sample
            //----------------------------------------------------------------
            // Delete all attributes created in the SAttributeDefinitionCreationICM
            // sample, including the Insurance Model.
            deleteAttrDef(dsICM,"S_char");
            deleteAttrDef(dsICM,"S_clob");
            deleteAttrDef(dsICM,"S_date");
            deleteAttrDef(dsICM,"S_decimal");
            deleteAttrDef(dsICM,"S_double");
            deleteAttrDef(dsICM,"S_integer");
            deleteAttrDef(dsICM,"S_short");
            deleteAttrDef(dsICM,"S_time");
            deleteAttrDef(dsICM,"S_timestamp");
            deleteAttrDef(dsICM,"S_firstName");
            deleteAttrDef(dsICM,"S_lastName");
            deleteAttrDef(dsICM,"S_street");
            deleteAttrDef(dsICM,"S_city");
            deleteAttrDef(dsICM,"S_state");
            deleteAttrDef(dsICM,"S_zipCode");
            deleteAttrDef(dsICM,"S_policyNumber");
            deleteAttrDef(dsICM,"S_vin");
            deleteAttrDef(dsICM,"S_reportNumber");
            deleteAttrDef(dsICM,"S_accidentDate");
            deleteAttrDef(dsICM,"S_licensePlate");
            deleteAttrDef(dsICM,"S_driverLicense");
            deleteAttrDef(dsICM,"S_claimNumber");
            deleteAttrDef(dsICM,"S_claimantFName");
            deleteAttrDef(dsICM,"S_claimantLName");
            deleteAttrDef(dsICM,"S_claimantLicns");
            deleteAttrDef(dsICM,"S_insuredFName");
            deleteAttrDef(dsICM,"S_insuredLName");
            deleteAttrDef(dsICM,"S_adjusterFName");
            deleteAttrDef(dsICM,"S_adjusterLName");
            deleteAttrDef(dsICM,"S_adjustmntDate");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete the specified Attribute Definition
    * @param dsICM     Connected DKDatastoreICM object.
    * @param attrName  Attribute Name for the Attribute Definition to Delete.
    **/
    void SAttributeDefinitionDeletionICM::deleteAttrDef(DKDatastoreICM * dsICM, char * attrName){
        cout << "Deleting the Attribute Definition '" << attrName << "'..." << endl;

            // First, we need acces to the Datastore Definition object.
            DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();  // Get the datastore definition object from the connected datastore.
            DKAttrDefICM* attrDef = (DKAttrDefICM *) dsDefICM->retrieveAttr(attrName); // Retrieve the definition.
            if(attrDef!=NULL){    // if it is not already deleted...
                attrDef->del();   // Delete the Attribute Definition
                delete(attrDef);  // Free up Memory
            }

        cout << "Deleted the Attribute Definition '" << attrName << "'." << endl;
    }
