/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Pre-requisit Informaiton:
    This sample pulls together all concepts learned so far, using the IBM COntent Management
    Data Modeling Concepts in a business model example.  The Insurance Model is defined and
    explained in SItemTypeCreationICM.
    
    Minimum Required Reading & Samples Run
        SConnectDisconnectICM                   SFolderICM
        SAttributeDefinitionCreationICM         SLinksICM
        SAttributeGroupDefinitionCreationICM    SResourceMgrDefCreationICM
        SReferenceAttrDefCreationICM            SResourceMgrSetDefaultICM
        SItemTypeCreationICM                    SSMSCollectionDefCreationICM
        SItemCreationICM                        SSMSCollectionDefSetDefaultICM
        SItemRetrievalICM                       SResourceItemCreaitonICM
        SItemUpdateICM                          SResourceItemRetrievalICM
        SItemDeletionICM

This sample shows how to simply retrieve each created in the creation sample.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKLobICM.hpp>
#include <DKTextICM.hpp>
#include <DKImageICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SItemRetrievalICM.hpp"
#include "SInsuranceModelItemCreationICM.hpp"
#include "SInsuranceModelItemRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SInsuranceModelItemRetrievalICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Putting everything together into a real-world example, the Sample 
 *                    Insurance Model.  Retrieves the Insurance Model Items.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Putting everything together into a real-world example
 *                    Retrieving Insurance Model's Auto Claims
 *                    Retrieving Insurance Model's Adjustment Reports
 *                    Retrieving Insurance Model's Police Reports
 *                    Retrieving Insurance Model's Insurance Policies
 *                    Retrieving Insurance Model's Auto Photos
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SInsuranceModelItemRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                         - SAttributeDefinitionCreationICM
 *                         - SAttributeGroupDefCreationICM
 *                         - SReferenceAttrDefCreationICM
 *                         - SItemTypeCreationICM
 *                    The Item Types must be set to use the specified default Resource Manager.
 *                    Plese run the following samples.
 *                         - SResourceMgrDefSetDefaultICM
 *                         - SSMSCollectionDefSetDefaultICM
 *                    User-Defined Link Types must be defined.  Please run the following:
 *                         - SLinkTypeDefinitionCreationICM
 *                    NOTE:  Sample SSampleModelBuildICM pulls together all of the above.  It 
 *                           must be edited first for the RM Confiuration information.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemRetrievalICM.hpp/cpp
 *                    SInsuranceModelItemCreationICM.hpp/cpp
 *                    SInsuranceModelItemRetrievalICM_launch.cpp
 *                    SInsuranceModelItemRetrievalICM.hpp
 *                    SInsuranceModelICM_AccidenReport_0120200130.tif
 *                    SInsuranceModelICM_AccidenReport_0506200110.tif
 *                    SInsuranceModelICM_AccidenReport_729200173.tif
 *                    SInsuranceModelICM_Adjustment_4-852369.tif
 *                    SInsuranceModelICM_Adjustment_5-654123.tif
 *                    SInsuranceModelICM_Adjustment_6-987654.tif
 *                    SInsuranceModelICM_Claim_4-852369.tif
 *                    SInsuranceModelICM_Claim_5-654123.tif
 *                    SInsuranceModelICM_Claim_6-987654.tif
 *                    SInsuranceModelICM_Photo_1ABC089.jpg
 *                    SInsuranceModelICM_Photo_4IBM134-1.tif
 *                    SInsuranceModelICM_Photo_4IBM134-2.tif
 *                    SInsuranceModelICM_Photo_6usa676-1.tif
 *                    SInsuranceModelICM_Photo_6usa676-2.tif
 *                    SInsuranceModelICM_Policy_10285265410.tif
 *                    SInsuranceModelICM_Policy_10315975308.tif
 *                    SInsuranceModelICM_Policy_10452136412.tif
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SInsuranceModelItemRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                             << endl;
            cout << "  SInsuranceModelItemRetrievalICM <database> <userName> <password>"   << endl;
            cout << "  *** Some parameters not specified, using defaults..."              << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "================================================"     << endl;
        cout << "IBM Information Integrator for Content      v" << ver << endl;
        cout << "Sample Program:  SInsuranceModelItemRetrievalICM"     << endl;
        cout << "------------------------------------------------"     << endl;
        cout << " Database: " << database                              << endl;
        cout << " UserName: " << userName                              << endl;
        cout << "================================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create Insurance Model Items
            //-------------------------------------------------------------
            // Uses Function provided in the creation sample.
            cout << "Creating Insurance Model Items..." << endl;

                dkCollection* ddoColl = SInsuranceModelItemCreationICM::createInsuranceModelItems(dsICM);  // Uses Function provided in the creation sample.
                
                dkIterator* iter = ddoColl->createIterator();  // Create iterator to go through the DDOs returned.

                DKLobICM* autoClaim1      = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* autoClaim2      = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* autoClaim3      = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo1a       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo1b       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo2        = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo3a       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKImageICM* photo3b       = (DKImageICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* adjusterReport1 = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* adjusterReport2 = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* adjusterReport3 = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policeReport1   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policeReport2   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policeReport3   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policeReport4   = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policy1         = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policy2         = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();
                DKLobICM* policy3         = (DKLobICM*)(DKDDO*)(dkDataObjectBase*) iter->next()->value();

            cout << "Created Insurance Model Items." << endl;

            //-------------------------------------------------------------
            // Reacreate Blank DDOs using Pid Stirngs
            //-------------------------------------------------------------
            cout << "Recreating Blank DDOs uisng Pid Strings..." << endl;

                DKString autoClaim1PidStr      = ((DKPidICM*)autoClaim1->getPidObject())->pidString();
                DKString autoClaim2PidStr      = ((DKPidICM*)autoClaim2->getPidObject())->pidString();
                DKString autoClaim3PidStr      = ((DKPidICM*)autoClaim3->getPidObject())->pidString();
                DKString photo1aPidStr         = ((DKPidICM*)photo1a->getPidObject())->pidString();
                DKString photo1bPidStr         = ((DKPidICM*)photo1b->getPidObject())->pidString();
                DKString photo2PidStr          = ((DKPidICM*)photo2->getPidObject())->pidString();
                DKString photo3aPidStr         = ((DKPidICM*)photo3a->getPidObject())->pidString();
                DKString photo3bPidStr         = ((DKPidICM*)photo3b->getPidObject())->pidString();
                DKString adjusterReport1PidStr = ((DKPidICM*)adjusterReport1->getPidObject())->pidString();
                DKString adjusterReport2PidStr = ((DKPidICM*)adjusterReport2->getPidObject())->pidString();
                DKString adjusterReport3PidStr = ((DKPidICM*)adjusterReport3->getPidObject())->pidString();
                DKString policeReport1PidStr   = ((DKPidICM*)policeReport1->getPidObject())->pidString();
                DKString policeReport2PidStr   = ((DKPidICM*)policeReport2->getPidObject())->pidString();
                DKString policeReport3PidStr   = ((DKPidICM*)policeReport3->getPidObject())->pidString();
                DKString policeReport4PidStr   = ((DKPidICM*)policeReport4->getPidObject())->pidString();
                DKString policy1PidStr         = ((DKPidICM*)policy1->getPidObject())->pidString();
                DKString policy2PidStr         = ((DKPidICM*)policy2->getPidObject())->pidString();
                DKString policy3PidStr         = ((DKPidICM*)policy3->getPidObject())->pidString();

                // Clean up Memory
                SInsuranceModelItemCreationICM::deleteMemoryObjectsReturnedByCreateInsuranceModelItems(ddoColl);

                autoClaim1      = (DKLobICM*) dsICM->createDDO(autoClaim1PidStr);
                autoClaim2      = (DKLobICM*) dsICM->createDDO(autoClaim2PidStr);
                autoClaim3      = (DKLobICM*) dsICM->createDDO(autoClaim3PidStr);
                photo1a         = (DKImageICM*) dsICM->createDDO(photo1aPidStr);
                photo1b         = (DKImageICM*) dsICM->createDDO(photo1bPidStr);
                photo2          = (DKImageICM*) dsICM->createDDO(photo2PidStr);
                photo3a         = (DKImageICM*) dsICM->createDDO(photo3aPidStr);
                photo3b         = (DKImageICM*) dsICM->createDDO(photo3bPidStr);
                adjusterReport1 = (DKLobICM*) dsICM->createDDO(adjusterReport1PidStr);
                adjusterReport2 = (DKLobICM*) dsICM->createDDO(adjusterReport2PidStr);
                adjusterReport3 = (DKLobICM*) dsICM->createDDO(adjusterReport3PidStr);
                policeReport1   = (DKLobICM*) dsICM->createDDO(policeReport1PidStr);
                policeReport2   = (DKLobICM*) dsICM->createDDO(policeReport2PidStr);
                policeReport3   = (DKLobICM*) dsICM->createDDO(policeReport3PidStr);
                policeReport4   = (DKLobICM*) dsICM->createDDO(policeReport4PidStr);
                policy1         = (DKLobICM*) dsICM->createDDO(policy1PidStr);
                policy2         = (DKLobICM*) dsICM->createDDO(policy2PidStr);
                policy3         = (DKLobICM*) dsICM->createDDO(policy3PidStr);

            cout << "Recreated Blank DDOs uisng Pid Strings." << endl;

            //-------------------------------------------------------------
            // Retrieve One of Each
            //-------------------------------------------------------------
            // IMPORTANT NOTE: In order to greatly improve performance of 
            //                 retrieval, specify specific retrieval options
            //                 other than "Item Tree" to only retrieve what
            //                 is desired.  Performance will improve immensely.
            //                 The following will check for all possibilities
            //                 in a worst-case scenerio.
            cout << "Retrieving One of Each Type..." << endl;

                autoClaim2->retrieve(DK_CM_CONTENT_YES + DK_CM_CONTENT_ITEMTREE);
                photo2->retrieve(DK_CM_CONTENT_YES + DK_CM_CONTENT_ITEMTREE);
                adjusterReport2->retrieve(DK_CM_CONTENT_YES + DK_CM_CONTENT_ITEMTREE);
                policeReport2->retrieve(DK_CM_CONTENT_YES + DK_CM_CONTENT_ITEMTREE);
                policy2->retrieve(DK_CM_CONTENT_YES + DK_CM_CONTENT_ITEMTREE);

            cout << "Retrieved One of Each Type." << endl;

            //-------------------------------------------------------------
            // Print Each
            //-------------------------------------------------------------
            // Using the SItemRetrievalICM's printDDO funciton, this will
            // demonstrate what each DDO looks like now.
            cout << "Printing Items..." << endl;

                SItemRetrievalICM::printDDO(autoClaim2);
                SItemRetrievalICM::printDDO(photo2);
                SItemRetrievalICM::printDDO(adjusterReport2);
                SItemRetrievalICM::printDDO(policeReport2);
                SItemRetrievalICM::printDDO(policy2);

                // Clean up Memory
                delete(autoClaim1);
                delete(autoClaim2);
                delete(autoClaim3);
                delete(photo1a);
                delete(photo1b);
                delete(photo2);
                delete(photo3a);
                delete(photo3b);
                delete(adjusterReport1);
                delete(adjusterReport2);
                delete(adjusterReport3);
                delete(policeReport1);
                delete(policeReport2);
                delete(policeReport3);
                delete(policeReport4);
                delete(policy1);
                delete(policy2);
                delete(policy3);

            cout << "Printed Items." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.
