/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Delete Rules
    Delete rules are set at design-type / data model definition and apply to references
    and child component types / multivalue attribute definitions.
    
    Delete Rules may prevent deletion of an item with children or references.
    
    1. Please refer to SItemTypeCreationICM for complete information on delete rules 
       that are set for child component types.

    2. Please refer to SReferenceAttrDefCreationICM for complete information on delete
       rules that are set for reference attributes.

Deleting Child Components / Multivalue Attributes
    Remember that Child Components are multivalue attributes of the parent.  
    Multivalue attributes must be modified, added, and removed / deleted through
    the parent.  All operations must be made persistent through the root component's
    DKDDO.add() or DKDDO.update() functions.
    
    Deleting a child component is performed through simply removing the particular
    DKDDO object from the parent's DKChildCollection such as using the 
    DKChildCollection.removeElementAt(dkIterator) function and then calling
    DKDDO.update() on the root component.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKConstantICM.h>
#include <DKDatastoreICM.hpp>
#include <DKResults.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SItemTypeRetrievalICM.hpp"
#include "SItemCreationICM.hpp"
#include "SItemDeletionICM.hpp"

/************************************************************************************************
 *          FILENAME: SItemDeletionICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Deleting Items, Deleting All Items of an Item Type
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Deleting Items
 *                    Deleting All Items of an Item Type
 *                    Deletes all Items created by SItemCreationICM, SItemRetrievalICM,
 *                        & SItemUpdateICM.
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: SItemDeletionICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SItemTypeRetrievalICM.hpp/cpp
 *                    SItemCreationICM.hpp/cpp
 *                    SItemDeletionICM_launch.cpp
 *                    SItemDeletionICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SItemDeletionICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argc < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                  << endl;
            cout << "  SItemDeletionICM <database> <userName> <password>"      << endl;
            cout << "  *** Some parameters not specified, using defaults..."   << endl;
            cout << endl;
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argc > 1) database = argv[1];
            if (argc > 2) userName = argv[2];
            if (argc > 3) password = argv[3];
        }//end else
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==========================================="     << endl;
        cout << "IBM Information Integrator for Content v" << ver << endl;
        cout << "Sample Program:  SItemDeletionICM"               << endl;
        cout << "-------------------------------------------"     << endl;
        cout << " Database: " << database                         << endl;
        cout << " UserName: " << userName                         << endl;
        cout << "===========================================\n"   << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // Create an Sample Items
            //-------------------------------------------------------------
            // create 3 Items, one of Item Proprty type Document, one Folder, & one Item.
            // Will use the function for creating a simple item with children, with the
            // attribute data set from the SItemCreationICM Sample.
            cout << "Creating & Saving Sample Items..." << endl;

                DKDDO* ddoDocument = SItemCreationICM::createSimpleItemWithChildren(dsICM, DK_CM_DOCUMENT);
                DKDDO* ddoFolder   = SItemCreationICM::createSimpleItemWithChildren(dsICM, DK_CM_FOLDER);   
                DKDDO* ddoItem     = SItemCreationICM::createSimpleItemWithChildren(dsICM, DK_CM_ITEM);     

            cout << "Created & Saved Sample Items." << endl;

            //-------------------------------------------------------------
            // Deleting items from persistent datastore
            //-------------------------------------------------------------
            // Delete Rules may prevent deletion of an item with children or
            // references.
            // 
            // 1. Please refer to SItemTypeCreationICM for complete information
            //    on delete rules that are set for child component types.
            // 2. Please refer to SReferenceAttrDefCreationICM for complete
            //    information on delete rules that are set for reference 
            //    attributes.
            cout << "Deleting Items..." << endl;

                ddoDocument->del();
                ddoFolder->del();
                ddoItem->del();

                delete(ddoDocument);    // Free Memory
                delete(ddoFolder);
                delete(ddoItem);

            cout << "Deleted Items." << endl;

            //-------------------------------------------------------------
            // Delete all items of an item type
            //-------------------------------------------------------------
            // The wrapper provided at the bottom of this sample will delete
            // all items of any specified Item Type.  This will be used to 
            // clean up all sample items created by SItemCreationICM, 
            // SItemRetrievalICM, and SItemUpdateICM.  The wrapper uses
            // Query/Search, which will be explained in another sample.
            cout << "Deleting All Items of Item Type 'S_withChild'" << endl;

                // The only item type used with this section of samples was "S_withChild".
                deleteAllItems(dsICM,"S_withChild");
            
            cout << "Deleting All Items of Item Type 'S_withChild'" << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Delete all items of the specified Item Type.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  Item Type Name for which to delete all items of.
    **/
    void SItemDeletionICM::deleteAllItems(DKDatastoreICM * dsICM, char * itemTypeName){
        cout << "Deleting All Items of Type '" << itemTypeName << "'..." << endl;

            // First check to see if the Item Type exists, if not, simply return.
        if(!SItemTypeRetrievalICM::doesEntityExist(dsICM,itemTypeName))
                return;

            int count = 0; // used to count number of items deleted.
            
            // Build the query string to look like: "/<ItemType>"
            DKString queryString = DKString("/")+DKString(itemTypeName);

            // Evaluate Query
            DKNVPair* parms = new DKNVPair[2];
            parms[0].set(DK_CM_PARM_RETRIEVE,(long)DK_CM_CONTENT_IDONLY); // Only the Pid Information will be needed to delete.
            parms[1].set(DK_CM_PARM_END,     (long)0);
            DKResults* results = (DKResults*)(dkCollection*)dsICM->evaluate(queryString, DK_CM_XQPE_QL_TYPE, parms); // Evaluate Query
    
            // Process Results
            if (results != NULL) {
                dkIterator* iter = results->createIterator();
    
                while(iter->more()) { // While there are items left to delete
                    DKDDO* item = (DKDDO*) iter->next()->value();  // get the next item in the result list
                    if (item != NULL) { // if an item was retrieved, add it to the vector.
                        item->del();
                        count++;        // count the number deleted
                    }
                }
                delete(iter);     // Free Memory
            }//end if

            delete(results);  // Results will delete all contents within.
            delete[] parms;
            
        cout << "Deleted All '" << count << "' Items of Type '" << itemTypeName << "'." << endl;
    }//end deleteAllItems
