/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Please refer to the documentation in SResourceMgrDefCreationICM for concept documentation.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKDatastoreAdminICM.hpp>
#include <DKConfigurationMgmtICM.hpp>
#include <DKRMConfigurationMgmtICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceMgrDefRetrievalICM.hpp"

/************************************************************************************************
 *          FILENAME: SResourceMgrDefRetrievalICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Resource Manager Definition, Retrieval, Listing, & Printing.
 *                    Checking for RM Definitions.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Accessing Datastore Administration
 *                    Accessing Configuration Management
 *                    Accessing RM Configuration Management
 *                    List Resource Managers in System
 *                    Retrieving Resource Manager Definitions
 *                    Checking to See if RM Definition is Setup
 *                    Print Resource Manager Definition
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SResourceMgrDefRetrievalICM <Existing RM User-Defined Reference Name>
 *                                                <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.  This name should consist of all upper-case letters.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: SResourceMgrDefRetrievalICM RMDB
 *                    ex: SResourceMgrDefRetrievalICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM_launch.cpp
 *                    SResourceMgrDefRetrievalICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SResourceMgrDefRetrievalICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        char * rmUserDefinedReferenceName  = "NOT_SPECIFIED";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argc < 5) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            cout << "Usage: "                                                                     << endl;
            cout << "  SResourceMgrDefRetrievalICM <Existing RM User-Defined Reference Name>"     << endl;
            cout << "                              <database> <userName> <password>"              << endl;
            cout << endl;
            cout << "  NOTE: The specified reference name must be already defined, such as the"   << endl;
            cout << "        default 'RMDB' specified during install.  If a new RM definition"    << endl;
            cout << "        is to be configured, please refer to SResourceMgrDefCreationICM"     << endl;
            cout << "        Sample.  This name should consist of all upper-case letters."        << endl;
            cout << endl; 
            cout << "  NOTE: Defaults will be used if the database, username, & password are not" << endl;
            cout << "        specified.  All others are required."                                << endl;
            cout << endl;
            cout << "  ex: SResourceMgrDefRetrievalICM RMDB"                                      << endl;
            cout << "  ex: SResourceMgrDefRetrievalICM MY_RM icmnlsdb icmadmin password"          << endl;
        }
        if((argc < 2) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            cout << "======================================================" << endl;
            cout << "PROGRAM TERMINATED:  All RM Options Must Be Specified." << endl;
            cout << "======================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);
        } else if (argc != 5)
            cout << "  *** Some LS Database parameters not specified, using defaults..." << endl;
        
        // Store commandline arguments.
        if (argc > 1) rmUserDefinedReferenceName = argv[1];

        // Store optional commandline arguments.
        if (argc > 2)  database = argv[2];
        if (argc > 3)  userName = argv[3];
        if (argc > 4)  password = argv[4];
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "============================================"         << endl;
        cout << "IBM Information Integrator for Content  v" << ver     << endl;
        cout << "Sample Program:  SResourceMgrDefRetrievalICM"         << endl;
        cout << "--------------------------------------------"         << endl;
        cout << "           Database:  " << database                   << endl;
        cout << "           UserName:  " << userName                   << endl;
        cout << "  RM Reference Name:  " << rmUserDefinedReferenceName << endl;       
        cout << "============================================\n"       << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //------------------------------------------------------------------
            // List Resource Managers
            //------------------------------------------------------------------
            // Listing / Printing Resource Managers found in the system
            cout << "Listing / Printing Resource Managers found in the system..." << endl;

                printAllResourceManagers(dsICM); // Uses wrapper at bottom of sample.

            cout << "Listing / Printing Resource Managers found in the system..." << endl;

            //-------------------------------------------------------------
            // Checking to See if RM Definition Already Setup
            //-------------------------------------------------------------
            // This sample will skip over operations that have already been completed
            // in a previous run of this sample or other program.
            //
            // Demonstrating using Wrapper at bottom of sample.
            cout << "Checking to see if RM Definition already setup..." << endl;

                if(isResourceManagerSetup(dsICM, rmUserDefinedReferenceName))
                    cout << "RM Definition '" << rmUserDefinedReferenceName << "' already setup." << endl;
                else
                    cout << "RM Definition '" << rmUserDefinedReferenceName << "' has not yet been setup." << endl;

            cout << "Completed checking to see if RM Definition already setup." << endl;

            //-------------------------------------------------------------
            // Obtaining an RM Code for a RM Definition
            //-------------------------------------------------------------
            // Obtaining the RM Code that is used to specify properties of Items
            // and Item Types.
            cout << "Obtaining RM Code for RM Definition '" << rmUserDefinedReferenceName << "'..." << endl;
            
                short rmCode = getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.
            
            cout << "Obtained RM Code '" << rmCode << "' for RM Definition '" << rmUserDefinedReferenceName << "'." << endl;

            //------------------------------------------------------------------
            // Print Resource Manager Definition
            //------------------------------------------------------------------
            cout << "Printing Resource Manager Definition..." << endl;

                printResourceManagerDefinition(dsICM, rmUserDefinedReferenceName); // Uses wrapper at bottom of sample.

            cout << "Printed Resource Manager Definition." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // getResourceManagerIDCode
    //******************************************************************
   /**
    * Get the specified Resource Manager's ID Code for the specified user defined reference name.  This function should not be used to simply validate that the RM definition is setup.
    * @param dsICM  Connected DKDatastoreICM object.
    * @param rmUserDefinedReferenceName  The user-defined reference name to the one of the defined resource managers in the Library Server.  It should consist of all upper-case letters.
    * @return  Returns the ID Code for the defined resource manager.
    **/
    short SResourceMgrDefRetrievalICM::getResourceManagerIDCode(DKDatastoreICM* dsICM, char * rmUserDefinedReferenceName){
        cout << "Obtaining Resource Manager ID Code for User Defined Reference Name of '" << rmUserDefinedReferenceName << "'." << endl;

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        short rmCode;
        // Find the Resource Manager ID code for the specified resource manager.
        try {
            DKResourceMgrDefICM* rmDef = rmCfg->retrieveResourceMgr(rmUserDefinedReferenceName);  // Get the RM Definition for the specified Resource Manager.
            rmCode = rmDef->getRMCode();                       // Get the ID Code for the specified RM.
            cout << "RM " << rmUserDefinedReferenceName << "'s ID Code Retrieved: " << rmCode << endl;

            delete(rmDef); // Free Memory
        }
        catch (DKException e){ // if an exception was thrown, it is because there are none defined in the system.
            // IMPORTANT: Please do not use try-catch exception logic for program logic.  Use DKRMConfigurationMgmtICM.listResourceMgrs() to search available Resource Managers.
            cout << "An error has occured, most likely because the specified reference, '" << rmUserDefinedReferenceName << "', to a resource manager is not found.  Please do not use try-catch exception logic for program logic.  Use DKRMConfigurationMgmtICM.listResourceMgrs() to search available Resource Managers." << endl;
            throw e;
        }
            
        return(rmCode);
    }

    //******************************************************************
    // isResourceManagerSetup
    //******************************************************************
   /**
    * Determines if the specified user defined reference name to a Resource Manager is setup.
    * @param dsICM  Connected DKDatastoreICM object.
    * @param rmUserDefinedReferenceName  The user-defined reference name to the one of the defined resource managers in the Library Server.  It should consist of all upper-case letters.
    * @return  Returns true if it is found, false if not.
    **/
    bool SResourceMgrDefRetrievalICM::isResourceManagerSetup(DKDatastoreICM* dsICM, char * rmUserDefinedReferenceName){
        cout << "Determining if Resource Manager referred to as '" << rmUserDefinedReferenceName << "' is Setup." << endl;

        bool found = false;  // Return value tracking if it was found.
        short rmCode = -1;

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Obtain a list of RM Definitions.
        dkCollection* RMDefinitions = rmCfg->listResourceMgrs(); // Obtain the RM Definitions defined in the system.
        dkIterator* iter = RMDefinitions->createIterator(); // create an iterator to search for the specified name.

        while(iter->more()){ // while there are more RM Definition objects 
            DKResourceMgrDefICM* rmDef = (DKResourceMgrDefICM*)iter->next()->value(); // Move pointer to next element & return that object.
            if(!found){ // Only continue checking if not found.  Otherwise continue deleting all in list for memory cleanup.
                if(strcmp(rmDef->getName(),rmUserDefinedReferenceName)==0){ // Check to see if names match
                    found = true;                // mark it as found
                    rmCode = rmDef->getRMCode(); // Get the RM Code to Print at End of Function
                }
            }
            delete(rmDef);
        }
        delete(iter);
        delete(RMDefinitions);

        if(found) // Report Status
            cout << "It is setup.  Found " << rmUserDefinedReferenceName << " with ID Code '" << rmCode << "'." << endl;
        else
            cout << "It is not setup.  Did not find an ID Code for '" << rmUserDefinedReferenceName << "'." << endl;

        return(found);
    }

    //******************************************************************
    // printAllResourceManagers
    //******************************************************************
   /**
    * Prints / Lists all Resource Managers defined in the system.
    * @param dsICM  Connected DKDatastoreICM object.
    **/
    void SResourceMgrDefRetrievalICM::printAllResourceManagers(DKDatastoreICM* dsICM){

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Obtain a list of RM Definitions.
        dkCollection* RMDefinitions = rmCfg->listResourceMgrs(); // Obtain the RM Definitions defined in the system.
        dkIterator* iter = RMDefinitions->createIterator();      // create an iterator to search for the specified name.

        cout << "Resource Managers Defined in System:  (" << RMDefinitions->cardinality() << ")" << endl;
        while(iter->more()){ // while there are more RM Definition objects & it has not been found, continue searching
            DKResourceMgrDefICM* rmDef = (DKResourceMgrDefICM*)iter->next()->value(); // Move pointer to next element & return that object.
            cout << "     - " << rmDef->getName() << " (" << rmDef->getRMCode() << ")" << endl; // Print its name & code.
            delete(rmDef);    // Free Memory
        }
        delete(iter);         // Free Memory
        delete(RMDefinitions);// Free Memory
    }

    //******************************************************************
    // printResourceManagerDefinition
    //******************************************************************
   /**
    * Print the specified Resource Manager's Definition for the specified user defined reference name. 
    * @param dsICM  Connected DKDatastoreICM object.
    * @param rmUserDefinedReferenceName  The user-defined reference name to the one of the defined resource managers in the Library Server.  It should consist of all upper-case letters.
    **/
    void SResourceMgrDefRetrievalICM::printResourceManagerDefinition(DKDatastoreICM* dsICM, char * rmUserDefinedReferenceName){

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                   // Get the Datastore Definition Object
        DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();            // Get the Datqastore Administration
        DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement(); // Get the configuration management.
        DKRMConfigurationMgmtICM* rmCfg    = cfgMgmt->rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        DKResourceMgrDefICM* rmDef = rmCfg->retrieveResourceMgr(rmUserDefinedReferenceName);  // Get the RM Definition for the specified Resource Manager.

        cout << "-----------------------------------------------"   << endl;
        cout << "         Resource Manager Definition"              << endl;
        cout << "-----------------------------------------------"   << endl;
        cout << "     Reference Name:  " << rmDef->getName()        << endl;
        cout << "            ID Code:  " << rmDef->getRMCode()      << endl;
        cout << "       INET Address:  " << rmDef->getINETAddress() << endl;    
        switch(rmDef->getPlatform()){
            case RM_PLATFORM_WIN:
                cout << "           Platform:  Windows"             << endl;
                break;
            case RM_PLATFORM_AIX:
                cout << "           Platform:  AIX"                 << endl;
                break;
            case RM_PLATFORM_SUN:
                cout << "           Platform:  Sun"                 << endl;
                break;
            case RM_PLATFORM_390:
                cout << "           Platform:  OS/390"              << endl;
                break;
            default:
                cout << "           Platform:  Other (" << rmDef->getPlatform() << ")" << endl;
                break;
        }//end switch on Platform
        cout << "         RM User ID:  " << rmDef->getRMUserid()      << endl;
        cout << "         Created By:  " << rmDef->getUserid()        << endl;
        cout << "     Token Duration:  " << rmDef->getTokenDuration() << endl;

        // Print Access Type Definition Information
        dkCollection* accessTypeDefs = rmDef->getAccessType(); // Get a list of all access types available for this RM Definition.
        dkIterator* iter = accessTypeDefs->createIterator();   // Create an iterator to go through all access types.
        cout << "Access Types:" << endl;
        while(iter->more()){ // while there are still Access Types to print, continue
            DKAccessTypeDef* accessTypeDef = (DKAccessTypeDef*) iter->next()->value();  // Move pointer to next element & return that object.
            int accessType      = accessTypeDef->getAccessType();   // access type, HTTP, FTP, LSRM, etc.
            DKString accessData = accessTypeDef->getAccessData();
            short port          = accessTypeDef->getPortNumber();   // The port at which the Resource Manager is running.

            cout << "               Name:  " << accessTypeDef->getName()        << endl; 
            cout << "        Description:  " << accessTypeDef->getDescription() << endl; 
            cout << "               Port:  " << port << endl; // using value from DKAccessTypeDef retrieved above.

            switch(accessType){
                case RM_ACCESSTYPE_HTTP:
                    cout << "        Access Type:  HTTP"             << endl;
                    break;
                case RM_ACCESSTYPE_FTP:
                    cout << "        Access Type:  FTP"              << endl;
                    break;
                case RM_ACCESSTYPE_RTP:
                    cout << "        Access Type:  RTP"              << endl;
                    break;
                case RM_ACCESSTYPE_JDBC:
                    cout << "        Access Type:  JDBC"             << endl;
                    break;
                case RM_ACCESSTYPE_LSRM:
                    cout << "        Access Type:  LSRM"             << endl;
                    break;
                case RM_ACCESSTYPE_HTTPS:
                    cout << "        Access Type:  HTTPS"            << endl;
                    break;
                default:
                    cout << "        Access Type:  Other (" << accessType << ")" << endl;
                    break;
            }//end switch on accessType
            cout << endl;
            delete(accessTypeDef);      // Free Memory
        }//end while all Access Type Definitions
        delete(iter);                // Free Memory
        delete(accessTypeDefs);      // Free Memory
        delete rmDef;
        cout << "-----------------------------------------------" << endl;

    }//end printResourceManagerDefinition

    //******************************************************************
    // printAdminDomainDef
    //******************************************************************
   /**
    * Prints the specified Admin Domain Definition.
    * @param adminDomainDef  dkAdminDomainDef to print.
    **/
    void SResourceMgrDefRetrievalICM::printAdminDomainDef(dkAdminDomainDef* adminDomainDef){

        cout << "---------------------------------------------------"            << endl;
        cout << "              Admin Domain Information"                         << endl;
        cout << "---------------------------------------------------"            << endl;
        if(adminDomainDef==NULL)
            cout << "               Info:  None Specified"                       << endl;
        else{
            cout << "               Name:  " << adminDomainDef->getName()        << endl;
            cout << "        Description:  " << adminDomainDef->getDescription() << endl;
            cout << "                 ID:  " << adminDomainDef->getID()          << endl;  
            cout << "       NLS Language:  " << adminDomainDef->getNLSLanguage() << endl;
            // RM Names
            int i;
            long arrayLength = -1;  // Array size to be determined by API.
            DKString* names = adminDomainDef->getResourceMgrNames(arrayLength);
            if(names==NULL)
                cout << "           RM Names:  null" << endl;
            else{
                cout << "           RM Names:  -"    << endl;
                for(i=0; i<arrayLength; i++) // print all RM Names
                    cout << "                      " << names[i] << endl;
            }
            delete[] names;  // Free Memory
            // SMS Collection Names
            names = adminDomainDef->getSMSCollectionNames(arrayLength);
            if(names==NULL)
                cout << "    SMS Coll. Names:  null" << endl;
            else{
                cout << "    SMS Coll. Names:  -"    << endl;
               for(i=0; i<arrayLength; i++) // print all SMS Collection Names
                    cout << "                      " << names[i] << endl;
            }
            delete[] names;  // Free Memory
            // ACL Names
            names = adminDomainDef->getACLNames(arrayLength);
            if(names==NULL)
                cout << "          ACL Names:  null" << endl;
            else{
                cout << "          ACL Names:  -"    << endl;
                for(i=0; i<arrayLength; i++) // print all ACL Names
                    cout << "                      " << names[i] << endl;
            }
            delete[] names;  // Free Memory
            // User Names            
            names = adminDomainDef->getUserNames(arrayLength);
            if(names==NULL)
                cout << "         User Names:  null" << endl;
            else{
                cout << "         User Names:  -"    << endl;
                for(i=0; i<arrayLength; i++) // print all User Names
                    cout << "                      " << names[i] << endl;
            }
            delete[] names;  // Free Memory
            // User Group Names            
            names = adminDomainDef->getUserGroupNames(arrayLength);
            if(names==NULL)
                cout << "   User Group Names:  null" << endl;
            else{
                cout << "   User Group Names:  -"    << endl;
                for(i=0; i<arrayLength; i++) // print all User Group Names
                    cout << "                      " << names[i] << endl;
            }
            delete[] names;  // Free Memory
            // Priv. Set Names
            names = adminDomainDef->getPrivSetNames(arrayLength);
            if(names==NULL)
                cout << "    Priv. Set Names:  null" << endl;
            else{
                cout << "    Priv. Set Names:  -" << endl;
                for(i=0; i<arrayLength; i++) // print all Privilege Set Names
                    cout << "                      " << names[i] << endl;
            }
            delete[] names;  // Free Memory
        }//end adminDomainDef
    }
