/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Default Resource Manager & SMS Collection
    The Item Type Definition contains information that specifies the default Resource 
    Manager and default SMS Collection that Items of that Item Type are to be stored
    if not otherwise specified.
    
    Please refer to the corresponding wrapper function at the bottom of this sample, 
    and to the Item Type Creation sample, SItemTypeCreationICM for more information
    and examples of how to set and modify these properties.
    
System SMS Choice
    The System must be told whether to use the default SMS Collection defined for the 
    User or the default SMS Collection defined for the Item Type.  This sample will
    set default SMS Collection codes for Item Types.  If the choice is not set to use
    the Item Type default, the connection to the RM will fail since we are not specifying
    a default SMS Collection for the user.
    
    Please note that SMS "Choice" is not the same as SMS "Code".  
    
    SMS Choice  Description
    ----------  --------------------------------------------------------
        0       Use the default SMS Collection defined for the User.
        1       Use the default SMS Collection defined in the Item Type.

********************************************************************/

// Includes / Imports
#include <iostream.h>
#include <DKConstant.h>
#include <DKDatastoreICM.hpp>
#include <DKDatastoreDefICM.hpp>
#include <DKDatastoreAdminICM.hpp>
#include <DKConfigurationMgmtICM.hpp>
#include <DKRMConfigurationMgmtICM.hpp>
#include <DKItemTypeDefICM.hpp>
#include <DKItemTypeRelationDefICM.hpp>
#include "SConnectDisconnectICM.hpp"
#include "SResourceMgrDefRetrievalICM.hpp"
#include "SSMSCollectionDefRetrievalICM.hpp"
#include "SSMSCollectionDefSetDefaultICM.hpp"

/************************************************************************************************
 *          FILENAME: SSMSCollectionDefSetDefaultICM.cpp
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Updating Item Type Definitions for Default SMS Collection specifications.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Updating the Default SMS Collection for an Item Type
 *                    Updates All Resource Item Types in Samples
 *                    Setting SMS Choice
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  SSMSCollectionDefSetDefaultICM 
 *                                            <Existing RM User-Defined Reference Name>
 *                                            <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: SSMSCollectionDefSetDefaultICM RMDB
 *                    ex: SSMSCollectionDefSetDefaultICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    The SMS Collections uses here must already be defined.  Please refer
 *                    to the SSMSCollectionDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.hpp/cpp
 *                    SResourceMgrDefRetrievalICM.hpp/cpp
 *                    SSMSCollectionDefRetrievalICM.hpp/cpp
 *                    SSMSCollectionDefSetDefaultICM_launch.cpp
 *                    SSMSCollectionDefSetDefaultICM.hpp
 ************************************************************************************************/

    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argc   Argument Count / Number of Arguments Specified
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     * @return  Returns SConnectDisconnectICM_RC_SAMPLE_PASSED (0) if no sample program completed, SConnectDisconnectICM_RC_SAMPLE_FAILED (-1) if any errors / exceptions were caught.
     **/
    int SSMSCollectionDefSetDefaultICM::main(int argc, char *argv[]){

        // Defaults for connecting to the database.
        char * database = SConnectDisconnectICM_DEFAULT_DATABASE;
        char * userName = SConnectDisconnectICM_DEFAULT_USERNAME;
        char * password = SConnectDisconnectICM_DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        char * rmUserDefinedReferenceName  = "NOT_SPECIFIED";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argc < 5) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            cout << "Usage: " << endl;
            cout << "   SSMSCollectionDefSetDefaultICM"                                            << endl;
            cout << "                         <Existing RM User-Defined Reference Name>"           << endl;
            cout << "                         <database> <userName> <password>"                    << endl;
            cout << endl;
            cout << "  NOTE: The specified reference name must be already defined, such as the"    << endl;
            cout << "        default 'RMDB' specified during install.  If a new RM definition"     << endl;
            cout << "        is to be configured, please refer to SResourceMgrDefCreationICM"      << endl;
            cout << "        Sample."                                                              << endl;
            cout << endl;
            cout << "  NOTE: Defaults will be used if the database, username, & password are not"  << endl;
            cout << "        specified.  All others are required."                                 << endl;
            cout << endl;
            cout << "  ex: SSMSCollectionDefSetDefaultICM RMDB"                                    << endl;
            cout << "  ex: SSMSCollectionDefSetDefaultICM MY_RM icmnlsdb icmadmin password"        << endl;
        }
        if((argc < 2) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            cout << "======================================================" << endl;
            cout << "PROGRAM TERMINATED:  All RM Options Must Be Specified." << endl;
            cout << "======================================================" << endl;
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);
        } else if (argc != 5)
            cout << "  *** Some LS Database parameters not specified, using defaults..." << endl;
        
        // Store commandline arguments.
        if (argc > 1) rmUserDefinedReferenceName = argv[1];

        // Store optional commandline arguments.
        if (argc > 2)  database = argv[2];
        if (argc > 3)  userName = argv[3];
        if (argc > 4)  password = argv[4];
        char * ver = SConnectDisconnectICM_EIP_VERSION;

        cout << "==============================================="         << endl;
        cout << "IBM Information Integrator for Content     v" << ver     << endl;
        cout << "Sample Program:  SSMSCollectionDefSetDefaultICM"         << endl;
        cout << "-----------------------------------------------"         << endl;
        cout << "           Database:  " << database                      << endl;
        cout << "           UserName:  " << userName                      << endl;
        cout << "  RM Reference Name:  " << rmUserDefinedReferenceName    << endl;       
        cout << "===============================================\n"       << endl;

        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Connecting to datastore (Database '" << database << "', UserName '" << userName << "')..." << endl;

                DKDatastoreICM* dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM->connect(database,userName,password,""); // Connect to the datastore.

            cout << "Connected to datastore (Database '" << dsICM->datastoreName() << "', UserName '" << dsICM->userName() << "')." << endl;

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //-------------------------------------------------------------
            // Obtaining an RM Code for a RM Definition
            //-------------------------------------------------------------
            // Obtaining the RM Code that is used to specify properties of Items
            // and Item Types.
            cout << "Obtaining RM Code for RM Definition '" << rmUserDefinedReferenceName << "'..." << endl;
            
                if(SResourceMgrDefRetrievalICM::isResourceManagerSetup(dsICM, rmUserDefinedReferenceName)==false) // if it is not setup, stop here
                    throw DKException(DKString("Resource Manager Definition Named '")+DKString(rmUserDefinedReferenceName)+DKString("' Not Defined."));

                short rmCode = SResourceMgrDefRetrievalICM::getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.
            
            cout << "Obtained RM Code '" << rmCode << "' for RM Definition '" << rmUserDefinedReferenceName << "'." << endl;

            //------------------------------------------------------------------
            // Obtaining an SMS Colleciton Code for a SMS Collection Definition
            //------------------------------------------------------------------
            // Obtaining the SMS Collection Code that is used to specify 
            // properties of Items and Item Types.
            cout << "Obtaining ID Code for SMS Collection named '" << RM_SMS_COLLECTION_NONMEDIA << "'..." << endl;

                short smsCode = SSMSCollectionDefRetrievalICM::getSMSCollectionCodeForRM(dsICM, RM_SMS_COLLECTION_NONMEDIA, rmCode); // Uses wrapper at bottom of sample.
                cout << "Collection '" << RM_SMS_COLLECTION_NONMEDIA << "' has ID Code '" << rmCode << "'." << endl;
                
            cout << "Obtained ID Code for SMS Collection named '" << RM_SMS_COLLECTION_NONMEDIA << "'." << endl;

            //------------------------------------------------------------------
            // Set Default SMS Collection in an Item Type
            //------------------------------------------------------------------
            // Set the Default SMS Collection for Resource Item Types.
            cout << "Setting Default SMS Code for Sample Resource Item Types..." << endl;

                // For each Resource Item Type created in SItemTypeCreationICM, each will be
                // updated to use by default the specified SMS Collection.
                setDefaultSMSCollectionForItemType(dsICM, "S_autoClaim", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_adjustrReport", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_policeReport", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_insurePolicy", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_autoPhoto", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_docModel", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_lob", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_text", smsCode);
                setDefaultSMSCollectionForItemType(dsICM, "S_image", smsCode);

            cout << "Set Default SMS Code for all Sample Resource Item Types." << endl;

            //-------------------------------------------------------------
            // Set SMS Choice in System
            //-------------------------------------------------------------
            cout << "Setting System SMS Choice..." << endl;

                setSystemSMSChoice(dsICM,(short) 1);  // '1' means Item Type's default.
            
            cout << "Set System SMS Choice." << endl;

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            cout << "Disconnecting from datastore & destroying reference..." << endl;            

                dsICM->disconnect();
                delete(dsICM);

            cout << "Disconnected from datastore & destroying reference." << endl;
    
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            cout << "\n==========================================" << endl;
            cout << "Sample program completed."                    << endl;
            cout << "==========================================\n" << endl;
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException &exc){
            SConnectDisconnectICM::printException(exc);    // Print the exception to the screen
            return(SConnectDisconnectICM_RC_SAMPLE_FAILED);  // Report using return code that it failed
        }
        return(SConnectDisconnectICM_RC_SAMPLE_PASSED);    // If no exceptions were thrown, return a code to indicate passed.
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // setDefaultSMSCollectionForItemType
    //******************************************************************
   /**
    * Set the Default SMS Collection in which to store items of this Item Types.  Item Type
    * must already be defined.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  The Name of the Item Type to modify.
    * @param rmUserDefinedReferenceName  The Reference Name given to the Resource Manager that this SMS Collection belongs to.
    * @param rmSMSCollectionUserDefinedReferenceName   The Name of the SMS Collection to use by default.
    **/
    void SSMSCollectionDefSetDefaultICM::setDefaultSMSCollectionForItemType(DKDatastoreICM* dsICM, char* itemTypeName, char* rmUserDefinedReferenceName, char* rmSMSCollectionUserDefinedReferenceName){
        cout << "Setting the default to SMS Collection referred to as '" << rmSMSCollectionUserDefinedReferenceName << "'." << endl;
        
        // Make sure the RM & SMS Collection are setup.
        if(SResourceMgrDefRetrievalICM::isResourceManagerSetup(dsICM,rmUserDefinedReferenceName)==false) // Make sure that this RM is setup.
            throw DKException("Resource Manager not already setup.  RM must be first setup.");

        short rmCode = SResourceMgrDefRetrievalICM::getResourceManagerIDCode(dsICM,rmUserDefinedReferenceName); // get the RM Code.

        if(SSMSCollectionDefRetrievalICM::isSMSCollectionSetupForRM(dsICM,rmSMSCollectionUserDefinedReferenceName, rmCode)==false) // if it is not defined throw error
            throw DKException(DKString("SMS Collection, '")+DKString(rmSMSCollectionUserDefinedReferenceName)+DKString("' not setup."));

        short smsCode = SSMSCollectionDefRetrievalICM::getSMSCollectionCodeForRM(dsICM, rmSMSCollectionUserDefinedReferenceName,rmCode);  // Get the SMS Collection ID Code.

        setDefaultSMSCollectionForItemType(dsICM,itemTypeName,smsCode); // set the item type to use this default smsCode.
    }

   /**
    * Set the Default SMS Collection in which to store items of this Item Type.  The Item Type 
    * must already be defined.  For more information on updating Item Type Definitions, please
    * consult SItemTypeUpdateICM Sample.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  The Name of the Item Type to modify.
    * @param smsCode       The ID Code in the SMS collection to use.
    **/
    void SSMSCollectionDefSetDefaultICM::setDefaultSMSCollectionForItemType(DKDatastoreICM* dsICM, char* itemTypeName, short smsCode){
        cout << "Setting the default SMS Collection for Item Type Named '" << itemTypeName << "' to SMS Collection with ID Code '" << smsCode << "'." << endl;

            DKDatastoreDefICM* dsDefICM = (DKDatastoreDefICM*)dsICM->datastoreDef();                    // Get the Datastore Definition Object

            DKItemTypeDefICM* itemTypeDef = (DKItemTypeDefICM*) dsDefICM->retrieveEntity(itemTypeName); // retrieve the Item Type Definition
            if( itemTypeDef ){
              // If CM Doc Model Item Type, update Relations instead
              if(itemTypeDef->getClassification()==DK_ICM_ITEMTYPE_CLASS_DOC_MODEL){
                  cout << "Setting Default for Item Type Relations..." << endl;
                  dkCollection* coll = dsDefICM->retrieveItemTypeRelations(itemTypeDef->getItemTypeId());
                  dkIterator* iter = coll->createIterator();
                  while(iter->more()){
                      DKItemTypeRelationDefICM* itemTypeRel = (DKItemTypeRelationDefICM*) iter->next()->value();
                      itemTypeRel->setDefaultCollCode(smsCode);
                      // Item Type Relations also Require Prefetch Code, which can be set to Default CollCode.
                      itemTypeRel->setDefaultPrefetchCollCode(smsCode);
                      // Both RM & SMS must be set once one is specified.  Detect if RM has not been set, 0,
                      // and set it in that case to '1' as default.
                      if(itemTypeRel->getDefaultRMCode()==0)
                          itemTypeRel->setDefaultRMCode((short)1);       // Set the default Resource Manager for Items of this Item Type to be stored to.

                      itemTypeRel->update();
                      delete(itemTypeRel);  // Free Memory
                  }
                  delete(iter);             // Free Memory
                  delete(coll);
              } else { // Othwerwise update item type directly
                  itemTypeDef->setDefaultCollCode(smsCode);     // Set the default SMS Colleciton to use for item of this Item Type.

                  // Both RM & SMS must be set once one is specified.  Detect if RM has not been set, 0,
                  // and set it in that case to '1' as default.
                  if(itemTypeDef->getDefaultRMCode()==0)
                      itemTypeDef->setDefaultRMCode((short)1);  // Set the default Resource Manager for Items of this Item Type to be stored to.

                  itemTypeDef->update();                        // Update the Item Type Definition.
              }
              delete(itemTypeDef);                          // Free Memory
            }else
              cout << "Undefined ItemType '" << itemTypeName <<"'"<<endl;

        cout << "ItemType '" << itemTypeName <<"' now set to default SMS Code '" << smsCode << "'." << endl;
    }

   /**
    * Set the System SMS Choice, determining which default SMS Collection code it will use, the User's or Item Type's.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param smsChoice     Determines which default SMS Collection code to use, that of the user or that defined in the Item Type.  0 - User, 1 - Item Type.
    **/
    void SSMSCollectionDefSetDefaultICM::setSystemSMSChoice(DKDatastoreICM* dsICM, short smsChoice){
        if(smsChoice==0)
            cout << "Setting System SMS Choice to 'User' (0)..." << endl;
        else if(smsChoice==1)
            cout << "Setting System SMS Choice to 'Item Type' (1)..." << endl;
        else
            cout << "Setting System SMS Choice to 'Unknown' (" << smsChoice << ")..." << endl;

            // Obtain the RM Configuration Management Object
            DKDatastoreDefICM*        dsDefICM = (DKDatastoreDefICM*) dsICM->datastoreDef();                     // Get the Datastore Definition Object
            DKDatastoreAdminICM*      dsAdmin  = (DKDatastoreAdminICM*) dsDefICM->datastoreAdmin();             // Get the Datqastore Administration
            DKConfigurationMgmtICM*   cfgMgmt  = (DKConfigurationMgmtICM*) dsAdmin->configurationManagement();  // Get the configuration management.
            DKLSConfigurationMgmtICM* lsCfgMgmt= (DKLSConfigurationMgmtICM*)cfgMgmt->lsConfigurationMgmt();

            // Set the SMS Choice for the System.
            DKLSCfgDefICM* configParms = lsCfgMgmt->retrieveConfigParameters();
            configParms->setSMSChoice(smsChoice);
            lsCfgMgmt->defineConfigParameters(configParms);        
            delete configParms;
            
        cout << "Set System SMS Choice to '" << smsChoice << "'." << endl;
    }
