//
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
//


// sample connect/execute/disconnect to DES
/*
 * Filename       : TCallbackExecuteDES.cpp
 * Description    : Connect to DES Datastore.
 *                  Create query string.
 *                  Specify to retrieve folder and documents.
 *                  Specify to retrieve a max result of 2 items.
 *                  execute the query with callback.
 *                  Display documents found and their parts and attributes,
 *                   if folders found display them and their documents/
 *                   folders.
 *                  Disconnect from DES Datastore.
 */

#include <iostream.h>
#include <stdio.h>                
#include <DKConstant.h>
#include <DKDatastoreDES.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <dkResultSetCursor.hpp>
#include <DKDDO.hpp>
#include <DKAny.hpp>
#include <DKDate.hpp>
#include <DKPid.hpp>
#include <dkCollection.hpp>
#include <DKBlobDES.hpp>
#include <DKPidXDODES.hpp>
#include <DKEnvironment.hpp>
#include "TCallbackDES.hpp"



void main(int argc, char *argv[])
{
    cout << "begin..." << endl;

    // Indicate this is a Console Application on Windows NT/95 for error
    // message code page conversion to OEM code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_SS_WINDOWS is the default.
    DKEnvironment::setSubSystem(DK_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 5)
    {
     cout<<"Usage: "<<endl;
     cout<< argv[0] << "  <libsrv> <userid> <pw> <PORT=80;DESAPPID=appid;DESAPPPW=apppass[;DESCFGPATH=<des cfg file>;DESLOCALE=<locale>;]>" << endl;
     return;
    }
    // ### get input parameters
    DKString libsrv = DKString(argv[1]);
    DKString userid = DKString(argv[2]);
    DKString pw = DKString(argv[3]);
	DKString str = argv[4];

    DKDatastoreDES dsDES;

    try {
    dkResultSetCursor* pCur = 0;
	DKNVPair* parms = 0;
    DKException* exc1 = 0;
    cout << "Datastore DES created" << endl;
    cout << "connecting to datastore" << endl;
    dsDES.connect(libsrv,userid,pw,str);
    cout << "datastore connected " << libsrv << " userid - " << userid << endl;

	DKString cmd = "SEARCH=(DATABASE=(Notes 5 Help);";
	cmd += "COND=((IN \"Subject\" \"Notes\")));";
	cmd += "OPTION=(SEARCHABLE_FIELD=(Doc_Number); RETRIEVABLE_FIELD=(Indent);";
	cmd += "MAX_RESULTS=2;TIME_LIMIT=60);";

    cout << "query string " << cmd << endl;
    cout << "executing query" << endl;
	TCallbackDES cbkDES;
	dsDES.executeWithCallback(cmd, DK_DES_GQL_QL_TYPE, parms,
		(dkCallback*)&cbkDES);
    cout << "query executed" << endl;

	DKBoolean isDone = FALSE;

	while (isDone == FALSE)
	{
	        cout << "**** press <enter> to continue..."<< endl;
		getchar();
		isDone = cbkDES.isDone();
		cout << "isDone ??? " << isDone << endl;
	}

    exc1 = cbkDES.getException();
    if (exc1 != 0)
    {
       cout << "Error found " << exc1->text(0) << endl;
       delete exc1;
       cbkDES.resetException();
    }

     dsDES.disconnect();
     cout << "datastore disconnected" << endl;
    }
    catch(DKException &exc)
    {

      cout << "Error id " << exc.errorId() << endl;
      cout << "Exception id " << exc.exceptionId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
       cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
       const DKExceptionLocation* p = exc.locationAtIndex(g);
       cout << "Filename: " << p->fileName() << endl;
       cout << "Function: " << p->functionName() << endl;
       cout << "LineNumber: " << p->lineNumber() << endl;
      }
      cout << "Exception Class Name: " << exc.name() << endl;
    }
    cout << "done ..." << endl;
}
