/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/
/************************************************************************/
/*
 * Filename       : TDatastorePoolDES.cpp
 * Description    : test DKDatastorePool for datastore connection pool
 *
 */
#include <iostream.h>
#include <stdio.h>
#include <DKConstant.h>
#include <DKBlobDES.hpp>
#include <DKDatastoreDES.hpp>
#include <DKDatastorePool.hpp>
#include <DKDatastoreDefDES.hpp>
#include <DKSequentialCollection.hpp>
#include <DKFieldDefDES.hpp>
#include <DKServerDefDES.hpp>
#include <DKDatabaseDefDES.hpp>
#include <DKException.hpp>
#include <DKString.hpp>
#include <DKEnvironment.hpp>

void main(int argc, char *argv[])
{
   cout << "begin..." << endl;
   // Indicate this is a Console Application on Windows NT/95 for error
   // message code page conversion to OEM code page. If this was a Windows
   // GUI (Graphic User Interface) Application this would not be necessary
   // because DK_SS_WINDOWS is the default.
   DKEnvironment::setSubSystem(DK_SS_CONSOLE);

   DKString libsrv = "";         // libserver
   DKString userid = "";         // userid
   DKString pw = "";             // password
   DKString connect_string = ""; // optional
   DKString factoryDESLName = "cmbdesfac816d"; 

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //--------------------------------------------------------------   
    if (argc < 4)
    {
     cout<<"Usage: "<<endl;
     cout<<"  TDatastorePoolDES <tcp/ipAddress> <userid> <pw>";
     cout<<" <PORT=80;DESAPPID=appid;DESAPPPW=apppass>" << endl;
     return;
    }
    else
    {
    // ### get input parameters
      libsrv = DKString(argv[1]);
      userid = DKString(argv[2]);
      pw = DKString(argv[3]);
      if (argc > 4)
        connect_string = DKString(argv[3]);
    }

   try {
      DKDatastorePool* dsrDES = new DKDatastorePool("DES", factoryDESLName);
      dsrDES->setDatastoreName(libsrv);
      dsrDES->setMaxPoolSize(10);
      dsrDES->setMinPoolSize(3);
      //dsrDES->setTimeOut(1); 
      if (connect_string == "")
        connect_string = "PORT=80;DESAPPID=appid;DESAPPPW=apppw";
      dsrDES->setConnectString(connect_string);

      dsrDES->initConnections(userid, pw, 2);
      dsrDES->setTimeOut(1); 
      cout<<" #getMaxPoolSize =  " << dsrDES->getMaxPoolSize()<<endl;
      cout<<" #getMinPoolSize =  " << dsrDES->getMinPoolSize()<<endl;
      cout<<" #getTimeOut=  " << dsrDES->getTimeOut()<<endl;

      dkDatastore *dsDES1, *dsDES2, *dsDES3, *dsDES4;;
      dsDES1 = dsrDES->getConnection(userid, pw);  //new ds1
      cout<<" #getConnection, dsDES1(ds1) =  " << dsDES1 << endl;
      dsDES2 = dsrDES->getConnection(userid, pw);  // new ds2
      cout<<" #getConnection, dsDES2(ds2) =  " << dsDES2 << endl;

      DKDatastoreSession *dsIdDES1, *dsIdDES2, *dsIdDES3, *dsIdDES4;
      dsIdDES1 = dsrDES->getConnection(userid, pw, 0);        //new dsid1
      cout<<" #getConnection, dsIdDES1(dsid1)="<< dsIdDES1->getDatastore()
        <<", sessionId=" << dsIdDES1->getId()<<endl;
      dsIdDES2 = dsrDES->getConnection(userid, pw, 0);        //new dsid2
      cout<<" #getConnection, dsIdDES2(dsid2)="<<dsIdDES2->getDatastore()
        <<", sessionId=" << dsIdDES2->getId()<<endl;

      dsrDES->returnConnection(dsIdDES2);                     //dsid2 in _freeIdPool
      cout<<" #returnConnection, dsIdDES2(dsid2) =  " << 
        dsIdDES2->getDatastore() << ", sessionId=" << dsIdDES2->getId()<<endl;
      dsrDES->returnConnection(dsDES2);                       //ds2 in _freePool
      cout<<" #returnConnection, dsDES2(ds2) =  " << dsDES2 <<endl; 

      dsIdDES3 = dsrDES->getConnection(userid, pw, 1); //ds2 in _usedPool
      cout<<" #getConnection, dsIdDES3(ds2)=" <<
        dsIdDES3->getDatastore() << ", sessionId=" << dsIdDES3->getId()<<endl;
      dsIdDES4 = dsrDES->getConnection(userid, pw, 5); //new dsid3  
      cout<<" #getConnection, dsIdDES4(dsid3)=" <<
        dsIdDES4->getDatastore() << ", sessionId=" << dsIdDES4->getId() <<endl;


      dsIdDES4 = dsrDES->getConnection(userid, pw, 6); // new dsid4
      cout<< " #getConnection, dsIdDES4(dsid4)=" <<
        dsIdDES4->getDatastore() << ", sessionId=" << dsIdDES4->getId()<<endl;

      dsDES1 = dsrDES->getConnection(userid, pw);  // new ds3
      cout<<" #getConnection, dsDES1(ds3) =  " << dsDES1 <<endl;
      dsDES2 = dsrDES->getConnection(userid, pw);  // new ds4
      cout<<" #getConnection, dsDES2(ds4) =  " << dsDES2 <<endl;
 
      dsDES3 = dsrDES->getConnection(userid, pw);  // new ds5
      cout<< " #getConnection, dsDES3(ds5) =  " << dsDES3 <<endl;
      dsDES4 = dsrDES->getConnection(userid, pw);  // new ds6
      cout<<" #getConnection, dsDES4(ds6) =  " << dsDES4 <<endl;


      dsIdDES3 = dsrDES->getConnection(userid, pw, 9); // dsid2 
      cout<<" #getConnection, dsIdDES3(dsid2)=" <<
      dsIdDES3->getDatastore() << ", sessionId=" << dsIdDES3->getId()<<endl;


/**/
      //**** use dsDES2 to do ListCatalog
      cout<<"||**** start: use dsDES2 to do ListCatalog ****||"<<endl;

      DKSequentialCollection *pCol = 0;
      dkIterator *pIter = 0;
      DKSequentialCollection *pCol2 = 0;
      dkIterator *pIter2 = 0;
      DKServerDefDES *pSV = 0;
      DKDatabaseDefDES *pEnt = 0;
      DKString strServerName;
      DKString strServerType;
      DKString strDBName;
      DKFieldDefDES *pAttr = 0;
      DKAny a;
      DKAny *pA = 0;
      long i = 0;
      long j = 0;
      cout << "list servers" << endl;
      a = dsDES2->listDataSources();
      pCol = (DKSequentialCollection*)((dkCollection*)a);
      pIter = pCol->createIterator();
      while (pIter->more() == TRUE)
      {
       i++;
       pSV = (DKServerDefDES*)((void*)(*pIter->next()));
       strServerName = pSV->getName();
       strServerType = pSV->datastoreType();
       cout << "Server Name [" << i << "] - " << strServerName << " Server Type - " << strServerType << endl; 
       delete pSV;
      }
      delete pIter;
      delete pCol;
      cout << i << " servers listed" << endl;

      cout << "list entities" << endl;
      pCol = (DKSequentialCollection*)((dkCollection*)dsDES2->listEntities());
      pIter = pCol->createIterator();
      i = 0;
      while (pIter->more() == TRUE)
      {
       i++;
       pEnt = (DKDatabaseDefDES*)((void*)(*pIter->next()));
       strDBName = pEnt->getName();
       cout << "\ndatabase name [" << i << "] - " << strDBName << endl;
		cout << "dispname:  " << pEnt->getDisplayName() << endl;
		cout << "helptext:  " << pEnt->getHelpText() << endl;
		cout << "lang:      " << pEnt->getLanguage() << endl;
		int iVCount = pEnt->getNumVals();
		cout << "NumValus:  " << iVCount << endl;
		cout << "datatype:  " << pEnt->getDataType() << endl;
		cout << "searchable:" << pEnt->isSearchable() << endl;
		cout << "retrievable" << pEnt->isRetrievable() << endl;
       cout << "\n  list attributes for " << strDBName << " database name" << endl;
       pCol2 = (DKSequentialCollection*)((dkCollection*)dsDES2->listEntityAttrs(strDBName));
       pIter2 = pCol2->createIterator();
       j = 0;
       while (pIter2->more() == TRUE)
       {
        j++;
        pA = pIter2->next();
        pAttr = (DKFieldDefDES*) pA->value();
        cout << "    Attribute name [" << j << "] - " << pAttr->getName() << endl;
        cout << "      datastoreName " << pAttr->datastoreName() << endl;
        cout << "      datastoreType " << pAttr->datastoreType() << endl;
        cout << "      attributeOf   " << pAttr->getEntityName() << endl;
        cout << "      type          " << pAttr->getType() << endl;
        cout << "      size          " << pAttr->getSize() << endl;
        cout << "      id            " << pAttr->getId() << endl;
        cout << "      nullable      " << pAttr->isNullable() << endl;
        cout << "      precision     " << pAttr->getPrecision() << endl;
        cout << "      scale         " << pAttr->getScale() << endl;
        cout << "      string type   " << pAttr->getStringType() << endl;
        cout << "      display name  " << pAttr->getDisplayName() << endl;
        cout << "      help text     " << pAttr->getHelpText() << endl;
        cout << "      language      " << pAttr->getLanguage() << endl;
        cout << "      isQueryable   " << pAttr->isQueryable() << endl;
        cout << "      isRetrievable " << pAttr->isRetrievable() << endl;
        delete pAttr;
       }
       cout << "  " << j << " attributes listed for " << strDBName << " database name" << endl;
       delete pIter2;
       delete pCol2;
       delete pEnt;
      }
      delete pIter;
      delete pCol;
      cout << i << " entities listed\n" << endl;

      cout<<"||**** end: use dsDES2 to do ListCatalog ****||"<<endl;
      //****end of use dsDES2 to do ListCatalog 
/**/
      dsrDES->returnConnection(dsDES2);
      cout<<" #returnConnection, dsDES2(ds4) =  "<< dsDES2 <<endl;
      dsDES4 = dsrDES->getConnection(userid, pw);
      cout<<" #getConnection, dsDES4 =  "<< dsDES4 <<endl;
      cout<<"    dsDES4 == dsDES2 =  " <<(dsDES4 == dsDES2)<<endl;

    //**** you can add codes to use dsDES4 to do other thing here  
    //**** end of use dsDES4 to do other thing

      dsrDES->returnConnection(dsDES1);
      cout<<" #returnConnection, dsDES1(ds3) =  " << dsDES1 <<endl;
      dsrDES->returnConnection(dsDES3);
      cout<<" #returnConnection, dsDES3(ds5) =  " << dsDES3 <<endl;
      dsrDES->returnConnection(dsIdDES4); //dsid4 in _freeIdPool
      cout<<" #returnConnection, dsIdDES4(dsid4) =  " <<
        dsIdDES4->getDatastore() << ", sessionId=" << dsIdDES4->getId() <<endl;
     

      cout<<"**** start sleep for 120050 mini sec*******"<<endl;
#ifdef WIN32
     Sleep(120050);     
#elif AIX
     sleep(120050);
#endif    
      cout<<"**** end sleep for 120050 mini sec*******"<<endl;


      dsDES2 = (DKDatastoreDES*)dsrDES->getConnection(userid, pw);  //new ds7
      cout<<" #getConnection, dsDES2(ds7) =  " << dsDES2<<endl;
      cout<<" #call delete dsrDES..." <<endl;
      delete dsrDES;  //take a while for all 4 in usedPool to be disconnected
   }
   catch(DKException &exc)
   {
      cout << "Error id " << exc.errorId() << endl;
      for(unsigned long i = 0; i< exc.textCount();i++)
      {
         cout << "Error text: " << exc.text(i) << endl;
      }
      for (unsigned long g=0; g< exc.locationCount();g++)
      {
         const DKExceptionLocation* p = exc.locationAtIndex(g);
         cout << "Filename: " << p->fileName() << endl;
         cout << "Function: " << p->functionName() << endl;
         cout << "LineNumber: " << p->lineNumber() << endl;
      }

      cout << "Exception Class Name: " << exc.name() << endl;
   }
   
   cout << "done ..." << endl;   
}
