/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

// *****************************************************************************
// File Name   = DemoSimpleAppl.java
// Description = This sample application will demonstrate the functions in non-visual beans.
//               This application will use the following beans:
//                      CMBConnection
//                      CMBSchemaManagement
//                      CMBSearchTemplate
//                      CMBQueryService
//                      CMBSearchResults
//                      CMBDataManagement
//                      util/CMBTraceLog
//               And involves the following functions:
//                  a.  Connect to federated server
//                  b.  get a list of search templates names
//                  c.  get a list of search criteria names based on the search template name
//                  c.  pick a search template, get its search criteria
//                  d.  fill in the values and submit a query.
//                  e.  print out the result in search results bean
//                  f.  select a result row and display it
//                  g.  disconnect
//
// *****************************************************************************
import com.ibm.mm.beans.*;
import com.ibm.mm.beans.util.*;

import java.beans.*;
import java.io.*;
import java.util.*;

public class DemoSimpleAppl {
    int docCounter = 0;
    UtilPersistent per = new UtilPersistent();
    UtilSimpleUI ui = new UtilSimpleUI();
    CMBSchemaManagement schemaBean = null;
    CMBSearchResults resultBean = null;
    CMBConnection connBean = null;
    CMBQueryService queryBean = null;
    CMBSearchTemplate stBean = null;
    CMBDataManagement dataBean = null;
    CMBTraceLog traceBean = null;

    public DemoSimpleAppl() {
        try {
            System.out.println("Creating CMBUserManagement bean using " + "ser/class/jar specified in the classpath.");
            connBean = (com.ibm.mm.beans.CMBConnection) Beans.instantiate(null, "com.ibm.mm.beans.CMBConnection");

            System.out.println("Creating CMBSchemaManagement bean from CMBConnection bean...");
            schemaBean = connBean.getSchemaManagement();

            System.out.println("Creating CMBQueryService bean from CMBConnection bean...");
            queryBean = connBean.getQueryService();
            schemaBean.setCacheEnabled(true);

            System.out.println("Creating CMBDataManagement bean from CMBConnection bean...");
            dataBean = connBean.getDataManagement();

            resultBean = (com.ibm.mm.beans.CMBSearchResults) Beans.instantiate(null, "com.ibm.mm.beans.CMBSearchResults");
            resultBean.setConnection(connBean);

            traceBean = (com.ibm.mm.beans.util.CMBTraceLog) Beans.instantiate(null, "com.ibm.mm.beans.util.CMBTraceLog");

            connBean.addCMBTraceListener(traceBean);
            schemaBean.addCMBTraceListener(traceBean);
            resultBean.addCMBTraceListener(traceBean);
            queryBean.addCMBTraceListener(traceBean);
            dataBean.addCMBTraceListener(traceBean);

            connBean.setTraceEnabled(true);
            schemaBean.setTraceEnabled(true);
            resultBean.setTraceEnabled(true);
            queryBean.setTraceEnabled(true);
            dataBean.setTraceEnabled(true);

            traceBean.setDebugLogEnabled(true);
            traceBean.setProgressLogEnabled(true);

            traceBean.setDisplayMode(CMBUtilConstant.DISPLAY_MESSAGE_IN_WINDOW);
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    /**
     *  Checks CMBConnection bean property before connect.
     */
    void checkConnBeanProperty() {
        try {
            if (connBean.getServerName().length() == 0) {
                String srv = ui.getString("Server name is empty, " + "please enter the server name:");
                connBean.setServerName(srv);
            }
            if (connBean.getUserid().length() == 0) {
                String uid = ui.getString("User ID is empty, please enter the user ID:");
                connBean.setUserid(uid);
            }
            if (connBean.getPassword().length() == 0) {
                String pwd = ui.getString("Password is empty, please enter the password:");
                connBean.setPassword(pwd);
            }
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    void connect() {
        checkConnBeanProperty();

        try {
            System.out.println("Connecting...");
            connBean.connect();
            System.out.println("OK, connected\n");
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    public void listTemplates() {
        try {
            String[] stNames = schemaBean.getSearchTemplateNames();
            System.out.println("There are " + stNames.length + " search templates:");
            for (int i = 0; i < stNames.length; i++) {
                System.out.println("\t" + stNames[i]);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void listSTCriteria() {
        try {
            String stName = ui.getString("Please enter the search template name:");
            CMBSearchTemplate stObj = schemaBean.getSearchTemplate(stName);
            System.out.println("Search template '" + stName + "' retrieved...");
            String[] critNames = stObj.listSearchCriteriaNames();
            System.out.println("There are " + critNames.length + " criteria: ");
            for (int i = 0; i < critNames.length; i++) {
                System.out.println("\t" + critNames[i]);
                CMBSTCriterion critObj = stObj.getSearchCriterion(i);
                System.out.println("\t\tdisplayName:  " + critObj.getDisplayName());
                System.out.println("\t\tattrName:     " + critObj.getAttrName());
                System.out.println("\t\tdisplayWidth: " + critObj.getDisplayWidth());
                System.out.println("\t\ttype:         " + critObj.getType());
                System.out.println("\t\tdefaultValue: " + critObj.getDefaultValue());
                System.out.println("\t\tdefaultOp:    " + critObj.getDefaultOperator());
                short[] validOps = critObj.getValidOperators();
                System.out.println("\t\tvalidOps:");
                for (int j = 0; j < validOps.length; j++) {
                    switch ((int) validOps[j]) {
                        case CMBBaseConstant.CMB_OP_EQUAL :
                            System.out.println("\t\t\t.EQ.");
                            break;
                        case CMBBaseConstant.CMB_OP_NOT_EQUAL :
                            System.out.println("\t\t\t.NEQ.");
                            break;
                        case CMBBaseConstant.CMB_OP_LESS_EQUAL :
                            System.out.println("\t\t\t.LEQ.");
                            break;
                        case CMBBaseConstant.CMB_OP_LESS :
                            System.out.println("\t\t\t.LT.");
                            break;
                        case CMBBaseConstant.CMB_OP_GREATER_EQUAL :
                            System.out.println("\t\t\t.GEQ.");
                            break;
                        case CMBBaseConstant.CMB_OP_GREATER :
                            System.out.println("\t\t\t.GT.");
                            break;
                        case CMBBaseConstant.CMB_OP_BETWEEN :
                            System.out.println("\t\t\t.BETWEEN.");
                            break;
                        case CMBBaseConstant.CMB_OP_NOT_BETWEEN :
                            System.out.println("\t\t\t.NOTBETWEEN.");
                            break;
                        case CMBBaseConstant.CMB_OP_IN :
                            System.out.println("\t\t\t.IN.");
                            break;
                        case CMBBaseConstant.CMB_OP_NOT_IN :
                            System.out.println("\t\t\t.NOTIN.");
                            break;
                        case CMBBaseConstant.CMB_OP_LIKE :
                            System.out.println("\t\t\t.LIKE.");
                            break;
                        case CMBBaseConstant.CMB_OP_NOT_LIKE :
                            System.out.println("\t\t\t.NOTLIKE.");
                            break;
                        default :
                            break;
                    }
                }
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    private void getSearchCriterion(CMBSearchTemplate stObj) {
        try {
            String critName = ui.getString("Please enter criterion name: ");
            Vector critList = stObj.getSearchCriteria();
            CMBSTCriterion critObj = null;
            for (int i = 0; i < critList.size(); i++) {
                critObj = (CMBSTCriterion) critList.elementAt(i);
                if (critObj.getName().equals(critName))
                    break;
            }

            if (critObj == null) {
                System.err.println("Invalid criterion name, please try again...");
                return;
            }

            int critOp =
                ui.getInteger(
                    "Please enter search operator: "
                        + "\nEQ: 1\t\tNEQ: 2\t\tLEQ: 3\t\tLT: 4"
                        + "\nGEQ: 5\t\tGT: 6\t\tBETWEEN: 7\tNOTBETWEEN: 8"
                        + "\nIN: 9\t\tNOTIN: 10\tLIKE: 11\tNOTLIKE: 12\n");
            switch (critOp) {
                case 1 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_EQUAL);
                    break;
                case 2 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_EQUAL);
                    break;
                case 3 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_LESS_EQUAL);
                    break;
                case 4 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_LESS);
                    break;
                case 5 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_GREATER_EQUAL);
                    break;
                case 6 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_GREATER);
                    break;
                case 7 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_BETWEEN);
                    break;
                case 8 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_BETWEEN);
                    break;
                case 9 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_IN);
                    break;
                case 10 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_IN);
                    break;
                case 11 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_LIKE);
                    break;
                case 12 :
                    critObj.setOperator(CMBBaseConstant.CMB_OP_NOT_LIKE);
                    break;
                default :
                    break;
            }
            if (critOp == 7 || critOp == 8) {
                String critVal1 = ui.getString("Please enter 1st search value: ");
                String critVal2 = ui.getString("Please enter 2nd search value: ");
                String[] critVals = new String[2];
                critVals[0] = critVal1;
                critVals[1] = critVal2;
                critObj.setValues(critVals);
            } else if (critOp == 9 || critOp == 10) {
                Vector vals = new Vector();
                boolean notDone = true;
                while (notDone) {
                    String val = ui.getString("Please enter value: (-1 if done entering)");
                    if (val.equals("-1"))
                        notDone = false;
                    else
                        vals.addElement(val);
                }
                String[] critVals = new String[vals.size()];
                vals.copyInto(critVals);
                critObj.setValues(critVals);
            } else {
                String critVal = ui.getString("Please enter search value: ");
                if (critVal.equals("null"))
                    critObj.setValue(critVal);
                else
                    critObj.setValue(critVal);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void runQuery() {
        try {
            boolean runQry = false;
            String stName = ui.getString("Please enter the search template name:");
            CMBSearchTemplate stObj = schemaBean.getSearchTemplate(stName);
            if (stObj == null) {
                System.out.println("Unknown template name, please try again");
                return;
            }

            String[] critNames = stObj.listSearchCriteriaNames();
            System.out.println("There are " + critNames.length + " criteria: ");
            for (int i = 0; i < critNames.length; i++) {
                System.out.println("\t" + critNames[i]);
            }

            while (!runQry) {
                System.out.println("Selections:");
                System.out.println("1. Specify a search criterion");
                System.out.println("2. Run query ...");
                switch (ui.getInteger("Enter your selection:")) {
                    case 2 :
                        runQry = true;
                        break;
                    case 1 :
                        getSearchCriterion(stObj);
                        break;
                }
            }

            stObj.addCMBSearchReplyListener(resultBean);
            stObj.setAsynchSearch(false);
            queryBean.setAsynchSearch(false);
            queryBean.runQueryWithCursor(stObj);
            resultBean.newResults(queryBean.getResults());
            CMBItem[] itemList = resultBean.getItem();
            docCounter = 0;
            for (int k = 0; k < itemList.length; k++) {
                CMBItem hit = (CMBItem) itemList[k];
                System.out.println("Hit item #" + k + ", pidString=" + hit.getPidString());
                try {
                    String[] values = hit.getAttrValues();
                    String[] names = hit.getAttrNames();
                    for (int l = 0; l < values.length; l++)
                        System.out.println("\t" + names[l] + ": " + values[l]);
                    if (docCounter < 5) {
                        docCounter++;
                        String dsType = hit.getServerType();
                        System.out.println("Found an item of dsType '" + dsType + "' - about to bring up " + dsType + " viewer ...");
                        launchViewer(hit);
                    }
                } catch (CMBException ex) {
                    ex.printStackTrace();
                    Object data = ex.getErrorData();
                    if (data != null && data instanceof Exception)
                         ((Exception) data).printStackTrace();
                }
            }
        } catch (CMBConnectFailedException ex) {
            System.out.println("Connection to back-end server failed...");
            ex.printStackTrace();
        } catch (CMBNoConnectionException ex) {
            System.out.println("There is no current connection, please logon..");
            ex.printStackTrace();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void launchViewer(CMBItem item) {
        try {
            dataBean.setDataObject(item);

            // Make sure there is content
            if (dataBean.getContentCount() == 0) {
                System.out.println("There is no content for this document.");
                return;
            }

            // Determine application to launch for mimetype
            Properties mime2appProps = schemaBean.getDefaultMime2App();
            String mimeType = dataBean.getItemMimeType();
            String appString = (String) mime2appProps.get(mimeType);
            if (appString == null) {
                System.out.println("No viewer is defined to view documents of type " + mimeType);
                System.out.println("Define an application for viewing " + mimeType + " using EIP Administration.");
                return;
            }
            System.out.println(appString);
            int appIndex = appString.indexOf("a=");
            int extIndex = appString.indexOf("x=");
            String app;
            String ext;
            if (extIndex > 0) {
                app = appString.substring(appIndex + 2, extIndex - 1);
                ext = appString.substring(extIndex + 2);
            } else {
                app = appString.substring(appIndex + 2);
                ext = "cmb";
            }

            // Write document to file
            String docFileName = "Doc" + docCounter + ext;
            FileOutputStream docFile = new FileOutputStream(docFileName);
            docFile.write(dataBean.getContent(0).getData());
            docFile.close();

            // Launch application to view document
            try {
                Runtime.getRuntime().exec(app + " " + docFileName);
                System.out.println("Viewer bean called ...");
            } catch (Exception e) {
                System.out.println("The application " + app + " could not be launched.");
                System.out.println("Make sure the application is available, or change the application defined");
                System.out.println("to view documents of type " + mimeType + " using EIP Administration.");
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public static void main(String argv[]) {
        String testName = "IBM CMB Demonstration program: DemoSimpleAppl";
        String copyright = "C) Copyright IBM Corp. 1994,1999. All Rights Reserved.";

        System.out.println("");
        System.out.println(testName);
        System.out.println(copyright);
        System.out.println("");
        System.out.println("");

        UtilSimpleUI ui = new UtilSimpleUI();
        DemoSimpleAppl psObj = new DemoSimpleAppl();
        boolean loopControl = true;

        System.out.println("Going to connect to the server...");
        psObj.connect();

        while (loopControl) {
            System.out.println("");
            System.out.println("Selections:");
            System.out.println("0. Exit");
            System.out.println("1. List Search Templates");
            System.out.println("2. List Search Criteria");
            System.out.println("3. Run query ...");
            switch (ui.getInteger("Enter your selection:")) {
                case 0 :
                    loopControl = false;
                    break;
                case 1 :
                    psObj.listTemplates();
                    break;
                case 2 :
                    psObj.listSTCriteria();
                    break;
                case 3 :
                    psObj.runQuery();
                    break;
            }
        }
        try {
            psObj.connBean.disconnect();
            psObj.traceBean.finalize();
            System.out.println("Bye.");
            System.exit(0);
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}