/*
@copyright(disclaimer)

Licensed Materials - Property of IBM
IBM  Enterprise Information Portal  for Multiplatforms V8.1  (program number 5724-B43)
(c) Copyright IBM Corp. 1998,  2002. All Rights Reserved.

US Government Users Restricted Rights
Use, duplication or disclosure restricted by GSA ADP Schedule
Contract with IBM Corp.

DISCLAIMER OF WARRANTIES :

Permission is granted to copy and modify this  Sample code, and to
distribute modified versions provided that both the copyright
notice,- and this permission notice and warranty disclaimer  appear in all copies and modified
versions.

THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.
IBM  AND ITS SUPPLIERS AND LICENSORS  DISCLAIM
ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY
OF NON-INFRINGEMENT AND THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
PARTICULAR PURPOSE. IN NO EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY
DAMAGES ARISING OUT OF THE USE OF  OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF
THE SAMPLE CODE, OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL
IBM OR ITS LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA,
OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS LICENSORS OR SUPPLIERS
HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

@endCopyright
*/

//*****************************************************************************
// File Name   = DemoTraceLog.java
// Description = Demonstrate CMBTraceLog bean functionality
// Author      = afic
// Date        = 04/04/1999
//
//*****************************************************************************

import com.ibm.mm.beans.*;
import com.ibm.mm.beans.util.*;

import java.beans.*;

public class DemoTraceLog {
    UtilPersistent per = new UtilPersistent();
    UtilSimpleUI ui = new UtilSimpleUI();
    DemoConnect connDemo = new DemoConnect();
    DemoSchema schemaDemo = new DemoSchema();
    CMBTraceLog traceBean = null;

    DemoTraceLog() {
        connDemo.connBean.setTraceEnabled(true);
        schemaDemo.schemaBean.setTraceEnabled(true);
        schemaDemo.schemaBean.setConnection(connDemo.connBean);

        try {
            System.out.println("Creating CMBTraceLog bean using " + "ser/class/jar specified in the classpath.");
            traceBean = (com.ibm.mm.beans.util.CMBTraceLog) Beans.instantiate(null, "com.ibm.mm.beans.util.CMBTraceLog");
            connDemo.connBean.addCMBTraceListener(traceBean);
            schemaDemo.schemaBean.addCMBTraceListener(traceBean);
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }

    void changeProperty() {
        if (ui.getBoolean("Display messages on a GUI Windows?")) {
            traceBean.setDisplayMode(CMBUtilConstant.DISPLAY_MESSAGE_IN_WINDOW);
        } else
            traceBean.setDisplayMode(CMBUtilConstant.DISPLAY_MESSAGE_ON_STDERR);

        String prompt =
            "Enter the trace log file name with the complete path:\n"
                + "Make sure the destination directory is write-permitted\n"
                + "Hit <Enter> to use default name cmbTrace.log in current directory";
        String logPath = ui.getStringAllowEmpty(prompt);
        if (logPath.length() > 0)
            traceBean.setLogFullpath(logPath);
    }

    void load() {
        connDemo.connBean.removeCMBTraceListener(traceBean);
        schemaDemo.schemaBean.removeCMBTraceListener(traceBean);
        traceBean = (CMBTraceLog) per.load();
        connDemo.connBean.addCMBTraceListener(traceBean);
        schemaDemo.schemaBean.addCMBTraceListener(traceBean);
    }

    void save() {
        per.save(traceBean);
    }

    public static void main(String argv[]) {
        String demoName = "IBM CMB Demonstration program: DemoTraceLog";
        String copyright = "C) Copyright IBM Corp. 1994,1999. All Rights Reserved.";

        System.out.println("");
        System.out.println(demoName);
        System.out.println(copyright);
        System.out.println("");
        System.out.println("");

        UtilSimpleUI ui = new UtilSimpleUI();
        DemoTraceLog demo = new DemoTraceLog();
        boolean loopControl = true;

        while (loopControl) {
            System.out.println("");
            System.out.println("Selections:");
            System.out.println("0. Exit");
            System.out.println("1. Re-load the CMBTraceLog bean from a file");
            System.out.println("2. Change property values");
            System.out.println("3. Try connect");
            System.out.println("4. Try schema functions");
            System.out.println("5. Try disconnect");
            System.out.println("6. Save this CMBTraceLog bean to a file");
            switch (ui.getInteger("Enter your selection:")) {
                case 0 :
                    loopControl = false;
                    break;
                case 1 :
                    demo.load();
                    break;
                case 2 :
                    demo.changeProperty();
                    break;
                case 3 :
                    demo.connDemo.connect();
                    break;
                case 4 :
                    demo.schemaDemo.run();
                    break;
                case 5 :
                    demo.connDemo.disconnect();
                    break;
                case 6 :
                    demo.save();
                    break;
            }
        }

        demo.connDemo.disconnect();
        System.out.println("Bye.");
        System.exit(0);
    }
}