/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/******************************************************************************
 * Filename       : TLoadSampleDL.java
 *
 * Description    : This sample loader using java class library's API to
 * load sample data into the Digital Library.  You should run it with the
 * provided input data file "sample.dat". The physical files for this
 * loader are packaged in "samples.jar".  You should use command
 * "jar -xvf samples.jar" to unpack it.
 *
 * To run it as default, this loader assumes following are already defined: 
 * 
 *    Index Class Name - "DLSAMPLE" (defined with the key fields in step 4)
 *    text search server(search service name) - "TM"
 *    text search index - "TMINDEX"
 * 
 * Invoke Syntax:
 *
 *    java TLoadSampleDL <inputDataFile> <outputLog> <userid> <password> <libraryServer>
 *
 *    where:  
 *      <inputDataFile>: a input control file, see sample.dat for example. 
 *      <outputLog>: the log file to store the process results, it can be used
 *                   for debug purpose, you can delete it later.  
 *      <userid>: the userid you use to logon the library server.
 *      <password>: the password to logon the library server.
 *      <libraryServer>: the library server name
 *
 * ### NOTE: to compile  ###
 *    NT:
 *      Edit the x:\cmbroot\cmbenv81.bat file to set the environment variables.
 *      for example:  set CMBROOT=e:\cmbroot
 *                    set DB2HOME=e:\sqllib
 *      under x:\cmbroot directory enter: cmbenv81
 *
 *      under x:\cmbroot\samples\java\dl directory enter following to compile:
 *            javac TLoadSampleDL.java
 *
 *      The TLoadSampleDL.class will be created if compiled successfully.     
 *
 *    AIX:
 *      Make sure the CLASSPATH environment variable has the following values:
 *            /usr/lpp/cmb/lib/cmbcm81.jar
 *            /usr/lpp/cmb/lib/cmbdl81.jar
 *      under /usr/lpp/cmb/samples/java/dl enter following to compile:
 *            javac TLoadSampleDL.java 
 *
 *      The TLoadSampleDL.class will be created if compiled successfully.      
 *
 * ### NOTE: ###
 *    This loader also allow you to invoke with "indexClassName" as the input 
 *    parameter.  Please be aware that your index class's attribute names and
 *    input data file should match. For example you invoke as:    
 *
 *    java TLoadSampleDL sample.dat my.log frnadmin password LIBSRVRN MYINDEX
 *
 *    The index class name "DLSAMPLE" will be replaced with "MYINDEX". 
 ******************************************************************************
 * Following is the instruction to customize the loader to run your own data 
 ******************************************************************************
 * This loader can be modified to work with any index class defined with		
 * different attribute types(key fields).  Rules to follow:
 *
 *  1. The attribute name in the inputDataFile(ie. sample.dat) should
 *     exactly match with the index class's attribute name(case sensitive). 
 *     for example: 
 *
 *     index class attribute names     in sample.dat file
 *     ---------------------------   ----------------------
 *      DLSEARCH_Author              DLSEARCH_Author=Janet Hedges
 *      DLSEARCH_Publisher           DLSEARCH_Publisher=IBM
 *
 *     index class with your           your <inputDataFile> 
 *     defined attribute names         (ie. mine.dat)
 *     ---------------------------   ----------------------
 *      Department                   Department=U6M
 *      Manager                      Manager=John Smith
 *
 *  2. In your inputDataFile, the prefix for the parts to be added should be 
 *     "xdo" or "smi".  The "xdo" is for a regular part, "smi" is for the part
 *     to be indexed by the text search manager.
 *
 *  3. The inputDataFile should use end= to represent the ending of an item.
 *
 *  4. Examples of index class's attributes and sample.dat
 *
 *     index class attr. name     character length min - max
 *     ----------------------    ----------------------------
 *      DLSEARCH_Author            1 - 50
 *      DLSEARCH_Publisher         1 - 50
 *      DLSEARCH_DocType           1 - 32
 *      DLSEARCH_Date              1 - 50
 *      DLSEARCH_Title             1 - 100
 *      DLSEARCH_Subject           1 - 100
 *      DLSEARCH_Authors           1 - 100
 *      DLSEARCH_Editors           1 - 50
 *      DLSEARCH_Source            1 - 100
 *      DLSEARCH_Volume            1 - 20
 *      DLSEARCH_Issue             1 - 20
 *      DLSEARCH_Relation          1 - 64
 *
 *    (all above key field names are defined with key field type as
 *     "Variable character" and character type as "Extended alphanumeric", the
 *     character length min - max are shown as above)
 *
 *     the sample.dat file: (shown as following)
 *    --------------------------------------------
 *   ########################################################
 *   DLSEARCH_Author=Anonymous
 *   DLSEARCH_Authors=
 *   DLSEARCH_Subject=Contents
 *   DLSEARCH_Editors=
 *   DLSEARCH_Publisher=IBM
 *   DLSEARCH_Date=1994
 *   DLSEARCH_Source=Software Quarterly
 *   DLSEARCH_Volume=1
 *   DLSEARCH_Issue=1
 *   DLSEARCH_Title=Contents of Software Quarterly Volume 1 Issue 1
 *   xdo=vol11\contents.htm
 *   xdo=images\321smt.gif
 *   xdo=images\sqcloud0.gif
 *   xdo=images\sqclouda.gif
 *   xdo=images\choice.gif
 *   xdo=images\321sm.gif
 *   xdo=images\sqhome.gif
 *   xdo=images\tellsq.gif
 *   xdo=images\getsq.gif
 *   xdo=images\software.gif
 *   xdo=vol11\contents.txt
 *   smi=vol11\contents.smi
 *   end=end
 *   ########################################################
 *
 *  note: here the prefix "smi" is for the part to be indexed by
 *  the text search manager.  ie. contents.smi 
 *
 *  5. Invoke example: (in NT system, with files in the following directories)
 *     x:\cmbroot\samples\java\dl\TLoadSampleDL.java -- this program
 *     x:\cmbroot\samples\java\dl\sample.dat       -- inputDataFile
 *     x:\cmbroot\samples\java\dl\vol11\*.*        -- input text files are here
 *     x:\cmbroot\samples\java\dl\images\*.*       -- input image files are here
 *
 *     In x:\cmbroot\samples\java\dl\] type as:
 *
 *     java TLoadSampleDL sample.dat sample.log frnadmin password LIBSRVRN 
 *
 *     (note: in AIX system path is /usr/lpp/cmb/samples/java/dl/  )
 *
 *  6. If physical files are in different directories then file path in
 *     sample.dat should be modified.
 *
 *  7. This loader requires the indexClass("DLSAMPLE") and
 *     searchIndex("TM-TMINDEX") have been defined first.
 *     The "DLSAMPLE" has attribute names defined as 4. and all
 *     the field types are "variable character".
 *
 *     If you have different index class name, 
 *     text search server(search service name) or text search index,
 *     please modify the following variables:
 *
 *       indexClassName = "DLSAMPLE",  //or other index class name ie. "BROADWAY"
 *       searchEngine = "SM",          //search manager ie. SM or QBIC
 *       searchIndex = "TM-TMINDEX",   //search server-search index name
 *       searchInfo = "ENU",           //ENU for english, FRA for french...
 *
 *  8. If you want to use this loader to load data without part to be indexed	
 *     by text search manager, please comment out the lines indicated by the 
 *     |**** no text search by comment out following line ****| &  
 *     |**** end of comment out for no text search ***********|
 *     In this case, you don't need to define the search index name. 
 * 
 *****************************************************************************/  
import java.util.*;
import java.io.*;
import java.lang.*;
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;

public class TLoadSampleDL
       implements DKConstantDL
{
  static String
             indexClassName = "DLSAMPLE", //Digital Library index class name
             searchEngine = "SM",         //search manager ie. SM or QBIC
             searchIndex = "TM-TMINDEX",  //search service name-search index name
             searchInfo = "ENU";          //ENU for english, FRA for french...

  public static void main (String[] args)
  {
    // Indicate this is a Console Application on Windows NT/95 for error
    // message display in dos console code page. If this was a Windows
    // GUI (Graphic User Interface) Application this would not be necessary
    // because DK_CM_SS_WINDOWS is the default.
    DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    //sample loader always loading English data samples
    Locale origLocale = Locale.getDefault();
    boolean reSet = false;
    if (!(Locale.getDefault().getCountry().equals("US")))
    { 
      reSet = true;
      Locale.setDefault(new Locale("en", "US"));
    }

    DKDatastoreDL dsDL = null;
    try
    {
      // *** for generated notepart file name
      String noteName = "notepart.txt";   
      
      int smiPartId = 1,                  //partId for search indexed part
          notePartId = 2;                 //partId for notepart

      boolean tocFlag = false,            // flag to show item is a toc item
              xdoFlag = false,            // flag to signal part is exist
              smiFlag = false,            // flag to signal search part is exist 
              reset = false;              // reset tocItemId for new Toc
           
      String  smiName = new String(""),   //filepathname of search part
              tocItemId = new String(""); //for DLSEARCH_relation attr. value

      short dataId = 0;
      short i, j;
      short docIndx = -1,
            itemIndx = -1;
      int xdoCnt = 0,
          attrCnt = 0,
          maxXDOs = 50,
          maxAttr = 50;
      String[] xdoPathNames = new String[maxXDOs];
      String[] attrName = new String[maxAttr];
      String[] attrValue = new String[maxAttr];
      short[]  attrId = new short[maxAttr];
      short[]  attrType  = new short[maxAttr];
      boolean[]  attrNullable  = new boolean[maxAttr];
      BufferedWriter log;  

      //-------------------------------------------------------------- 
      // Checking for input parameters
      //-------------------------------------------------------------- 
      if (args.length < 5)
      {
       System.out.println("Invoke Syntax: " );
       System.out.println("  java TLoadSampleDL <inputDataFile> <outputLogFile> " +
        "<userid> <password> <libraryServer>" );
       return;
      }
      // *** get input parameters
      File inputFile = new File(args[0]);
      File outputFile = new File(args[1]);
      String userid = args[2];
      String password = args[3];
      String libSrv = args[4];
      //----------------------------------------------------------------- 
      // support to run with user's own DL index class - at your own risk
      //----------------------------------------------------------------- 
      if (args.length > 5)
      {
       indexClassName = args[5];
      }   
      log   = new BufferedWriter(new FileWriter(outputFile));
      // *** for writing to log
      if (args.length == 5)  
      {    
       log.write("You enter: java TLoadSampleDL " +
       args[0] + " " + args[1] + " " + args[2] + " " + args[3] + " " + args[4]);
      }
      else
      {
       log.write("You enter: java TLoadSampleDL " + args[0] + " " +
       args[1] + " " + args[2] + " " + args[3] + " " + args[4] + " " + args[5]);
      }
      log.newLine();
 
      if (inputFile.exists() == false)
      {
       System.out.println("Input data file does not exist: " + args[0]);
       return;
      }

      //-------------------------------------------------------------- 
      // verify the TextMiner "TMINDEX" is defined
      //-------------------------------------------------------------- 
      verifySetUp(userid, password);

      //-------------------------------------------------------------- 
      // connecting to data store
      //-------------------------------------------------------------- 
      dsDL = new DKDatastoreDL();
      System.out.println("Connecting datastore, please wait...");
      dsDL.connect(libSrv, userid, password,"");
      System.out.println("datastore connected");
      System.out.println("Loading " + args[0] + ",  please wait.... ");
      Integer opt = new Integer(DK_CM_FALSE);
      dsDL.setOption(DK_DL_OPT_WAKEUPSRV, (Object)opt);
      DKSequentialCollection aCol;
      dkIterator aIter;
      DKAttrDefDL aDef;
      aCol = (DKSequentialCollection) dsDL.listEntityAttrs(indexClassName);
      aIter = aCol.createIterator();
      i = 0;

      //-------------------------------------------------------------- 
      // obtains data for setting up attrbuteList
      //-------------------------------------------------------------- 
      short sAttrType = 0;
      while (aIter.more())
      {
        aDef = (DKAttrDefDL) aIter.next();
        attrName[i] = aDef.getName();
        if (attrName[i].equals("DLSEARCH_DocType"))
        {
           docIndx = i;
           attrValue[i] = "html";
        }
        else if (attrName[i].equals("DLSEARCH_Relation"))
        {
           itemIndx = i;
           attrValue[i] = tocItemId;
        }
        sAttrType        = aDef.getType();
        attrType[i]      = translateAttrTypeToDataItemType(sAttrType);
        attrId[i] = aDef.getId();
        attrNullable[i] = aDef.isNullable();
        i++;
        attrCnt++;
      }

      //-------------------------------------------------------------- 
      // start to read input data file
      //-------------------------------------------------------------- 
      String line, name = "", value = "";
      BufferedReader in
          = new BufferedReader(new FileReader(inputFile));

      read: while ((line  = in.readLine()) != null)
      {
       int n;
       name = value = "";
       line = line.trim();
       if (line.startsWith("#"))
                continue;
       n = line.indexOf("=");
       if (n == -1)
          continue;
       name = (line.substring(0,n)).trim();
       value = (line.substring(n + 1)).trim();
       if (value.equals(""))
          continue;

       //-------------------------------------
       // read and store the attribute valude
       //-------------------------------------
       for (i = 0; i < attrCnt; i++)
       {
         if ((attrName[i].toUpperCase()).equals(
               name.toUpperCase()))
	     {
           attrValue[i] = value;
           continue read;
	     }
       }
       if (itemIndx != -1)
         attrValue[itemIndx] = tocItemId;

       //-------------------------------------
       // read and store the xdo FilePathName
       //-------------------------------------
       if (name.toLowerCase().equals("xdo"))
       {
         xdoFlag = true;
         xdoPathNames[xdoCnt] = value;
         xdoCnt = xdoCnt + 1;
       }
       else if (name.toLowerCase().equals("smi"))
       {
         smiName = value;
         smiFlag = true;
       }
       else
       if (name.toLowerCase().equals("end"))
       {
         //-------------------------------------
         // start to create DDO for an item
         //-------------------------------------
         if (value.toLowerCase().equals("toc"))
             tocFlag = true;
         if (value.toLowerCase().equals("endtoc"))
             reset = true;
         System.out.println("start to create a DDO...");
         DKPid pid = new DKPid();
         pid.setObjectType(indexClassName);
         DKDDO ddo = new DKDDO(dsDL, pid);
         ddo.addProperty(DK_CM_PROPERTY_ITEM_TYPE,
                         new Short(DK_CM_DOCUMENT));

         for (i = 0; i < attrCnt; i++)
         {
          dataId = ddo.addData(attrName[i]);
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_ATTRIBUTE_ID,
              new Short(attrId[i]));
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_TYPE,
          new Short(attrType[i]));
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_NULLABLE,
              new Boolean(attrNullable[i]));
          ddo.addDataProperty(dataId, DK_CM_PROPERTY_ACCESS_RIGHT,
              new Integer(DK_CM_ATTRIBUTE_READWRITE));
          ddo.setData(dataId, attrValue[i]);
         }

         log.write(" dataCount = " + ddo.dataCount());
         log.newLine();
         DKParts parts = new DKParts();
         int  partId;  
         DKBlobDL  smixdo = new DKBlobDL(dsDL);
         if (smiFlag)
	     {
           smixdo.setContentClass(DK_DL_CC_ASCII);
           smixdo.setAffiliatedType(DK_DL_BASE);

/**** no text search by comment out following line ****/
           DKSearchEngineInfoDL aSrchEx = new DKSearchEngineInfoDL();
           aSrchEx.setSearchEngine(searchEngine);
           aSrchEx.setSearchIndex(searchIndex);
           aSrchEx.setSearchInfo(searchInfo);
           smixdo.setExtension("DKSearchEngineInfoDL", (dkExtension)aSrchEx);
/**** end of comment out for no text search ***********/

           smixdo.setContentFromClientFile(smiName);
           DKPidXDODL apid = new DKPidXDODL();
           apid.setPartId(smiPartId);
           smixdo.setPidObject(apid);
           parts.addElement(smixdo);
           log.write("creat smi part: " + smiName);
           log.newLine();
         }
         if (xdoFlag)
	     {
          String suffix = "";
          partId = 3;
          int m, CClass;
          for (i = 0; i < xdoCnt; i++)
	      {
            m = xdoPathNames[i].lastIndexOf(".");
            if (m != -1) suffix = xdoPathNames[i].substring(m + 1);
            if ( (suffix.toLowerCase()).equals("gif") )
                  CClass = DK_DL_CC_GIF;
            else if ( ((suffix.toLowerCase()).equals("html")) ||
                      ((suffix.toLowerCase()).equals("htm")) )
                  CClass = DK_DL_CC_HTML;
            else if ( (suffix.toLowerCase()).equals("jpg") )
                  CClass = DK_DL_CC_JPG;
            else if ( (suffix.toLowerCase()).equals("avi") )
                  CClass = DK_DL_CC_AVI;
            else if ( (suffix.toLowerCase()).equals("wav") )
                  CClass = DK_DL_CC_WAV;
            else if ( (suffix.toLowerCase()).equals("mid") )
                  CClass = DK_DL_CC_MIDI;
            else if ( (suffix.toLowerCase()).equals("mov") )
                  CClass = DK_DL_CC_MOV;
            else if ( (suffix.toLowerCase()).equals("wpd") )
                  CClass = DK_DL_CC_WP;
            else if ( (suffix.toLowerCase()).equals("wpg") )
                  CClass = DK_DL_CC_WP;
            else if ( (suffix.toLowerCase()).equals("wks") )
                  CClass = DK_DL_CC_WKS;
            else if ( (suffix.toLowerCase()).equals("bmp") )
                  CClass = DK_DL_CC_WINV3_BMP;
            else if ( (suffix.toLowerCase()).equals("mpg") )
                  CClass = DK_DL_CC_MPEG;
            else CClass = DK_DL_CC_ASCII;   // Add more ContentClasses here!!!                        
            DKBlobDL  axdo = new DKBlobDL(dsDL);
            axdo.setContentClass(CClass);
            axdo.setAffiliatedType(DK_BASE);
            axdo.setContentFromClientFile(xdoPathNames[i]);
            DKPidXDODL apid = new DKPidXDODL();
            apid.setPartId(partId);
            axdo.setPidObject(apid);
            parts.addElement(axdo);
            partId = partId + 1;
            log.write("creat xdo part: " + xdoPathNames[i]);
            log.newLine();
          }
         }
         
         //-------------------------------------
         // create the notepart file
         //-------------------------------------
         BufferedWriter note = new BufferedWriter(
                 new FileWriter(noteName));
         String tmp;
         int k, m;
         if (smiFlag)
	     {
           m = smiName.lastIndexOf("\\");
           k = smiName.lastIndexOf("/");
           if ((k == -1) && (m == -1))
           { 
              tmp = smiName + ":1";                   
           }
           else if (k == -1)
              tmp = (smiName.substring(m + 1)) + ":1";
           else 
              tmp = (smiName.substring(k + 1)) + ":1";
           note.write(tmp, 0, tmp.length());
           note.newLine();
           log.write(tmp);
           log.newLine();
	     }
         if (xdoFlag)
	     {
           partId = 3;
           for (i = 0; i < xdoCnt; i++)
	       {
             m = xdoPathNames[i].lastIndexOf("\\");
             k = xdoPathNames[i].lastIndexOf("/");
             if ((k == -1) && (m == -1))
             { 
                tmp = xdoPathNames[i] + ":" + String.valueOf(partId);                   
             }
             else if (k == -1)
                tmp = (xdoPathNames[i].substring(m + 1)) + ":"
                      + String.valueOf(partId);
             else 
                 tmp = (xdoPathNames[i].substring(k + 1)) + ":" 
                      + String.valueOf(partId);
             note.write(tmp, 0, tmp.length());
             note.newLine();
             partId = partId + 1;
             log.write(tmp);
             log.newLine();
	       }
         }
         note.close();

         //*** create notepart xdo
         DKBlobDL  notexdo = new DKBlobDL(dsDL);
         notexdo.setContentClass(DK_DL_CC_ASCII);
         notexdo.setAffiliatedType(DK_DL_NOTE);
         notexdo.setContentFromClientFile(noteName);
         DKPidXDODL apid = new DKPidXDODL();
         apid.setPartId(notePartId);
         notexdo.setPidObject(apid);
         parts.addElement(notexdo);

         //*** add ddo
         dataId = ddo.addData(DKPARTS);
         ddo.addDataProperty(dataId, DK_CM_PROPERTY_TYPE,
              new Short(DK_CM_DATAITEM_TYPE_COLLECTION_XDO));
         ddo.setData(dataId, parts);
/**** no text search by comment out following line ****/
         DKSearchEngineInfoDL srch = (DKSearchEngineInfoDL)smixdo.getExtension("DKSearchEngineInfoDL");
         log.write("smi searchEngine=" +  srch.getSearchEngine());
         log.newLine();
         log.write("smi searchIndex=" +  srch.getSearchIndex());
         log.newLine();
         log.write("smi searchInfo=" +  srch.getSearchInfo());
         log.newLine();
/**** end of comment out for no text search ***********/
         log.write("about to call ddo.add().... ");
         log.newLine();
         ddo.add();
         log.write("Add success with itemId = " + ddo.getPidObject().getPrimaryId());
         log.newLine();
         log.write("===================================================");
         log.newLine();
         //*** if toc, save created itemId for DLSEARCH_relation value of latter items
         if (tocFlag)
	     {
           tocItemId = notexdo.getPidObject().getPrimaryId();
           tocFlag = false;
	     }
         if (reset)
	     {
           tocItemId = "";
           reset = false;
	     }
         log.write("DLSEARCH_relation = " + tocItemId);
         log.newLine();
         parts.removeAllElements();

         //*** reset the flags
         smiFlag = xdoFlag = false;
         xdoCnt = 0; 
         //*** reset attrValue
         for (i = 0; i < attrCnt; i++)
         {
           if (i != docIndx) 
             attrValue[i] = null;
         }  
 
       } //if is "end"

      }  //end of read: while
      in.close();
/**** no text search by comment out following line ****/
      System.out.println("about to call invokeSearchEngine .... ");
      log.write("about to call invokeSearchEngine(searchEngine,searchIndex).... ");
      log.newLine();
      dsDL.invokeSearchEngine(searchEngine, searchIndex);
      log.write("invokeSearchEngine(searchEngine,searchIndex) done. ");
      log.newLine();
      System.out.println("invokeSearchEngine(searchEngine,searchIndex) done. ");
/**** end of comment out for no text search ***********/
      log.close();
      System.out.println("Finish loading " + args[0] );
      dsDL.disconnect();
      System.out.println("datastore disconnected. Done....");
      dsDL.destroy();

    } // end of try
    catch (DKException exc)
    {
     try {
        if (dsDL != null)
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println("Exception name " + exc.name());
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (IOException exc)
    {
     try {
        if (dsDL != null)
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println("Exception name " );
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    catch (Exception exc)
    {
     try {
        if (dsDL != null)
         dsDL.destroy();
     }
     catch (Exception e)
     {
      e.printStackTrace();
     }      
     System.out.println("Exception message " + exc.getMessage());
     exc.printStackTrace();
    }
    if (reSet) Locale.setDefault(origLocale);  //re-set back to original Locale
  }

  //**********************************************************
  // translateAttrTypeToDataItemType - change attribute type to
  //   ddo dataitem type 
  //**********************************************************
  public static short translateAttrTypeToDataItemType(short attrType)
  {
   short rAttrType = DK_CM_DATAITEM_TYPE_UNDEFINED;
   switch(attrType)
   {
    case DK_CM_VSTRING:
    case DK_CM_FSTRING:
    case DK_CM_DECIMAL:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_STRING;
    }
    break;
    case DK_CM_SHORT:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_SHORT;
    }
    break;
    case DK_CM_LONG:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_LONG;
    }
    break;
    case DK_CM_DATE:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_DATE;
    }
    break;
    case DK_CM_TIME:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIME;
    }
    break;
    case DK_CM_TIMESTAMP:
    {
     rAttrType = DK_CM_DATAITEM_TYPE_TIMESTAMP;
    }
    break;
   }
   return rAttrType;
  }

  //**************************************************************
  // verifySetUp - verify the TextMiner index "TMINDEX" is defined
  //**************************************************************
  public static void  verifySetUp(String userId, String passWord)
            throws DKException, IOException, Exception
  {
     String searchServerTS, searchIndexTS ;
     int  n;	
     n = searchIndex.indexOf("-");
     searchServerTS = searchIndex.substring(0,n);
     searchIndexTS = searchIndex.substring(n + 1);
     System.out.println("verifying: " + searchIndex );
     DKDatastoreTS dsTS = new DKDatastoreTS();
     DKIndexInfoTS pIndxInfo = null;
     System.out.println("connecting to datastoreTS... " );
     dsTS.connect(searchServerTS, "", ' '); 
     System.out.println("datastoreTS connected with server: " + searchServerTS );
     pIndxInfo = dsTS.getIndexInformation(searchIndexTS);
     System.out.println("Index Name = " + pIndxInfo.getIndexName());
     dsTS.disconnect();
     System.out.println("datastoreTS disconnected." );
     dsTS.destroy();
     return;
  }

}

