/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Filename       : TADLServersFed.java
 * Description    : test add, delete, list servers
 *
 */

import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;
import java.sql.*;
import java.util.*;

public class TADLServersFed implements DKConstantFed 
{
   // Main method
   public static void main(String argv[])
   {
    
      // Indicate this is a Console Application on Windows NT/95 for error
      // message display in dos console code page. If this was a Windows
      // GUI (Graphic User Interface) Application this would not be necessary
      // because DK_CM_SS_WINDOWS is the default.
      DKEnvironment.setSubSystem(DK_CM_SS_CONSOLE);

    //-------------------------------------------------------------- 
    // Checking for input parameters
    //-------------------------------------------------------------- 
      if (argv.length < 4) {
        System.out.println("Usage Syntax: ");
        System.out.println("\t java TADLServersFed <cmbdb> <userid> <pw> <add/del/list> ");
        return;
      }
      
      String  libSrv = argv[0];
      String  userid = argv[1];
      String  pw     = argv[2];
      String  cmd    = argv[3];

      try {
         dkDatastore dsFed = new DKDatastoreFed();
         System.out.println(" *** connecting to datastore");

         dsFed.connect(libSrv, userid, pw,"");
         System.out.println(" *** datastore connected"); 
         String userName = dsFed.userName();
         String dsName = dsFed.datastoreName();
         System.out.println("     user " + userName + " dsName " + dsName); 

         // get the Def and Admin
         dkDatastoreDef  defFed = dsFed.datastoreDef();
         DKDatastoreAdminFed    adm = (DKDatastoreAdminFed) defFed.datastoreAdmin();

         //=======================================================================================================
         //    Define your own Library server here.
         //       a) Create new server map def fed object
         //       b) Set name to one of the available Library Server
         //       c) Set remote host name
         //       d) The rest is optional.
         
         // Map LSMUFASA servers
         DKServerMapDefFed mufasa = new DKServerMapDefFed();
                           mufasa.setName("LSMUFASA");         // DL Library Server Name
                           mufasa.setServerType("DL");         // Library server type
                           mufasa.setConnectString("");
                           mufasa.setDescription("LSMUFASA : DL Library Server at Mufasa (AIX)");  // LS Description
                           mufasa.setInitializationString("DSNAME=LSMUFASA;");
                           mufasa.setJavaClass("com.ibm.mm.sdk.cs.DKDatastoreDL");
                           mufasa.setCppClass("DKDatastoreDL");
                           mufasa.setAXClass("DXDatastoreDL"); 
                                        
                           java.util.Date d = new java.util.Date();
                           DKTimestamp ts = new DKTimestamp(d.getTime());
                           mufasa.setRefreshTime(ts);
      
      
         // Map LSCHIUN servers
         DKServerMapDefFed talisman2 = new DKServerMapDefFed();
                           talisman2.setName("LSCHIUN");          // DL Library Server Name
                           talisman2.setServerType("DL");         // Library Server Type
                           talisman2.setConnectString("");
                           talisman2.setDescription("LSCHIUN : DL Library Server at talisman2 (NT) ");   // LS Description
                           talisman2.setInitializationString("DSNAME=LSCHIUN;");
                           talisman2.setJavaClass("com.ibm.mm.sdk.cs.DKDatastoreDL");
                           talisman2.setCppClass("DKDatastoreDL");
                           talisman2.setAXClass("DXDatastoreDL"); 
                           d = new java.util.Date();
                           ts = new DKTimestamp(d.getTime());
                           talisman2.setRefreshTime(ts);
         
         // Map TMMUF server        
         DKServerMapDefFed tmmuf = new DKServerMapDefFed();
                           tmmuf.setName("TMMUF");
                           tmmuf.setServerType("TS");
                           tmmuf.setConnectString("");
                           tmmuf.setDescription("TMMUF : TM Text Search Server at Mufasa (AIX)");
                           tmmuf.setInitializationString("DSNAME=TMMUF;");
                           tmmuf.setJavaClass("com.ibm.mm.sdk.cs.DKDatastoreTS");
                           tmmuf.setCppClass("DKDatastoreTS");
                           tmmuf.setAXClass("DXDatastoreTS"); 
                           d = new java.util.Date();
                           ts = new DKTimestamp(d.getTime());
                           tmmuf.setRefreshTime(ts);
                           
      
         // TODO: Define your LS server here.
         // Server type available are "DL", "IP", "V4"
      
         //=======================================================================================================
         // Add or delete define server here.
         //    NOTE: If create a new server object above, need to add code to add or delete server object here.
      
         try {
            if (cmd.equalsIgnoreCase("add")) {
               // Add LSMUFASA to the federated database
               System.out.println("Add Server: mufasa, LSMUFASA - DL Library Server on AIX ");
               adm.addServer(mufasa);

               // Add LSCHIUN to the federated database
               System.out.println("Add Server: talisman2, LSCHIUN - DL Library Server on NT ");
               adm.addServer(talisman2);

               // Add TMMUF to the federated database
               System.out.println("Add Server: mufasa, TMMUF - TM Text Search Server on AIX ");
               adm.addServer(tmmuf);
               adm.assocSearchServer("LSMUFASA", "TMMUF");
               // TODO: Add your LS server here
            }
            else if (cmd.equalsIgnoreCase("del")) {
               // Delete LSMUFASA from the federated database
               System.out.println("Delete Server: mufasa, LSMUFASA - DL Library Server on AIX ");
               adm.removeServer(mufasa.getName());

               // Delete LSCHIUN from the federated database
               System.out.println("Delete Server: talisman2, LSCHIUN - DL Library Server on NT ");
               adm.removeServer(talisman2.getName());

               // Delete TMMUF from the federated database
	       System.out.println("Delete Server: mufasa, TMMUF - TM Text Search Server on AIX ");
               adm.removeServer(tmmuf.getName());
               
               // TODO: Delete your LS server here

            }
         }
         catch (DKException exc) // Check for adding duplicate server.
         {
            System.out.println("Exception name " + exc.name());
            System.out.println("Exception message " + exc.getMessage());
            exc.printStackTrace();
         }
        
         //=======================================================================================================
         // list back-end servers

         String servers[] = adm.listServerNames();
         System.out.println("List of servers in federated database");
         for (int index = 0; index < servers.length; index++)
            System.out.println("Server [" + index + "] = " + servers[index].trim());
      
         System.out.println("\nDetail servers description");
         // retrieve the definition of the servers
         DKServerMapDefFed serverdat;
         for (int index = 0; index < servers.length; index ++) {
            serverdat = adm.getServer(servers[index]);
            System.out.println("\n DKServerMapDefFed = " + serverdat.getName().trim());
            System.out.println("          name: " + serverdat.getName().trim());
            System.out.println("          type: " + serverdat.getServerType());
            System.out.println("   Connect Str: " + serverdat.getConnectString());
            System.out.println("         desc : " + serverdat.getDescription());
            System.out.println("      refTime : " + serverdat.getRefreshTime());
         }

         // disconnects
         System.out.println("\n\n *** disconnect from the datastore Fed ...");
         dsFed.disconnect();
         System.out.println(" *** datastore Fed disconnected"); 
         System.out.println("Finished");
      }
      catch (DKException exc)
      {
         System.out.println("Exception name " + exc.name());
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
      catch (Exception exc)
      {
         System.out.println("Exception message " + exc.getMessage());
         exc.printStackTrace();
      }
   }
}
