/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

NOTE:  When retrieving Item Type, Component Type, or any Entity Definitions, if it 
       does not exist, null is returned.  Usage of the null object will result in 
       a NullPointerException.

Listing Attribute Options
    When listing attributes & Attribute Groups of Item or Component Type Definitions,
    options may be specified to obtain resource, system, and user-defined attributes.
    
    ------------------------------------------------------------------------------                                  
    Option                        Description
    ----------------------------  ------------------------------------------------
    DK_ICM_ALL_ATTR               List attributes that match all the options below 
    DK_ICM_USER_ATTR (Default)    List user defined attributes (Default)
    DK_ICM_RESOURCE_ATTR          List resource manager attributes 
    DK_ICM_SYSTEM_ATTR            List system-defined attributes 
    DK_ICM_USER_ATTR_NO_RESOURCE  List user defined attributes without resource 
                                  manager attributes 
    ------------------------------------------------------------------------------                                  
    Note: Constants defined in com.ibm.mm.sdk.common.DKConstantICM       

       
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SItemTypeRetrievalICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Listing All Item Types Names & Definitions in System, Lisitng Attributes of
 *                    an Item Type Definition, Retrieving Individual Item Type Definitions, and
 *                    Printing Item Type Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Listing All Item Type Names Defined in the System
 *                    Listing All Item Type Definitions Defined in the System
 *                    Listing Attribute Options
 *                    Listing All Attribute Names of an Item or Component Type
 *                    Listing All Attribute Group Names of an Item or Component Type
 *                    Listing All Attribute Definitions of an Item or Component Type
 *                    Listing All Sub-Entities / Children / Multivalue Attributes of an Item Type or Child Component.
 *                    Retrieving an Individual Item Type Definition
 *                    Printing an Item Type or Component Type Definition
 *                    Sequantial Collections
 *                    Iterators
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SItemTypeRetrievalICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: Item Types & Component Types must be defined.  If not, please run Sample 
 *                    SItemTypeCreationICM first.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 ************************************************************************************************/
public class SItemTypeRetrievalICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SItemTypeRetrievalICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SItemTypeRetrievalICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------------
            // Accessing / Listing All Item Type Names Defined in System
            //-------------------------------------------------------------------
            // Access a list of Item Type Names defined in the Datastore Definition
            // and print them for the user.
            System.out.println("Accessing / Listing All Item Type Names Defined in System...");

                String itemTypeName[] = dsICM.listEntityNames();  // Get String Array of all Entity Types.

                // Print Names to Commandline
                System.out.println("\nItem Type Names in System:");

                    for(int i=0; i < itemTypeName.length; i++)
                        System.out.println("     - "+ itemTypeName[i]);

                System.out.println("");

            System.out.println("Accessed / Listed All Item Type Names Defined in System.");

            //-------------------------------------------------------------------
            // Accessing / Listing All Item Type Definitions Defined in System
            //-------------------------------------------------------------------
            // Access a list of all Item Type Definitions defined in the Datastore Definition
            // and print them for the user.
            //
            // NOTE:  For a complete listing all properties and attributes everything defined
            //        in the sytem, please refer to the section covered later in this sample
            //        named "Printing all Definition Informaiton in the System".
            System.out.println("Accessing / Listing All Item Type Definitions Defined in System...");

                DKSequentialCollection itemTypeColl = (DKSequentialCollection) dsICM.listEntities();  // Get a collection containing all Item Type Definitions.

                // Accessing each and printing the name.  For printing all information, please
                // refer to later sections in this sample.
                System.out.println("\nItem Type Names in System:");

                    dkIterator iter = itemTypeColl.createIterator(); // Create an iterator to iterate through the collection
                    while(iter.more()){ // while there are still items in the list, continue
                        DKItemTypeDefICM itemType = (DKItemTypeDefICM) iter.next(); // Move pointer to next element & return that object.
                        System.out.println("     - "+ itemType.getName() +":  "+ itemType.getDescription());
                    }

                System.out.println("");

            System.out.println("Accessed / Listed All Item Type Definitions Defined in System.");

            //-------------------------------------------------------------------
            // Retrieving an Individual Item Type Definition
            //-------------------------------------------------------------------
            // Retrieve a single Item Type Definition given the Item Type Name.
            //
            // NOTE:  When retrieving Item Type, Component Type, or any Entity Definitions, if it 
            //        does not exist, null is returned.  Usage of the null object will result in 
            //        a NullPointerException.
            System.out.println("Retrieving Item Type Definition of 'S_adjustrReport'...");

                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the datastore definition object from the connected datastore.
                DKItemTypeDefICM itemType = (DKItemTypeDefICM) dsDefICM.retrieveEntity("S_adjustrReport"); // Retrieve the Item Type Definition.

            System.out.println("Retrieved Item Type Definition of 'S_adjustrReport'.");

            //-------------------------------------------------------------------
            // Listing All Attributes Names of a Component Type
            //-------------------------------------------------------------------
            // Access a list of Attribute Names for a particular Item Type and print
            // it to the commandline.  Note that Item Type extends Component Type.
            //
            // The list function can also take options to display system-defined
            // attributes as well.  By default, only user-defined attributes are
            // listed.  More information can be found at the top of this sample.
            //
            // The DKComponentTypeDefICM object can be used to retrieve attributes 
            // defined for this Component Type.
            System.out.println("Accessing & Listing All Attribute Names Used in Item Type 'S_adjustrReport'...");

                String attrs[] = itemType.listAttrNames();  // Get String Array of all Attribute Names for this Component / Item Type.

                // Print Names to Commandline
                System.out.println("\nAttributes of Item Type 'S_adjustrReport':");

                    for(int i=0; i < attrs.length; i++)
                        System.out.println("     - "+ attrs[i]);

                System.out.println("");

            System.out.println("Accessed & Listed All Attribute Names Used in Item Type 'S_adjustrReport'.");

            //------------------------------------------------------------------------------
            // Listing All Attribute Group & Reference Attribute Names of an Component Type
            //------------------------------------------------------------------------------
            // Access a list of Attribute Group and Reference Attribute Names for a 
            // particular Item Type and print it to the commandline.  Note that Item Type 
            // extends Component Type
            //
            // As in SReferenceAttrDefCreationICM, the Reference Attribute is handled in 
            // definitions as an Attribute Group.
            //
            // The DKComponentTypeDefICM object can be used to retrieve attribute groups
            // defined for this Component Type.
            System.out.println("Accessing & Listing All Attribute Group & Reference Attribute Names Used in Item Type 'S_adjustrReport'...");

                String attrGroups[] = itemType.listAttrGroupNames();  // Get String Array of all Attribute Group Names for this Component / Item Type.

                // Print Names to Commandline
                System.out.println("\nAttribute Groups & Reference Attributes of 'S_adjustrReport':");

                    for(int i=0; i < attrGroups.length; i++)
                        System.out.println("     - "+ attrGroups[i]);

                System.out.println("");

            System.out.println("Accessed & Listed All Attribute Group Names Used in Item Type 'S_adjustrReport'.");

            //-------------------------------------------------------------------
            // Listing All Attributes Definitions of an Item or Component Type
            //-------------------------------------------------------------------
            // Access a list of Attribute Definitions for a particular Item or 
            // Component Type.  This particular list will include all Attributes,
            // including those belonging to Attribute Groups defined for this 
            // Item or Component Type.  Refer to the Programming Reference documentation
            // for ather methods tat are available.
            //
            // Note that with the default list attribute options, only user-defined
            // attributes will be listed.  As covered in SReferenceAttrDefICM, Reference 
            // Attributes are actually an Attribute Group with system-defined Attributes,
            // which means that they will not appear in the list below.
            //
            // For more information on listing and printing individual attribute properties,
            // pease refer to Sample SAttributeDefinitionRetrievalICM.
            System.out.println("Accessing & Listing All Attribute Definitions Used in Item Type 'S_adjustrReport'...");

                DKSequentialCollection attrColl = (DKSequentialCollection) itemType.listAllAttributes();  // Get a collection containing all Attribute Definitions for the Item or Component Type.

                // Accessing each and printing the name & description.  For printing all information, please
                // refer to later sections in this sample.
                System.out.println("\nAttributes of Item Type 'S_adjustrReport':");

                    iter = attrColl.createIterator(); // Create an iterator to iterate through the collection
                    while(iter.more()){ // while there are still items in the list, continue
                        DKAttrDefICM attr = (DKAttrDefICM) iter.next(); // Move pointer to next element & return that object.
                        System.out.println("     - "+ attr.getName() +":  "+ attr.getDescription());
                    }

                System.out.println("");

            System.out.println("Accessed & Listed All Attribute Definitions Used in Item Type 'S_adjustrReport.");

            //-------------------------------------------------------------------
            // Retrieving the Insurance Policy Item Type Definition
            //-------------------------------------------------------------------
            System.out.println("Retrieving Item Type Definition of 'S_insurePolicy'...");

                itemType = (DKItemTypeDefICM) dsDefICM.retrieveEntity("S_insurePolicy"); // Retrieve the Item Type Definition.

            System.out.println("Retrieved Item Type Definition of 'S_insurePolicy'.");

            //-------------------------------------------------------------------
            // Listing All Sub-Entities / Children / Multivalue Attributes
            //-------------------------------------------------------------------
            // Listing works for both Item Types and Child Component.  This method
            // can be applied to the children to obtain sub-entities of them.
            System.out.println("Accessing / Listing Sub-Entities / Multivalue Attributes / Children...");
                
                DKSequentialCollection compTypes = (DKSequentialCollection) itemType.listSubEntities(); // Obtain the collection of child component types.

                // Accessing each and printing the name and description. 
                System.out.println("\nSub Entities of 'S_insurePolicy':");

                    iter = compTypes.createIterator(); // Create an iterator to iterate through the collection
                    while(iter.more()){ // while there are still items in the list, continue
                        DKComponentTypeDefICM compType = (DKComponentTypeDefICM) iter.next(); // Move pointer to next element & return that object.
                        System.out.println("     - "+ compType.getName() +":  "+ compType.getDescription());
                    }

                    // Since Reference Attributes consist of system-defined attributes and the default list 
                    // option was used above, references will be added on the end, using attribute groups.
                    DKSequentialCollection attrGroupDefs = (DKSequentialCollection) itemType.listAttrGroups();  // Get a collection containing all Attribute Group Definitions for the Item / Component Type.
                    iter = attrGroupDefs.createIterator();                                                         // Create an iterator to iterate through the collection
                    while(iter.more()){                                                 // while there are still items in the list, continue
                        DKAttrGroupDefICM attrGroup = (DKAttrGroupDefICM) iter.next();  // Move pointer to next element & return that object.
                        if(attrGroup.isReferenceAttributeGroup())                       // If it is a reference attribute, print it, otherwise just skip
                            System.out.println("     - "+ attrGroup.getName() +": "+ attrGroup.getDescription());
                    }

                System.out.println("");

            System.out.println("Accessed / Listed Sub-Entities / Multivalue Attributes / Children.");

            //-------------------------------------------------------------------
            // Printing an Item Type Definition
            //-------------------------------------------------------------------
            // Item Type Definition already retrieve from the section above.
            //
            // Uses wrapper function defined at end of sample file.
            System.out.println("Printing the Item Type Definition of 'S_insurePolicy'...");

                printItemType(itemType); // Uses wrapper funciton defined at end of sample file.

            System.out.println("Printed the Item Type Definition of 'S_insurePolicy'.");

            //-------------------------------------------------------------------
            // Printing Adjuster Report with Attribute Group & Reference Attribute
            //-------------------------------------------------------------------
            System.out.println("Printing the Item Type Definition of 'S_adjustrReport'...");

                printItemType(dsICM,"S_adjustrReport"); // Uses wrapper function defined at end of sample file.

            System.out.println("Printing the Item Type Definition of 'S_adjustrReprot'...");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

   /**
    * Printing All Item Type Names Defined in System.  Accesses a list
    * of Item Type Definitions defined in the Datastore Definition and
    * prints the name and descripiton them to the commandline.
    * @param dsICM  Conncted DKDatastoreICM object.
    **/
    public static void printAllItemTypeNames(DKDatastoreICM dsICM) throws DKException, Exception{

        DKSequentialCollection itemTypeColl = (DKSequentialCollection) dsICM.listEntities();  // Get a collection containing all Item Type Definitions.

        // Accessing each and printing the name & description.  
        System.out.println("\nItem Type Names in System:");

            dkIterator iter = itemTypeColl.createIterator(); // Create an iterator to iterate through the collection
            while(iter.more()){ // while there are still items in the list, continue
                DKItemTypeDefICM itemType = (DKItemTypeDefICM) iter.next(); // Move pointer to next element & return that object.
                System.out.println("     - "+ itemType.getName() +": "+ itemType.getDescription());
            }

        System.out.println("");
    }

   /**
    * Printing All Attributes Defined in particular Item Type.  Accesses a 
    * list of Attribute Definitions for a particular Item Type, and prints 
    * the name and description to the commandline.
    * @param dsICM         Conncted DKDatastoreICM object.
    * @param itemTypeName  Name of the Item Type for which Print the Attributes.
    **/
    public static void printAllAttributesOfItemType(DKDatastoreICM dsICM, String itemTypeName) throws DKException, Exception{

        DKSequentialCollection attrColl = (DKSequentialCollection) dsICM.listEntityAttrs(itemTypeName);  // Get a collection containing all Attribute Definitions for the Item Type.

        // Accessing each and printing the name & description.  For printing all information, please
        // refer to later sections in this sample.
        System.out.println("\nAttributes of Item Type '"+itemTypeName+"':");

            dkIterator iter = attrColl.createIterator(); // Create an iterator to iterate through the collection
            while(iter.more()){ // while there are still items in the list, continue
                DKAttrDefICM attr = (DKAttrDefICM) iter.next(); // Move pointer to next element & return that object.
                System.out.println("     - "+ attr.getName() +": "+ attr.getDescription());
            }

        System.out.println("");
    }

   /**
    * Printing All Sub-Entities / Multivalue Attributes / Child Components defined in the 
    * particular Item Type.  Accesses a list of SubEntity Definitions for a particular Item Type,
    * and prints the name and description to the commandline.
    * @param dsICM         Conncted DKDatastoreICM object.
    * @param itemTypeDef  Name of the Item Type for which Print the SubEntities.
    **/
    public static void printAllAttributesOfItemType(DKDatastoreICM dsICM, DKItemTypeDefICM itemTypeDef) throws DKException, Exception{

        DKSequentialCollection compTypes = (DKSequentialCollection) itemTypeDef.listSubEntities(); // Obtain the collection of child component types.

        // Accessing each and printing the name and description. 
        System.out.println("\nSub Entities of '"+itemTypeDef.getName()+"':");

            dkIterator iter = compTypes.createIterator(); // Create an iterator to iterate through the collection
            while(iter.more()){ // while there are still items in the list, continue
                DKComponentTypeDefICM compType = (DKComponentTypeDefICM) iter.next(); // Move pointer to next element & return that object.
                System.out.println("     - "+ compType.getName() +": "+ compType.getDescription());
            }

        System.out.println("");
    }
    
   /**
    * Print an Item Type Definition.
    * @param itemTypeDef  The DKItemTypeDefICM object to print.
    **/
    public static void printItemType(DKItemTypeDefICM itemTypeDef) throws DKException, Exception{

        System.out.println("--------------------------------------------------");
        System.out.println("               Item Type Definition");
        System.out.println("--------------------------------------------------");
        System.out.println("              Name:  "+ itemTypeDef.getName());
        System.out.println("       Description:  "+ itemTypeDef.getDescription());
        System.out.println("         Entity ID:  "+ itemTypeDef.getIntId());
        switch(itemTypeDef.getClassification()){
            case DKConstantICM.DK_ICM_ITEMTYPE_CLASS_ITEM:
                System.out.println("    Classification:  Item");
                break;
            case DKConstantICM.DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM:
                System.out.println("    Classification:  Resource Item");
                break;
            case DKConstantICM.DK_ICM_ITEMTYPE_CLASS_DOC_MODEL:
                System.out.println("    Classification:  ICM Document Model Item");
                break;
            case DKConstantICM.DK_ICM_ITEMTYPE_CLASS_DOC_PART:
                System.out.println("    Classification:  ICM Document Model Part");
                break;
            default:
                System.out.println("    Classification:  Other ("+itemTypeDef.getClassification()+")");
                break;
        }//end switch on classification
        System.out.println("         XDO Class:  "+ itemTypeDef.getXDOClassID() +" - "+ itemTypeDef.getXDOClassName());
        System.out.println(" Default Coll Code:  "+ itemTypeDef.getDefaultCollCode());
        System.out.println("   Default RM Code:  "+ itemTypeDef.getDefaultRMCode());
        // Dispay Version Control Information
        switch(itemTypeDef.getVersionControl()){
            case DKConstantICM.DK_ICM_VERSION_CONTROL_NEVER:
                System.out.println("   Version Control:  Off / Never");
                break;
            case DKConstantICM.DK_ICM_VERSION_CONTROL_ALWAYS:
                System.out.println("   Version Control:  Always / Automatic");
                break;
            case DKConstantICM.DK_ICM_VERSION_CONTROL_BY_APPLICATION:
                System.out.println("   Version Control:  By Application");
                break;
            default:
                System.out.println("   Version Control:  Unknown ("+itemTypeDef.getVersionControl()+")");
                break;
        }//end switch on version control
        switch(itemTypeDef.getVersioningType()){
            case DKConstantICM.DK_ICM_ITEM_VERSIONING_FULL:
                System.out.println("   Versioning Type:  Full");
                break;
            case DKConstantICM.DK_ICM_ITEM_VERSIONING_OPTIMIZED:
                System.out.println("   Versioning Type:  Optimized / Only Resource Content");
                break;
            default:
                System.out.println("   Versioning Type:  Unknown ("+itemTypeDef.getVersioningType()+")");
            break;
        }//end switch on versioning type
        if(itemTypeDef.getDefaultItemRetention()==0)
            System.out.println(" Default Retention:  never expires");
        else
            System.out.println(" Default Retention:  "+ itemTypeDef.getDefaultItemRetention());
        switch(itemTypeDef.getDefaultRetentionUnit()){
            case 3:
                System.out.println(" Default Ret. Unit:  day");
                break;
            case 2:
                System.out.println(" Default Ret. Unit:  week");
                break;
            case 1:
                System.out.println(" Default Ret. Unit:  month");
                break;
            case 0:
                System.out.println(" Default Ret. Unit:  year");
                break;
            default:
                System.out.println(" Default Ret. Unit:  other ("+itemTypeDef.getDefaultRetentionUnit()+")");
                break;
        }
        // List Sub Entities 
        System.out.println("       Sub-Entities:  -");
        DKSequentialCollection compTypes = (DKSequentialCollection) itemTypeDef.listSubEntities(); // Obtain the collection of child component types.
        // Accessing each and printing the name and description. 
        dkIterator iter = compTypes.createIterator(); // Create an iterator to iterate through the collection
        while(iter.more()){ // while there are still items in the list, continue
            DKComponentTypeDefICM compType = (DKComponentTypeDefICM) iter.next(); // Move pointer to next element & return that object.
            System.out.println("                     "+ compType.getName() +": "+ compType.getDescription());
        }
        // List Attributes
        System.out.println("         Attributes:  -");
        DKSequentialCollection attrColl = (DKSequentialCollection) itemTypeDef.listAllAttributes();  // Get a collection containing all Attribute Definitions for the Item Type.
        iter = attrColl.createIterator(); // Create an iterator to iterate through the collection
        while(iter.more()){ // while there are still items in the list, continue
            DKAttrDefICM attr = (DKAttrDefICM) iter.next(); // Move pointer to next element & return that object.
            System.out.println("                     "+ attr.getName() +": "+ attr.getDescription());
        }
        // Since Reference Attributes consist of system-defined attributes and the default list 
        // option was used above, references will be added on the end, using attribute groups.
        DKSequentialCollection attrGroups = (DKSequentialCollection) itemTypeDef.listAttrGroups();  // Get a collection containing all Attribute Group Definitions for the Item / Component Type.
        iter = attrGroups.createIterator(); // Create an iterator to iterate through the collection
        while(iter.more()){ // while there are still items in the list, continue
            DKAttrGroupDefICM attrGroup = (DKAttrGroupDefICM) iter.next(); // Move pointer to next element & return that object.
            // If it is a reference attribute, print it, otherwise just skip
            if(attrGroup.isReferenceAttributeGroup())
                System.out.println("                     "+ attrGroup.getName() +": "+ attrGroup.getDescription());
        }
        System.out.println("---------------------------------------------------------\n");

        // Print Sub-Components
        DKSequentialCollection subCompTypes = (DKSequentialCollection) itemTypeDef.listSubEntities(); // Obtain the collection of child component types.
        iter = subCompTypes.createIterator(); // Create an iterator to iterate through the collection
        while(iter.more()){ // while there are still items in the list, continue
            DKComponentTypeDefICM subCompType = (DKComponentTypeDefICM) iter.next(); // Move pointer to next element & return that object.
            printComponentType(subCompType); // print the sub component
        }
    }
    
   /**
    * Print an Item Type Definition.  First retrieves the specified Item Type Definition, and
    * calls another wrapper function to print it.
    * @param dsICM         Conncted DKDatastoreICM object.
    * @param itemTypeName  Name of the Item Type to Print.
    **/
    public static void printItemType(DKDatastoreICM dsICM, String itemTypeName) throws DKException, Exception{

        DKDatastoreDefICM dsDefICM = new DKDatastoreDefICM(dsICM);  // Create a new Datastore Definition Object
        DKItemTypeDefICM itemType = (DKItemTypeDefICM) dsDefICM.retrieveEntity(itemTypeName); // Retrieve the Item Type Definition.
        printItemType(itemType); // Call the wrapper funciton to print it.
    }

   /**
    * Print an Component Type Definition.
    * @param compTypeDef  The DKComponentTypeDefICM object to print.
    **/
    public static void printComponentType(DKComponentTypeDefICM compTypeDef) throws DKException, Exception{

        System.out.println("-------------------------------------------------");
        System.out.println("            Component Type Definition");
        System.out.println("-------------------------------------------------");
        System.out.println("               Name: "+ compTypeDef.getName());
        System.out.println("        Description: "+ compTypeDef.getDescription());
        System.out.println("          Entity ID: "+ compTypeDef.getIntId());
        switch(compTypeDef.getDeleteRule()){
            case DKConstantICM.DK_ICM_DELETE_RULE_RESTRICT:
                System.out.println("        Delete Rule: Restrict (Default)");
                break;
            case DKConstantICM.DK_ICM_DELETE_RULE_CASCADE:
                System.out.println("        Delete Rule: Cascade");
                break;
            case DKConstantICM.DK_ICM_DELETE_RULE_SET_NULL:
                System.out.println("        Delete Rule: Set Null");
                break;
            case DKConstantICM.DK_ICM_DELETE_RULE_NO_ACTION:
                System.out.println("        Delete Rule: No Action");
                break;
            default:
                System.out.println("        Delete Rule: Other ("+compTypeDef.getDeleteRule()+")");
                break;
        }
        
        // List Sub Entities 
        System.out.println("       Sub-Entities:  -");
        DKSequentialCollection subCompTypes = (DKSequentialCollection) compTypeDef.listSubEntities(); // Obtain the collection of child component types.
        // Accessing each and printing the name and description. 
        dkIterator iter = subCompTypes.createIterator(); // Create an iterator to iterate through the collection
        while(iter.more()){ // while there are still items in the list, continue
            DKComponentTypeDefICM subCompType = (DKComponentTypeDefICM) iter.next(); // Move pointer to next element & return that object.
            System.out.println("                     "+ subCompType.getName() +": "+ subCompType.getDescription());
        }
        // List Attributes
        System.out.println("         Attributes:  -");
        DKSequentialCollection attrColl = (DKSequentialCollection) compTypeDef.listAllAttributes();  // Get a collection containing all Attribute Definitions for the Item / Component Type.
        iter = attrColl.createIterator(); // Create an iterator to iterate through the collection
        while(iter.more()){ // while there are still items in the list, continue
            DKAttrDefICM attr = (DKAttrDefICM) iter.next(); // Move pointer to next element & return that object.
            System.out.println("                     "+ attr.getName() +": "+ attr.getDescription());
        }
        // Since Reference Attributes consist of system-defined attributes and the default list 
        // option was used above, references will be added on the end, using attribute groups.
        DKSequentialCollection attrGroups = (DKSequentialCollection) compTypeDef.listAttrGroups();  // Get a collection containing all Attribute Group Definitions for the Item / Component Type.
        iter = attrGroups.createIterator(); // Create an iterator to iterate through the collection
        while(iter.more()){ // while there are still items in the list, continue
            DKAttrGroupDefICM attrGroup = (DKAttrGroupDefICM) iter.next(); // Move pointer to next element & return that object.
            // If it is a reference attribute, print it, otherwise just skip
            if(attrGroup.isReferenceAttributeGroup())
                System.out.println("                     "+ attrGroup.getName() +": "+ attrGroup.getDescription());
        }
        System.out.println("---------------------------------------------------------\n");

        // Print Sub-Components
        subCompTypes = (DKSequentialCollection) compTypeDef.listSubEntities(); // Obtain the collection of child component types.
        iter = subCompTypes.createIterator(); // Create an iterator to iterate through the collection
        while(iter.more()){ // while there are still items in the list, continue
            DKComponentTypeDefICM subCompType = (DKComponentTypeDefICM) iter.next(); // Move pointer to next element & return that object.
            printComponentType(subCompType); // print the sub component
        }
    }
    
   /**
    * Print a Component Type Definition.  First retrieves the specified Component Type Definition, and
    * calls another wrapper function to print it.
    * @param dsICM         Conncted DKDatastoreICM object.
    * @param compTypeName  Name of the Component Type to Print.
    **/
    public static void printComponentType(DKDatastoreICM dsICM, String compTypeName) throws DKException, Exception{

        DKDatastoreDefICM dsDefICM = new DKDatastoreDefICM(dsICM);  // Create a new Datastore Definition Object
        DKComponentTypeDefICM compType = (DKComponentTypeDefICM) dsDefICM.retrieveEntity(compTypeName); // Retrieve the Item Type Definition.
        printComponentType(compType); // Call the wrapper funciton to print it.
    }

   /**
    * Determine if the Entity exists.
    * @param dsICM       Conncted DKDatastoreICM object.
    * @param entityName  Name of the Entity.
    * @return  Returns true if it exists, false if not.
    */
	public static boolean doesEntityExist(DKDatastoreICM dsICM, String entityName) throws Exception {

        // First, we need acces to the Datastore Definition object.
        DKDatastoreDefICM dsDefICM = new DKDatastoreDefICM(dsICM); // Create a datastore definition object given the connected datastore.
        dkEntityDef entityTypeDef  = dsDefICM.retrieveEntity(entityName); // Retrieve the definition.
        if(entityTypeDef==null) // If null is returned, it does not exist.
            return(false);
        else
            return(true);
	}

}//end class SItemTypeRetrievalICM
              
