/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Import Item from XML (limited)
    Items may be imported from an XML file that follows the "ddo.dtd" that should be
    installed with IBM Information Integrator for Content.  The DKDDO.fromXML()
    function is used to Import from an XML file.  Please refer documentation of
    DKDDO.fromXML in the Application Programming Reference (Javadoc) for the latest,
    most current information.

    LIMITATION:  The DKDDO.toXML() and DKDDO.fromXML() interfaces demonstrated in
                 this sample cannot be used to export and item from one datastore
                 and import it into another if it references any other items through
                 links, folder contents, or reference attributes.  The import 
                 operation will create new items and will not update or overwrite
                 existing items or support multiple versions of an item.
                 
    NEW TOOLS:   New Sample Tools and Sample Tool API have been provided that will
                 perform more comprehensive import and export functionality, building
                 on top of the existing DKDDO.toXML() and DKDDO.fromXML() operations.
                 The new tools demonstrate functionality that work around limitations
                 and provide a new features.  
                 
                 -> Please refer to the Sample Import / Export Tools & API document,
                    TExportPackageICM.doc, for more information.

    The Data Model must already be defined using the OOAPI or by using the tools provided
    by the Samples.
    
    XML Source
        The XML Source is specified in a DKNVPair (Name-Value Pair).
        
        Name       Value           Description
        ---------  --------------  ---------------------------------------------------------------
        BUFFER     <String>        String Buffer Reference containing the XML contents in memory.           
        ..........................................................................................        
        FILE       <File Name>     String containing the name of the XML file.
        ..........................................................................................        
        URL        <URL Address>   String containing the complete URL address of the XML file.
        ..........................................................................................        
        STDIN      null            The contents of the XML file will be taken from the standard
                                   input (STDIN).
        ------------------------------------------------------------------------------------------

    Options
        
          Option    Constant              Description
        ----------  --------------------  --------------------------------------------------------
        Validation  DK_CM_XML_VALIDATION  Perform validation to ensure that file completely 
                                          satasfies the ddo.dtd specification.
        ------------------------------------------------------------------------------------------
        Note:  Constants defined in com.ibm.mm.sdk.common.DKConstant

Export Item to XML
    The DKDDO.toXML() function is used to export an Item to XML.

    XML Destination
        The XML Destination is specified in a DKNVPiar (Name-Value Pair).

        Name       Value             Description
        ---------  ----------------  -------------------------------------------------------------
        BUFFER     <String>          String Buffer Reference containing the XML contents in memory.
        ..........................................................................................        
        FILE       <Fully-Specified  String containing the fully-specified file path of the XML
                      File Path>     file.  Ex: "C:\\myFolder\\myFile.xml".  
                                     Note: Don't forget to use the "\\" escape character if needed.
        ..........................................................................................        
        URL        <URL Address>     String containing the complete URL address of the XML file.
        ..........................................................................................        
        STDOUT     null              The output XML is directed to the standard output (STDOUT)   
        ------------------------------------------------------------------------------------------

    DTD Path
        The fully-specified path of the ddo.dtd file must be specified (not including the actual
        ddo.dtd file name).  Ex:  "C:\\myDirectory".

    Options

           Option       Constant            Description
        -------------  -------------------  ------------------------------------------------------
         Embeded XDO   DK_CM_XDO_EMBED      For a Resource Item, embeds resource binary content 
                                            directly in XML file.
        ..........................................................................................        
        Reference XDO  DK_CM_XDO_REFERENCE  For a Resource Item, stores binary Resource Content
         As New File                        in a seperate file and references the seperate file
                                            in the XML file.
        ------------------------------------------------------------------------------------------
        Note:  Constants defined in com.ibm.mm.sdk.common.DKConstant

    Example Usage
        ddo.toXML(new DKNVPair("FILE","C:\\myFile.xml"),"C:\\directoryWithDTD",0);

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SItemXMLImportExportICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Import / Export an Item to/from XML.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Import an Item from XML
 *                    Export an Item to XML
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE: java SItemXMLImportExportICM <database> <userName> <password>
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Data Model must be defined.  If needed please run the following Samples  
 *                        - SAttributeDefinitionCreationICM
 *                        - SAttributeGroupDefCreationICM
 *                        - SReferenceAttrDefCreationICM
 *                        - SItemTypeCreationICM
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SItemCreationICM.java
 *                    ddo.dtd
 ************************************************************************************************/
public class SItemXMLImportExportICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 3) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SItemXMLImportExportICM <database> <userName> <password>" );
            System.out.println("  *** Some parameters not specified, using defaults..." );
            System.out.println("");
        } else {  // otherwise enough parameters were specified, use the first 3 for the 3 parameters.
            if (argv.length > 0) database = argv[0];
            if (argv.length > 1) userName = argv[1];
            if (argv.length > 2) password = argv[2];
        }//end else
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SItemXMLImportExportICM");
        System.out.println("-------------------------------------------");
        System.out.println(" Database: "+database);
        System.out.println(" UserName: "+userName);
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Create an Sample Items
            //-------------------------------------------------------------
            // create 3 Items, one of Item Property type Document, one Folder, & one Item.
            // Will use the function for creating a simple item with children, with the
            // attribute data set from the SItemCreationICM Sample.
            System.out.println("Creating & Saving Sample Items...");

	            DKDDO ddoDocument = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_DOCUMENT);
	            DKDDO ddoFolder   = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_FOLDER);   
	            DKDDO ddoItem     = SItemCreationICM.createSimpleItemWithChildren(dsICM, DKConstant.DK_CM_ITEM);     

	            ddoDocument.add();
	            ddoFolder.add();
	            ddoItem.add();

            System.out.println("Created & Saved Sample Items.");

            //-------------------------------------------------------------
            // Export to XML
            //-------------------------------------------------------------
            // Exporting the Items created above to XML files.
            System.out.println("Exporting Items to XML...");

                // Get User's Current Directory
                java.util.Properties props = System.getProperties();
                String userPath = props.getProperty("user.dir");
                
                ddoDocument.toXML(new DKNVPair("FILE",userPath+"/"+"SItemXMLImportExportICM_ExportedFile1.xml"),userPath,0);
                ddoFolder.toXML(new DKNVPair("FILE",userPath+"/"+"SItemXMLImportExportICM_ExportedFile2.xml"),userPath,0);
                ddoItem.toXML(new DKNVPair("FILE",userPath+"/"+"SItemXMLImportExportICM_ExportedFile3.xml"),userPath,0);

            System.out.println("Exported Items to XML.");

            //-------------------------------------------------------------
            // Delete Items
            //-------------------------------------------------------------
            // Items can not be deleted.  This is not necessarily required.
            // When importing XML files back, new PIDs will be assigned.
            System.out.println("Deleting Items...");

                ddoDocument.del();
                ddoFolder.del();
                ddoItem.del();

            System.out.println("Deleted Items.");

            //-------------------------------------------------------------
            // Import from XML
            //-------------------------------------------------------------
            System.out.println("Importing Items from XML...");
            
                DKDDO ddo1 = (DKDDO) DKDDO.fromXML(new DKNVPair("FILE","SItemXMLImportExportICM_ExportedFile1.xml"));
                DKDDO ddo2 = (DKDDO) DKDDO.fromXML(new DKNVPair("FILE","SItemXMLImportExportICM_ExportedFile2.xml"));
                DKDDO ddo3 = (DKDDO) DKDDO.fromXML(new DKNVPair("FILE","SItemXMLImportExportICM_ExportedFile3.xml"));
                        
            System.out.println("Imported Items from XML.");

            //-------------------------------------------------------------
            // Add to datastore
            //-------------------------------------------------------------
            System.out.println("Adding Items to datastore...");
            
                ddo1.setDatastore(dsICM);
                ddo2.setDatastore(dsICM);
                ddo3.setDatastore(dsICM);
            
                ddo1.add();
                ddo2.add();
                ddo3.add();

            System.out.println("Added Items to datastore.");

            //-------------------------------------------------------------
            // Clean Up
            //-------------------------------------------------------------
            System.out.println("Cleaning Up...");

                ddo1.del();
                ddo2.del();
                ddo3.del();
        
            System.out.println("Cleaned Up.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SItemXMLImportExportICM
              
