/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Default Resource Manager & SMS Collection
    The Item Type Definition contains information that specifies the default Resource 
    Manager and default SMS Collection that Items of that Item Type are to be stored
    if not otherwise specified.
    
    Please refer to the corresponding wrapper function at the bottom of this sample, 
    and to the Item Type Creation sample, SItemTypeCreationICM for more information
    and examples of how to set and modify these properties.

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SResourceMgrDefSetDefaultICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Updating Item Type Definitions for Default RM specifications.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Updating the Default RM for an Item Type
 *                    Updates All Resource Item Types in Samples
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SResourceMgrDefSetDefaultICM <Existing RM User-Defined Reference Name>
 *                                                      <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.  It should consist of all upper-case letters.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: java SResourceMgrDefSetDefaultICM RMDB
 *                    ex: java SResourceMgrDefSetDefaultICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM
 *                    SResourceMgrDefRetrievalICM
 ************************************************************************************************/
public class SResourceMgrDefSetDefaultICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        String rmUserDefinedReferenceName  = "NOT_SPECIFIED";
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argv.length < 4) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SResourceMgrDefSetDefaultICM <Existing RM User-Defined Reference Name>");
            System.out.println("                                    <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE: The specified reference name must be already defined, such as the");
            System.out.println("        default 'RMDB' specified during install.  If a new RM definition");
            System.out.println("        is to be configured, please refer to SResourceMgrDefCreationICM");
            System.out.println("        Sample.  It should consist of all upper-case letters.");
            System.out.println(""); 
            System.out.println("  NOTE: Defaults will be used if the database, username, & password are not");
            System.out.println("        specified.  All others are required.");
            System.out.println("");
            System.out.println("  ex: java SResourceMgrDefSetDefaultICM RMDB");
            System.out.println("  ex: java SResourceMgrDefSetDefaultICM MY_RM icmnlsdb icmadmin password");
        }
        if((argv.length < 1) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            System.out.println("======================================================");
            System.out.println("PROGRAM TERMINATED:  All RM Options Must Be Specified.");
            System.out.println("======================================================");
            return;
        } else if (argv.length != 4)
            System.out.println("  *** Some LS Database parameters not specified, using defaults..." );
        
        // Store commandline arguments.
        if (argv.length > 0) rmUserDefinedReferenceName = argv[0];

        // Store optional commandline arguments.
        if (argv.length > 1)  database = argv[1];
        if (argv.length > 2)  userName = argv[2];
        if (argv.length > 3)  password = argv[3];
        String ver = SConnectDisconnectICM.EIP_VERSION;
            
        System.out.println("=============================================");
        System.out.println("IBM Information Integrator for Content   v"+ver);
        System.out.println("Sample Program:  SResourceMgrDefSetDefaultICM");
        System.out.println("---------------------------------------------");
        System.out.println("           Database:  "+database);
        System.out.println("           UserName:  "+userName);
        System.out.println("  RM Reference Name:  "+rmUserDefinedReferenceName);       
        System.out.println("=============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //------------------------------------------------------------------
            // Set Default Resource Manager
            //------------------------------------------------------------------
            // Set the Default Resource Manager for Resource Item Types.
            System.out.println("Setting Default RM for Sample Resource Item Types...");

                // For each Resource Item Type created in SItemTypeCreationICM, each will be
                // updated to use by default the specified Resource Manager.
                setDefaultResourceManagerForItemType(dsICM, "S_autoClaim", rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_adjustrReport", rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_policeReport", rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_insurePolicy", rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_autoPhoto", rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_docModel", rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_lob", rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_text", rmUserDefinedReferenceName);
                setDefaultResourceManagerForItemType(dsICM, "S_image", rmUserDefinedReferenceName);

            System.out.println("Set Default RM Coll for all Sample Resource Item Types.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");
        
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main


    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // setDefaultResourceManagerForItemType
    //******************************************************************
   /**
    * Set the Default Resource Manager in which to store items of this Item Types.  Item Type
    * must already be defined.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  The Name of the Item Type to modify.
    * @param rmUserDefinedReferenceName  The Reference Name given to the Resource Manager that Items of this Item Type should be stored in.  It should consist of all upper-case letters.
    **/
    public static void setDefaultResourceManagerForItemType(DKDatastoreICM dsICM, String itemTypeName, String rmUserDefinedReferenceName) throws DKException, Exception{
        System.out.println("Setting the default Resource Manager for Item Type Named '"+itemTypeName+"' to Resource Manager Referred to as '"+rmUserDefinedReferenceName+"'.");
        
        // Make sure the RM Definition is setup.
        if(SResourceMgrDefRetrievalICM.isResourceManagerSetup(dsICM,rmUserDefinedReferenceName)==false) // Make sure that this RM is setup.
            throw new Exception("Resource Manager not already setup.  RM must be first setup.");

        short rmCode = SResourceMgrDefRetrievalICM.getResourceManagerIDCode(dsICM,rmUserDefinedReferenceName); // get the RM Code.

        setDefaultResourceManagerForItemType(dsICM,itemTypeName,rmCode); // set the item type to use this default rmCode.
    }

   /**
    * Set the Default Resource Manager in which to store items of this 
    * Item Types.  The Item Type must already be defined.  For more information on updating
    * Item Type Definitions, please consult SItemTypeUpdateICM Sample.
    * @param dsICM         Connected DKDatastoreICM object.
    * @param itemTypeName  The Name of the Item Type to modify.
    * @param rmCode        The ID Code in the datastore for this Resource Manager that Items of this Item Type should be stored in.
    **/
    public static void setDefaultResourceManagerForItemType(DKDatastoreICM dsICM, String itemTypeName, short rmCode) throws DKException, Exception{
        System.out.println("Setting the default Resource Manager for Item Type Named '"+itemTypeName+"' to Resource Manager with ID Code '"+rmCode+"'.");

            DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object

            DKItemTypeDefICM itemTypeDef = (DKItemTypeDefICM) dsDefICM.retrieveEntity(itemTypeName); // retrieve the Item Type Definition
            // If CM Doc Model Item Type, update Relations instead
            if(itemTypeDef.getClassification()==DKConstantICM.DK_ICM_ITEMTYPE_CLASS_DOC_MODEL){
                System.out.println("Setting Default for Item Type Relations...");
                dkCollection coll = dsDefICM.retrieveItemTypeRelations(itemTypeDef.getItemTypeId());
                dkIterator iter = coll.createIterator();
                while(iter.more()){
                    DKItemTypeRelationDefICM itemTypeRel = (DKItemTypeRelationDefICM) iter.next();
                    itemTypeRel.setDefaultRMCode(rmCode);
                    // Both RM & SMS must be set once one is specified.  Detect if SMS has not been set, 0,
                    // and set it in that case to '1' as default.
                    if(itemTypeRel.getDefaultCollCode()==0)
                        itemTypeRel.setDefaultCollCode((short)1); // Set the default SMS Colleciton to use for item of this Item Type.
                    // Item Type Relations also Require Prefetch Code, which can be set to Default CollCode.
                    if(itemTypeRel.getDefaultPrefetchCollCode()==0)
                        itemTypeRel.setDefaultPrefetchCollCode((short)1); // Set the default SMS Colleciton to use for item of this Item Type.

                    itemTypeRel.update();
                }
            } else { // Othwerwise update item type directly
                itemTypeDef.setDefaultRMCode(rmCode);         // Set the default Resource Manager for Items of this Item Type to be stored to.

                // Both RM & SMS must be set once one is specified.  Detect if SMS has not been set, 0,
                // and set it in that case to '1' as default.
                if(itemTypeDef.getDefaultCollCode()==0)
                    itemTypeDef.setDefaultCollCode((short)1); // Set the default SMS Colleciton to use for item of this Item Type.

                itemTypeDef.update();                         // Update the Item Type Definition.
            }
        System.out.println("ItemType '"+itemTypeName+"' now set to default RM Code of '"+rmCode+"'.");
    }
   
}//end class SResourceMgrDefSetDefaultICM
              
