/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SSMSCollectionDefRetrievalICM
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: SMS Collection Definition Retrieval, Listing, & Printing.  Checking for
 *                    SMS Definitions
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: List SMS Collections in System
 *                    Retrieving SMS Collection Definitions
 *                    Checking to See if SMS Collection is Defined
 *                    Pre-defined SMS Collections
 *                    Print SMS Collection Definition
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SSMSCollectionDefRetrievalICM 
 *                                            <Existing RM User-Defined Reference Name>
 *                                            <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    ex: java SSMSCollectionDefRetrievalICM RMDB
 *                    ex: java SSMSCollectionDefRetrievalICM MY_RM icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The specified reference name must be already defined, such as the default
 *                    "RMDB" specified during install.  If a new RM definition is to be 
 *                    configured, please refer to SResourceMgrDefCreationICM Sample.
 *                    The SMS Collections uses here must already be defined.  Please refer
 *                    to the SSMSCollectionDefCreationICM Sample.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SResourceMgrDefRetrievalICM.java
 ************************************************************************************************/
public class SSMSCollectionDefRetrievalICM{
    
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        String rmUserDefinedReferenceName  = "NOT_SPECIFIED";
        
        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        // Require all RM information be specified
        if (argv.length < 4) { // if not all LS Database arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("   java SSMSCollectionDefRetrievalICM");
            System.out.println("                         <Existing RM User-Defined Reference Name>");
            System.out.println("                         <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE: The specified reference name must be already defined, such as the");
            System.out.println("        default 'RMDB' specified during install.  If a new RM definition");
            System.out.println("        is to be configured, please refer to SResourceMgrDefCreationICM");
            System.out.println("        Sample.");
            System.out.println("");
            System.out.println("  NOTE: Defaults will be used if the database, username, & password are not");
            System.out.println("        specified.  All others are required.");
            System.out.println("");
            System.out.println("  ex: java SSMSCollectionDefRetrievalICM RMDB");
            System.out.println("  ex: java SSMSCollectionDefRetrievalICM MY_RM icmnlsdb icmadmin password");
        }
        if((argv.length < 1) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            System.out.println("======================================================");
            System.out.println("PROGRAM TERMINATED:  All RM Options Must Be Specified.");
            System.out.println("======================================================");
            return;
        } else if (argv.length != 4)
            System.out.println("  *** Some LS Database parameters not specified, using defaults..." );
        
        // Store commandline arguments.
        if (argv.length > 0) rmUserDefinedReferenceName = argv[0];

        // Store optional commandline arguments.
        if (argv.length > 1)  database = argv[1];
        if (argv.length > 2)  userName = argv[2];
        if (argv.length > 3)  password = argv[3];
        String ver = SConnectDisconnectICM.EIP_VERSION;
            
        System.out.println("==============================================");
        System.out.println("IBM Information Integrator for Content    v"+ver);
        System.out.println("Sample Program:  SSMSCollectionDefRetrievalICM");
        System.out.println("----------------------------------------------");
        System.out.println("           Database:  "+database);
        System.out.println("           UserName:  "+userName);
        System.out.println("  RM Reference Name:  "+rmUserDefinedReferenceName);       
        System.out.println("==============================================\n");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // PLEASE READ
            //-------------------------------------------------------------
            // This sample will heavily utilize the wrapper functions provided
            // at the bottom of this sample.  These funcitons will be used to
            // demonstrate usage of concepts in this sample.

            //------------------------------------------------------------------
            // List SMS Collections
            //------------------------------------------------------------------
            // Listing / Printing SMS Collections found in the system
            System.out.println("Listing / Printing SMS Collections found in the system...");

                printAllSMSCollections(dsICM); // Uses wrapper at bottom of sample.

            System.out.println("Listed / Printed SMS Collections found in the system.");

            //-------------------------------------------------------------
            // Obtaining an RM Code for a RM Definition
            //-------------------------------------------------------------
            // Obtaining the RM Code that is used to specify properties of Items
            // and Item Types.
            System.out.println("Obtaining RM Code for RM Definition '"+rmUserDefinedReferenceName+"'...");            
            
                if(SResourceMgrDefRetrievalICM.isResourceManagerSetup(dsICM, rmUserDefinedReferenceName)==false) // if it is not setup, stop here
                    throw new Exception("Resource Manager Definition Named '"+rmUserDefinedReferenceName+"' Not Defined.");

                short rmCode = SResourceMgrDefRetrievalICM.getResourceManagerIDCode(dsICM, rmUserDefinedReferenceName); // Use the wrapper function at bottom of sample.
            
            System.out.println("Obtained RM Code '"+rmCode+"' for RM Definition '"+rmUserDefinedReferenceName+"'.");            

            //-------------------------------------------------------------
            // Checking to See if SMS Collection Definitions Already Setup
            //-------------------------------------------------------------
            // This sample will skip over operations that have already been completed
            // in a previous run of this sample or other program.
            //
            // This sample will check for the system's determined collection names.
            // User-Defined Collections may be substituted.
            //
            // Demonstrating using Wrapper at bottom of sample.
            System.out.println("Checking to see if system pre-determined SMS Collection Definitions already setup...");

                if(isSMSCollectionSetupForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_NONMEDIA, rmCode))
                    System.out.println("Pre-Defined SMS Collection Name '"+DKConstantICM.RM_SMS_COLLECTION_NONMEDIA+"' (Default) Already Setup.");
                else 
                    System.out.println("Pre-Defined SMS Collection Name '"+DKConstantICM.RM_SMS_COLLECTION_NONMEDIA+"' (Default) Not Setup.");

                if(isSMSCollectionSetupForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_MEDIA, rmCode))
                    System.out.println("Pre-Defined SMS Collection Name '"+DKConstantICM.RM_SMS_COLLECTION_MEDIA+"' Already Setup.");
                else 
                    System.out.println("Pre-Defined SMS Collection Name '"+DKConstantICM.RM_SMS_COLLECTION_MEDIA+"' Not Setup.");

                if(isSMSCollectionSetupForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_VIDEO, rmCode))
                    System.out.println("Pre-Defined SMS Collection Name '"+DKConstantICM.RM_SMS_COLLECTION_VIDEO+"' Already Setup.");
                else 
                    System.out.println("Pre-Defined SMS Collection Name '"+DKConstantICM.RM_SMS_COLLECTION_VIDEO+"' Not Setup.");
                
            System.out.println("Checked to see if system pre-determined SMS Collection Definitions already setup.");

            //------------------------------------------------------------------
            // Obtaining an SMS Colleciton Code for a SMS Collection Definition
            //------------------------------------------------------------------
            // Obtaining the SMS Collection Code that is used to specify 
            // properties of Items and Item Types.
            System.out.println("Obtaining ID Codes for system pre-determined SMS Collection Definitions...");

                short smsCode = getSMSCollectionCodeForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_VIDEO, rmCode); // Uses wrapper at bottom of sample.
                System.out.println("Collection '"+DKConstantICM.RM_SMS_COLLECTION_VIDEO+"' has ID Code '"+rmCode+"'.");

                smsCode = getSMSCollectionCodeForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_MEDIA, rmCode); // Uses wrapper at bottom of sample.
                System.out.println("Collection '"+DKConstantICM.RM_SMS_COLLECTION_MEDIA+"' has ID Code '"+rmCode+"'.");

                smsCode = getSMSCollectionCodeForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_NONMEDIA, rmCode); // Uses wrapper at bottom of sample.
                System.out.println("Collection '"+DKConstantICM.RM_SMS_COLLECTION_NONMEDIA+"' has ID Code '"+rmCode+"'.");
                
            System.out.println("Obtained ID Codes for system pre-determined SMS Collection Definitions.");

            //------------------------------------------------------------------
            // Print SMS Collection Definition
            //------------------------------------------------------------------
            // User-Defined names may be substituted for the values below.
            System.out.println("Printing SMS Collection Definition...");

                printSMSCollectionDefinition(dsICM, DKConstantICM.RM_SMS_COLLECTION_NONMEDIA, rmCode); // Uses wrapper at bottom of sample.
                printSMSCollectionDefinition(dsICM, DKConstantICM.RM_SMS_COLLECTION_MEDIA, rmCode);    // Uses wrapper at bottom of sample.
                printSMSCollectionDefinition(dsICM, DKConstantICM.RM_SMS_COLLECTION_VIDEO, rmCode);    // Uses wrapper at bottom of sample.

            System.out.println("Printed SMS Collection Definition.");

            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");
        
            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main


    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    //******************************************************************
    // getSMSCollectionCodeForRM
    //******************************************************************
   /**
    * Obtain the specified SMS Collection's ID Code for the specified RM.  This function should not be used to simply validate that the SMS Collection exists or is setup.
    * @param dsICM   Connected DKDatastoreICM object.
    * @param rmSMSCollectionUserOrPreDefinedReferenceName  The RM's SMS Collection's User-Defined or Pre-Defined Reference Name.  If using a predefined collection, use DKConstant.RM_SMS_COLLECTION_xxxxxx.  Ex: MEDIA, NONMEDIA, VIDEO.
    * @param rmCode  Resource Manager ID Code that this SMS Collection is setup for.
    * @return Returns the SMS Collection ID Code for the specified Resource Manager.
    **/
    public static short getSMSCollectionCodeForRM(DKDatastoreICM dsICM, String rmSMSCollectionUserOrPreDefinedReferenceName, short rmCode) throws DKException, Exception{
        System.out.println("Obtaining "+rmSMSCollectionUserOrPreDefinedReferenceName+"'s SMS Collection ID Code in Resource Manager wiht ID Code of '"+rmCode+"'.");

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
        DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
        DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
        DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        try { 
            // Find the SMS Collection ID code for the specified SMS Collection.
            DKSMSCollectionDefICM smsCollDef = rmCfg.retrieveSMSCollection(rmSMSCollectionUserOrPreDefinedReferenceName, rmCode);  // Get the SMS Collection Definition for the SMS Colleciton referred to by the User-Defined Reference Name.
            short collCode = smsCollDef.getSMSCollectionCode();    // Get hte ID Code for the particular SMS Collection.
            System.out.println("Obtained "+rmSMSCollectionUserOrPreDefinedReferenceName+"'s SMS Collection ID Code of '"+collCode+"'.");
            return(collCode);
        } catch (Exception exc){ // Then it doesn't exist
            // IMPORTANT: Please do not use try-catch exception logic for program logic.  Use DKRMConfigurationMgmtICM.listSMSCollections() to search available SMS Collections.
            System.out.println("An error had occurd probably because it did not find SMS Collection with User Defined Reference Name '"+rmSMSCollectionUserOrPreDefinedReferenceName+"' in Resource Manager with ID Code of '"+rmCode+"'.  Please do not use try-catch exception logic for program logic.  Use DKRMConfigurationMgmtICM.listSMSCollections() to search available SMS Collections.");
            throw exc;
        }
    }//end getSMSCollectionCodeForRM

    //******************************************************************
    // isSMSCollectionSetupForRM
    //******************************************************************
   /**
    * Determines if the SMS Collection is setup or not for the specified
    * Resource Manager.
    * @param dsICM   Connected DKDatastoreICM object.
    * @param rmSMSCollectionUserOrPreDefinedReferenceName  The RM's SMS Collection's User-Defined or Pre-Defined Reference Name.  If using a predefined collection, use DKConstant.RM_SMS_COLLECTION_xxxxxx.  Ex: MEDIA, NONMEDIA, VIDEO.
    * @param rmCode  Resource Manager ID Code.
    * @return Returns true if one is setup, false if not.
    **/
    public static boolean isSMSCollectionSetupForRM(DKDatastoreICM dsICM, String rmSMSCollectionUserOrPreDefinedReferenceName, short rmCode) throws DKException, Exception{
        System.out.println("Determining if SMS Collection '"+rmSMSCollectionUserOrPreDefinedReferenceName+"' is setup for Resource Manager with ID Code of '"+rmCode+"'...");

        boolean found = false;  // Return value tracking if it was found.
        short smsCode = -1;

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
        DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
        DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
        DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Obtain a list of SMS Collection Definitions.
        dkCollection SMSDefinitions = rmCfg.listSMSCollections(); // Obtain the SMS Collection Definitions defined in the system.
        dkIterator iter = SMSDefinitions.createIterator(); // create an iterator to search for the specified name.

        while(iter.more() && (found==false)){ // while there are more SMS Collection Definition objects & it has not been found, continue searching
            DKSMSCollectionDefICM smsCollDef = (DKSMSCollectionDefICM)iter.next(); // Move pointer to next element & return that object.
            if((smsCollDef.getName().compareTo(rmSMSCollectionUserOrPreDefinedReferenceName)==0) && (rmCode == smsCollDef.getRMCode())){ // Check to see if names match
                found   = true;               // mark it as found
                smsCode = smsCollDef.getSMSCollectionCode();    // Get the ID Code for the particular SMS Collection.
            }
        }

        if(found) // Report Status
            System.out.println("Found "+rmSMSCollectionUserOrPreDefinedReferenceName+"'s SMS Collection ID Code of '"+smsCode+"'.");
        else
            System.out.println("Did not find SMS Collection with User Defined Reference Name '"+rmSMSCollectionUserOrPreDefinedReferenceName+"' in Resource Manager with ID Code of '"+rmCode+"'.");

        return(found);
    }//end IsSMSCollectionSetupForRM

    //******************************************************************
    // printAllSMSCollections
    //******************************************************************
   /**
    * Prints / Lists all SMS Collections found in the system.
    * @param dsICM   Connected DKDatastoreICM object.
    **/
    public static void printAllSMSCollections(DKDatastoreICM dsICM) throws DKException, Exception{

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
        DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
        DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
        DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        // Obtain a list of SMS Collection Definitions.
        dkCollection SMSDefinitions = rmCfg.listSMSCollections(); // Obtain the SMS Collection Definitions defined in the system.
        dkIterator iter = SMSDefinitions.createIterator(); // create an iterator to search for the specified name.

        while(iter.more()){ // while there are more SMS Collection Definition objects & it has not been found, continue searching
            DKSMSCollectionDefICM smsCollDef = (DKSMSCollectionDefICM)iter.next(); // Move pointer to next element & return that object.
            System.out.println("     - "+ smsCollDef.getName()+" ("+ smsCollDef.getSMSCollectionCode() +")"); // Print its name & code.
        }
    }//end printAllSMSCollections

    //******************************************************************
    // printSMSCollectionDefinition
    //******************************************************************
   /**
    * Print the specified SMS Collection Definition. 
    * @param dsICM   Connected DKDatastoreICM object.
    * @param rmSMSCollectionUserOrPreDefinedReferenceName  The RM's SMS Collection's User-Defined or Pre-Defined Reference Name.  If using a predefined collection, use DKConstant.RM_SMS_COLLECTION_xxxxxx.  Ex: MEDIA, NONMEDIA, VIDEO.
    * @param rmCode  Resource Manager ID Code that this SMS Collection is setup for.
    **/
    public static void printSMSCollectionDefinition(DKDatastoreICM dsICM, String rmSMSCollectionUserOrPreDefinedReferenceName, short rmCode) throws DKException, Exception{

        // Obtain the RM Configuration Management Object
        DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
        DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
        DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
        DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.

        DKSMSCollectionDefICM smsDef = rmCfg.retrieveSMSCollection(rmSMSCollectionUserOrPreDefinedReferenceName,rmCode);  // Get the SMS Coll. Definition for the specified Resource Manager.
        System.out.println("-----------------------------------------------");
        System.out.println("          SMS Collection Definition");
        System.out.println("-----------------------------------------------");
        System.out.println("         Name:  "+smsDef.getName());
        System.out.println("      RM Code:  "+smsDef.getRMCode());
        System.out.println("     SMS Code:  "+smsDef.getSMSCollectionCode());
        System.out.println("      User ID:  "+smsDef.getUserid());
        System.out.println("-----------------------------------------------");

    }//end printSMSCollectionDefinition
   
}//end class SSMSCollectionDefRetrievalICM
              
