/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Information Integrator for Content V8.2  (program number 5724-B43)             *
 * (c ) Copyright IBM Corp. 1998, 2002, 2003. All Rights Reserved.                    *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************

Extension of SItemTypeCreationICM Sample
    This Sample is an extension of the documentation of SItemTypeCreationICM Sample.
    Please refer to that sample for Data Modeling concepts and all information not
    specific to Video Streams.
    
Streams
    Video Stream is an extension of Stream.  This sample will provide an example of
    an Item Type using Video Streams.  Similar principles may be applied to use a 
    Stream Resource.
    
Video Streams
    Video Stream Resources require Video Charger to be installed and configured for 
    use.  Please ensure that it is installed and configured to work with the 
    Resource Manager.
    
    This Sample will provide an example of creating an Item Type that will use
    Video Stream Resource content.  The concepts, options, and settings for 
    Item Types are the same as those covered in SItemTypeCreationICM and therefore
    will not be repeated in this sample.  The Video Stream sample was seperated
    from the other sample so that users do not have to have Video Charger installed
    to run the other Samples.

-----------------------------------
Model of Item Types in this Sample:
-----------------------------------

                     Multivalue Attr. / 
    Item Type Name   Component Type Name  Attribute Name
    ---------------  -------------------  ----------------------------
    S_videoType      <root>               S_author
                                          S_length
    ------------------------------------------------------------------

----------------------------
Attributes Defined in Sample
----------------------------

    Attribute Name  Type          
    --------------- -------------
    S_author        Varchar(64)
    S_length        Varchar(64)
   
********************************************************************/

// Imports
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.server.*;
import java.io.*;

/************************************************************************************************
 *          FILENAME: SVideoItemTypeCreateICM.java
 *                    ---------------------------------------------------------------------------
 *       DESCRIPTION: Creating an Item Type for Steam or Video Stream Resources.  This Sample
 *                    will setup the Video SMS Collection for the specified Resource Manager
 *                    Definition if not already setup.  It will also ensure that the system
 *                    is set to use the Item Type's default SMS collection, not the user's.
 *                    Also defines attributes used by this the Item Type created.
 *                    ---------------------------------------------------------------------------
 *     DEMONSTRATION: Creating an Item Type for Stream Resources
 *                    Creating an Item Type for Video Stream Resources
 *                    ---------------------------------------------------------------------------
 * COMMANDLINE USAGE  java SVideoItemTypeCreateICM <Existing RM User-Defined Reference Name>
 *                                                 <database> <userName> <password>
 *                   
 *                    NOTE: The specified reference name must be already defined, such as the
 *                          default "RMDB" specified during install.  If a new RM definition 
 *                          is to be configured, please refer to SResourceMgrDefCreationICM 
 *                          Sample.
 * 
 *                    NOTE: Defaults will be used if the database, username, & password are not 
 *                          specified.  All others are required.
 *
 *                    Ex:  java SVideoItemTypeCreateICM RMDB
 *                    Ex:  java SVideoItemTypeCreateICM RMDB icmnlsdb icmadmin password
 *                    ---------------------------------------------------------------------------
 *      PREREQUISITS: The Item Type Defined here must not already exist.  If it does, please
 *                    run SVideoItemTypeDeletionICM first.
 *                    
 *                    A Resource Manager definition must already be created and configured
 *                    to work with Video Charger.  The reference name for that Resource
 *                    Manager is one of the commandline parameters to this sample.  You
 *                    may refer to the SResourceMgrDefCreationICM Sample for more 
 *                    information on creating Resource Manager Definition objects.  Please
 *                    refer to the installation instructions for information on Configuring
 *                    Video Charger with Content Manager / Resource Manager.
 *                    ---------------------------------------------------------------------------
 *    FILES REQUIRED: SConnectDisconnectICM.java
 *                    SResourceMgrDefRetrievalICM.java
 *                    SSMSCollectionDefCreationICM.java
 *                    SSMSCollectionDefRetrievalICM.java
 *                    SSMSCollectionDefSetDefaultICM.java
 *                    SAttributeDefinitionRetrievalICM.java
 *                    SItemTypeRetrievalICM.java
 ************************************************************************************************/
public class SVideoItemTypeCreateICM
{
    //-------------------------------------------------------------
    // Main
    //-------------------------------------------------------------
    /**
     * Run the Sample.
     * @param argv[] String Array containing arguments.  Optional arguments are <databse> <userName> <password>.
     */
    public static void main(String argv[]) throws DKException, Exception{
    
        // Defaults for connecting to the database.
        String database = SConnectDisconnectICM.DEFAULT_DATABASE;
        String userName = SConnectDisconnectICM.DEFAULT_USERNAME;
        String password = SConnectDisconnectICM.DEFAULT_PASSWORD;

        // RM Configuration Variables.  Used to store commandline arguments.
        String rmUserDefinedReferenceName  = "NOT_SPECIFIED";

        //------------------------------------------------------------
        // Checking for input parameters
        //--------------------------------------------------------------
        if (argv.length < 4) { // if not all 3 arguments were specified, use defaults and report correct usage.
            System.out.println("Usage: " );
            System.out.println("  java SVideoItemTypeCreateICM <Existing RM User-Defined Reference Name>");
            System.out.println("                               <database> <userName> <password>");
            System.out.println("");
            System.out.println("  NOTE: The specified reference name must be already defined, such as the");
            System.out.println("        default 'RMDB' specified during install.  If a new RM definition");
            System.out.println("        is to be configured, please refer to SResourceMgrDefCreationICM");
            System.out.println("        Sample.");
            System.out.println("");
            System.out.println("  NOTE: Defaults will be used if the database, username, & password are not");
            System.out.println("        specified.  All others are required.");
            System.out.println("");
            System.out.println("  Ex:  java SVideoItemTypeCreateICM RMDB");
            System.out.println("  Ex:  java SVideoItemTypeCreateICM RMDB icmnlsdb icmadmin password");
            System.out.println("");
        }
        if((argv.length < 1) ){ // if every RM parameter not specified if going to attempt to setup, stop the program.
            System.out.println("======================================================");
            System.out.println("PROGRAM TERMINATED:  All RM Options Must Be Specified.");
            System.out.println("======================================================");
            return;
        } else if (argv.length != 4)
            System.out.println("  *** Some LS Database parameters not specified, using defaults..." );

        // Store commandline arguments.
        if (argv.length > 0) rmUserDefinedReferenceName = argv[0];

        // Store optional commandline arguments.
        if (argv.length > 1)  database = argv[1];
        if (argv.length > 2)  userName = argv[2];
        if (argv.length > 3)  password = argv[3];
        String ver = SConnectDisconnectICM.EIP_VERSION;
        
        System.out.println("===========================================");
        System.out.println("IBM Information Integrator for Content v"+ver);
        System.out.println("Sample Program:  SVideoItemTypeCreateICM");
        System.out.println("-------------------------------------------");
        System.out.println("           Database:  "+database);
        System.out.println("           UserName:  "+userName);
        System.out.println("  RM Reference Name:  "+rmUserDefinedReferenceName);       
        System.out.println("===========================================");
        
        try{
            //-------------------------------------------------------------
            // Connect to datastore
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Connecting to datastore (Database '"+database+"', UserName '"+userName+"')...");

                DKDatastoreICM dsICM = new DKDatastoreICM();  // Create new datastore object.
                dsICM.connect(database,userName,password,""); // Connect to the datastore.

            System.out.println("Connected to datastore (Database '"+dsICM.datastoreName()+"', UserName '"+dsICM.userName()+"').");

            //-------------------------------------------------------------
            // Ensure RM Definition Setup
            //-------------------------------------------------------------
            // Make sure that the RM Reference Name is valid.  This example will not
            // detect if Video Charger is setup, but will check for the RM
            // Definition.
            System.out.println("Validating Specified RM Name '"+rmUserDefinedReferenceName+"'...");

                if(!SResourceMgrDefRetrievalICM.isResourceManagerSetup(dsICM, rmUserDefinedReferenceName))
                    throw new Exception("The Specified RM Name '"+rmUserDefinedReferenceName+"' was not found.");

            System.out.println("Validated Specified RM Name '"+rmUserDefinedReferenceName+"'.");

            //-------------------------------------------------------------
            // Setup Video SMS Collection If Needed
            //-------------------------------------------------------------
            // Check to see if the default Video SMS Collection is already setup,
            // if not, set it up.
            System.out.println("Setting Up Video SMS Collection if Needed...");

                short smsCode;

                // First get the RM Code
                short rmCode = SResourceMgrDefRetrievalICM.getResourceManagerIDCode(dsICM,rmUserDefinedReferenceName);

                // Determine if SMS Colleciton is setup.
                if(!SSMSCollectionDefRetrievalICM.isSMSCollectionSetupForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_VIDEO,rmCode)){
                    System.out.println("Setting Up Video SMS Collection (Not already setup)...");                
                    // Obtain the RM Configuration Management Object
                    DKDatastoreDefICM        dsDefICM = (DKDatastoreDefICM)dsICM.datastoreDef();                     // Get the Datastore Definition Object
                    DKDatastoreAdminICM      dsAdmin  = (DKDatastoreAdminICM) dsDefICM.datastoreAdmin();             // Get the Datqastore Administration
                    DKConfigurationMgmtICM   cfgMgmt  = (DKConfigurationMgmtICM) dsAdmin.configurationManagement();  // Get the configuration management.
                    DKRMConfigurationMgmtICM rmCfg    = cfgMgmt.rmConfigurationMgmt();                               // Get the Resource Manager configuration.
                    // Get the RM User Id
                    DKResourceMgrDefICM rmDef = rmCfg.retrieveResourceMgr(rmUserDefinedReferenceName);  // Get the RM Definition for the specified Resource Manager.
                    String rmUserName = rmDef.getRMUserid();
                    // Setup Video SMS Collection
                    smsCode = SSMSCollectionDefCreationICM.setupSMSCollectionForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_VIDEO, rmCode, rmUserName);
                }else{ // Otherwise get the existing SMS Collection code.
                    System.out.println("Already Setup.");
                    smsCode = SSMSCollectionDefRetrievalICM.getSMSCollectionCodeForRM(dsICM, DKConstantICM.RM_SMS_COLLECTION_VIDEO, rmCode);
                }
                
            System.out.println("Set Up Video SMS Collection if Needed.");
            
            //-------------------------------------------------------------
            // Set SMS Choice in System
            //-------------------------------------------------------------
            // Samples will rely on using the Default collection in the Item Type, not
            // the default for the user.  Please refer to SSMSCollectionDefSetDefaultICM
            // for more information.
            System.out.println("Setting System SMS Choice...");

                SSMSCollectionDefSetDefaultICM.setSystemSMSChoice(dsICM,(short) 1);  // 1 - Use Item Type's Default.
            
            System.out.println("Set System SMS Choice.");

            //-------------------------------------------------------------
            // Validating that the new Item Type Doesn't Exist
            //-------------------------------------------------------------
            // Making sure that it is not already defined.
            System.out.println("Validating that Item Type 'S_videoType' is not already defined...");

                if(SItemTypeRetrievalICM.doesEntityExist(dsICM,"S_videoType"))
                    throw new Exception("Item Type 'S_videoType' already Exists.  Please run SVideoItemTypeDeleteICM first.");

            System.out.println("Validated that Item Type 'S_videoType' is not already defined.");

            //-------------------------------------------------------------
            // Defining Attribute Definitions
            //-------------------------------------------------------------
            // Defining Attributes Used by the 'S_videoType' Item Type.
            System.out.println("Defining Attributes used by 'S_videoType' Item Type...");

                // If it doesn't exist, create it
                if(!SAttributeDefinitionRetrievalICM.doesAttrDefExist(dsICM,"S_author")) 
                    SAttributeDefinitionCreationICM.defineAttribute(dsICM, "S_author", "Author of the Video", DKConstant.DK_CM_VARCHAR, 64);
                    
                // If it doesn't exist, create it
                if(!SAttributeDefinitionRetrievalICM.doesAttrDefExist(dsICM,"S_length")) // If it doesn't exist, create it
                    SAttributeDefinitionCreationICM.defineAttribute(dsICM, "S_length", "Length of the Video", DKConstant.DK_CM_VARCHAR, 64);

            System.out.println("Defined Attributes used by 'S_videoType' Item Type.");

            //-------------------------------------------------------------
            // Create New Video Item Type
            //-------------------------------------------------------------
            System.out.println("Creating Video Item Type 'S_videoType'...");
        
                DKItemTypeDefICM itemType = new DKItemTypeDefICM(dsICM);                     // create a new Item Type Definition object.
                itemType.setName("S_videoType");                            // The name should be less than 15 characters.  It will be used to create items of this item type later.
                itemType.setDescription("Simple Resource Video Item Type"); // More detailed description than name.

                // Add an attribute to it
                DKDatastoreDefICM dsDefICM = (DKDatastoreDefICM) dsICM.datastoreDef(); // Get the Datastore Definition object.
                DKAttrDefICM attr1 = (DKAttrDefICM) dsDefICM.retrieveAttr("S_author"); // Retrieve a reference to the desired attribute definitions from the datastore definition
                DKAttrDefICM attr2 = (DKAttrDefICM) dsDefICM.retrieveAttr("S_length"); 
                itemType.addAttr(attr1);                                               // Add the desired attribute to the item type defintion.
                itemType.addAttr(attr2);

                // Since this particular Item Type will hold computer files, it should have a resource classification.
                itemType.setClassification(DKConstantICM.DK_ICM_ITEMTYPE_CLASS_RESOURCE_ITEM);
                
                // Now an XDO class / Type of Resource must be specified.
                itemType.setXDOClassName(DKConstantICM.DK_ICM_XDO_VIDEOSTREAM_CLASS_NAME); 
                itemType.setXDOClassID(DKConstantICM.DK_ICM_XDO_VIDEOSTREAM_CLASS_ID);

                // The default Resource Manager & SMS Colleciton in which to store Resource 
                // Items of this Item Type may be specified here or updated later. 
                itemType.setDefaultRMCode(rmCode);       // Set the default Resource Manager for Items of this Item Type to be stored to.
                itemType.setDefaultCollCode(smsCode);    // Set the default SMS Colleciton to use for item of this Item Type.

                // Version Control Setting
                itemType.setVersionControl((short)DKConstantICM.DK_ICM_VERSION_CONTROL_NEVER); // Default - Version Control is not enabled.

                itemType.add();                                           // Add / Save the Item Type Definition.

            System.out.println("Created Video Item Type 'S_videoType'.");
        
            //-------------------------------------------------------------
            // Disconnect from datastore & Destroy Reference
            //-------------------------------------------------------------
            // See Sample SConnectDisconnectICM for more information
            System.out.println("Disconnecting from datastore & destroying reference...");            

                dsICM.disconnect();
                dsICM.destroy();

            System.out.println("Disconnected from datastore & destroying reference.");

            //-------------------------------------------------------------
            // Sample program completed without exception
            //-------------------------------------------------------------
            System.out.println("\n==========================================");
            System.out.println("Sample program completed.");
            System.out.println("==========================================\n");
        }
        //------------------------------------------------------------
        // Catch & Print Exceptions        
        //------------------------------------------------------------
        catch (DKException exc){
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        } catch (Exception exc) {
            SConnectDisconnectICM.printException(exc);  // Print the exception using the function listed below.
            throw(exc);
        }
    }// end main

    //=================================================================
    // Wrapper Functions
    //=================================================================
    // The following are wrapper functions for functionality covered
    // in this sample.  These functions may be used by other samples.

    // No additional wrapper functions from this sample.

}//end class SVideoItemTypeCreateICM
