
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************/

/*
 * Description    : Add document content to an already existing document by specifying the
 *                   tempId of the existing document and the content class, page count and
 *                   file name for the document content being added.
 *
 */
import com.ibm.mm.sdk.server.*;
import com.ibm.mm.sdk.common.*;
import java.io.*;
import java.lang.*;

public class TAddXDOIP
   implements DKConstantIP {

  // Main method
  public static void main(String[] args)
  {
    String  tempId = "";
    String  fileName = "1040Page2.DAT";     // Default...
    int     contentClass = DK_CC_MODCA_IS2; // Default...
    int     pageCount = 1;                  // Default...

    String libsrv = "";
    String userid = "";
    String pw = "";

    if (args.length == 5)
    {
      libsrv = args[0];
      userid = args[1];
      pw = args[2];
      tempId = args[3];
      contentClass = new Integer(args[4]).intValue();
      pageCount = new Integer(args[5]).intValue();
      fileName = args[6];
    }
    else if (args.length == 4)
    {
      libsrv = args[0];
      userid = args[1];
      pw = args[2];
      tempId = args[3];
    }
    else
    {
      System.out.println("To add a part (i.e. ODM document)");
      System.out.println("Invoke: java TAddXDOIP <libsrv> <userid> <pw> <tempDocId> <content class> <page count> <filename>");
      System.out.println("  or to use default values");
      System.out.println("Invoke: java TAddXDOIP <libsrv> <userid> <pw> <tempDocId>");
      return;
    }

    DKDatastoreIP dsIP = null;
    try
    {
      dsIP = new DKDatastoreIP();

      String s = new String( "APPL=01;FAFIP=9.67.43.83;IODMIP=9.67.43.83;FAFPORT=3061;"
                           + "IODMPORT=3082;FAFPROT=4000;IODMPROT=4000;FAFSITE=CS61" );

      System.out.println("connecting to datastore");
      dsIP.connect(libsrv,userid,pw,s);

      System.out.println("datastore ImagePlus connected");

      // Using the tempid, get the document associated with the blob being added...
      DKPidIP aDDOpid = new DKPidIP();
      aDDOpid.setIdStringCount(DK_IP_PID_COUNT_DOC);
      aDDOpid.setIdString( DK_IP_PID_DOC_FLDID, "IPSAMP" );
      aDDOpid.setIdString( DK_IP_PID_DOC_ID, tempId );
      aDDOpid.setObjectType("DOCUMENT");
      DKDDO aDDO = new DKDDO( dsIP, aDDOpid);
      aDDO.retrieve();

      DKBlobIP axdo = new DKBlobIP(dsIP);
      DKPidXDOIP  apid = new DKPidXDOIP();
      apid.setDocId(tempId);   System.out.println("TEMPID: " + tempId);
      axdo.setPidObject(apid);
      axdo.setContentClass(contentClass);
      axdo.setPageCount(pageCount);

      axdo.add(fileName);

      System.out.println("add successfully.....");
      apid = (DKPidXDOIP)axdo.getPidObject();
      System.out.println( "after add pid = "
                        + apid.getPrimaryId());
      System.out.println( "after add docId = "
                        + apid.getDocId());


      // Get COLLECTION from associated DDO in order to retrieve XDO...
      apid.setIdString( DK_IP_PIDXDO_DOC_COLLECTION, aDDO.getPidObject().getIdString(DK_IP_PID_DOC_COLLECTION) );
      axdo.setPidObject( apid );

      System.out.println("about to do retrieve()...");
      axdo.retrieve();

      System.out.println("after retrieve() length=" + axdo.length());

      //String fileNameB = "tempRetrieve.DAT";
      //System.out.println("about to do retrieve to file=" + fileNameB);
      //axdo.retrieve(fileNameB);

      //axdo.getContentToClientFile("temp.DAT", 1);

      // Default viewer
      axdo.setInstanceOpenHandler("frnjview ", true);

      int cc = axdo.getContentClass();
      System.out.println("Content class " + cc);
      if ( cc == DK_CC_GIF)
        axdo.setInstanceOpenHandler("lviewpro", true); //use lviewpro in NT
      else if (cc == DK_CC_ASCII)
        axdo.setInstanceOpenHandler("notepad", true);

      System.out.println("about to do open()...");
      axdo.open();

      dsIP.disconnect();
      System.out.println("datastore disconnected");
      dsIP.destroy();
    }
    catch (DKException exc)
    {
     	try
       {
          if ( dsIP != null )
             dsIP.destroy();
     	}
     	catch (Exception e)
     	{
     	   e.printStackTrace();
     	}
       System.out.println("Exception name " + exc.name());
       System.out.println("Exception message " + exc.getMessage());
       exc.printStackTrace();
    }
    catch (Exception exc)
    {
     	try
       {
          if ( dsIP != null )
             dsIP.destroy();
     	}
     	catch (Exception e)
     	{
     	   e.printStackTrace();
     	}
       System.out.println("Exception message " + exc.getMessage());
       exc.printStackTrace();
    }
  }
}

