
/*BEGINPROLOGUE************************************************************************
 * @copyright(disclaimer)                                                             *
 * Licensed Materials - Property of IBM                                               *
 * IBM Enterprise Information Portal for Multiplatforms V8.1 (program number 5724-B43)*
 * (c ) Copyright IBM Corp. 1998,  2002. All Rights Reserved.                         *
 *                                                                                    *
 * US Government Users Restricted Rights                                              *
 * Use, duplication or disclosure restricted by GSA ADP Schedule                      *
 * Contract with IBM Corp.                                                            *
 *                                                                                    *
 * DISCLAIMER OF WARRANTIES :                                                         *
 *                                                                                    *
 * Permission is granted to copy and modify this Sample code, and to                  *
 * distribute modified versions provided that both the copyright                      *
 * notice,- and this permission notice and warranty disclaimer appear in all copies   * 
 * and modified versions.                                                             *
 *                                                                                    *
 * THIS SAMPLE CODE IS LICENSED TO YOU AS-IS.                                         *
 * IBM AND ITS SUPPLIERS AND LICENSORS DISCLAIM ALL WARRANTIES, EITHER EXPRESS OR     *
 * IMPLIED, IN SUCH SAMPLE CODE, INCLUDING THE WARRANTY OF NON-INFRINGEMENT AND THE   *
 * IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE. IN NO   *
 * EVENT WILL IBM OR ITS LICENSORS OR SUPPLIERS BE LIABLE FOR ANY DAMAGES ARISING OUT *
 * OF THE USE OF OR INABILITY TO USE THE SAMPLE CODE, DISTRIBUTION OF THE SAMPLE CODE,*
 * OR COMBINATION OF THE SAMPLE CODE WITH ANY OTHER CODE. IN NO EVENT SHALL IBM OR ITS*
 * LICENSORS AND SUPPLIERS BE LIABLE FOR ANY LOST REVENUE, LOST PROFITS OR DATA, OR   *
 * FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL,INCIDENTAL OR PUNITIVE DAMAGES,       *
 * HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,-, EVEN IF IBM OR ITS     *
 * LICENSORS OR SUPPLIERS HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.       *
 *                                                                                    *
 * @endCopyright                                                                      *
 **************************************************************************************
 */

// sample callback class to OnDemand

import com.ibm.mm.sdk.internal.*;
import com.ibm.mm.sdk.common.*;
import com.ibm.mm.sdk.client.*;
import java.io.*;
import java.lang.*;
import java.util.*;

/**
This class defines an interface to the callback object.

<p>
5697-250, 5648-A08 and 5765-C86 (C) Copyright IBM Corp. 1994, 1997<p>

<HR>
@author Nelson Chen (nnchen@us.ibm.com)<BR>
@version 1.0 March 2001
@modified by Nelson Chen (nnchen@us.ibm.com)<BR>
@version 1.1 May 2002

*/

public class TClientCallbackOD implements dkCallback, DKConstantOD
{
  private DKDatastoreOD _dsOD = null;

  private int _threshold = 20;
  private int _iterationsValue = -1;
  private int _iterations = 0;

  private boolean _retrieveHits = false;
  private boolean _cancel = false;
  private boolean _isDone = false;
  private boolean _displayed = false;

  private int    _cnt = 0;
  private Vector _hitListVector = null;
  private DKException _exc = null;

  //**********************************************************************
  TClientCallbackOD()
  {
  }

  TClientCallbackOD(DKDatastoreOD dsOD,
              int thresholdValue,
              int iterationsValue,
              boolean retrieveHits)
  {
    _dsOD = dsOD;
    _threshold = thresholdValue;
    _iterationsValue = iterationsValue;
    _retrieveHits = retrieveHits;
    _hitListVector = new Vector();
  }

  // dkCallback implementation...
  /**
   *  Sets the number of returned hits for each data callback.
   *  This method should be set by applications for the datastore.
   *  @param  nValue the number of hits in the result collection
   *      for each callback.
   */
  public void setThreshold(int nValue)
  {
    _threshold = nValue;
  }

  /**
   *  Gets the number of returned hits that the server return
   *  for each callback.  The datastore will call this method for
   *  each query.
   *  @return the threshold value,
   */
  public int getThreshold()
  {
    return _threshold;
  }

  /**
   *  Indicates that the user has canceled the query.
   *  It should be called by the application to let the server
   *  knows to stop the query.
   */
  public void cancelQuery()
  {
    _cancel = true;
  }

  /**
   *  Returns nHits from the backend datastore
   *  @param  ddoList a dkCollection of DKDDO objects
   *  @param  isDone  a boolean value indicating whether the server is
   *        done with the query.  That is, there is more hits.
   *  @return true if query is done or user has canceled the query.
   */
  public boolean dataCallback(dkCollection ddoList, boolean isDone)
  {
    _iterations++;
    if (isDone)
      System.out.println("\ndone value is: " + isDone);
    //else
    //  System.out.println("\nnumber of hits returned is: " + ddoList.cardinality());

    // show the world we got hits!!!
    dkIterator pIter = ddoList.createIterator();
    while (pIter.more())
    {
      try
      {
        _cnt++;
        DKDDO ddo = (DKDDO)pIter.next();
        //System.out.println(" ddo : " + ddo);
        displayHit(_cnt, ddo);

        if (_retrieveHits)
        {
          // save the ddo to the end of the hitlist
          _hitListVector.addElement((Object)ddo);
        }
      }
      catch (DKException exc)
      {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        System.out.println("Exception error code " + exc.errorCode());
        System.out.println("Exception error state " + exc.errorState());
        exc.printStackTrace();
      }
      catch (Exception exc)
      {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
      }
    }
    if (_retrieveHits == false)
      System.out.println("Number of items processed: " + _cnt );

    if (_iterationsValue != -1 && _iterations == (_iterationsValue))
    {
      cancelQuery();
    }
    if (_cancel || isDone)
    {
      _isDone = true;
    }

    if (isDone && !_displayed)
    {
      _displayed = true;
      System.out.println("Total number of items received: " + _cnt );
      System.out.println("TClientCallbackOD done ...");
    }
    return _cancel;
  }

  public boolean isDone()
  {
    return _isDone;
  }

  /**
   *  Report an exception from the backend datastore
   *  @param     exc an exception from the backend datastore
   */
  public void reportException(DKException exc)
  {
    _exc = exc;
  }

  /**
   *  Get an exception from the backend datastore
   *  @return an exception from the backend datastore
   */
  public DKException getException()
  {
    return _exc;
  }

  /**
   *  Clear exception object
   */
  public void resetException()
  {
    _exc = null;
  }

  // --------------------------------------------------------------------
  // retrieve results
  // --------------------------------------------------------------------
  public void retrieveDDO()
  {
    if (_hitListVector.size() > 0)
    {
      _cnt = 0;
      try
      {
        for (int i=0; i<_hitListVector.size(); i++)
        {
          _cnt++;
          DKDDO ddoold = (DKDDO)_hitListVector.elementAt(i);
          String pidStr = ((DKPid)ddoold.getPidObject()).pidString();
          System.out.println(" pidString : " + pidStr);

          // the retrieval action
          DKPid pid = new DKPid (pidStr);
          short id, docType = 0;
          if ((id = ddoold.propertyId(DK_CM_PROPERTY_ITEM_TYPE)) > 0)
            docType = ((Short)ddoold.getProperty(id)).shortValue();
          if (docType == DK_CM_DOCUMENT)
          {
            System.out.println("create the DDO from the pidstring...");
            DKDDO ddo = _dsOD.createDDO(ddoold.getObjectType(), DK_CM_DOCUMENT);
            ddo.setPidObject(pid);

            // This step is needed for all callback testcases
            _dsOD.retrieveObject((dkDataObject)ddo);

            displayDDO(i+1, _dsOD, ddo);
          }
        }
        System.out.println("Total number of items processed: " + _cnt );
        _hitListVector.removeAllElements();
      }
      catch (DKException exc)
      {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        System.out.println("Exception error code " + exc.errorCode());
        System.out.println("Exception error state " + exc.errorState());
        exc.printStackTrace();
      }
      catch (Exception exc)
      {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
      }
    }
  }

  // --------------------------------------------------------------------
  // display results
  // --------------------------------------------------------------------
  public void displayHit(int cnt, DKDDO p)
  {
    short j = 0;
    if (p != null)
    {
      try
      {
        int numDataItems = numDataItems = p.dataCount();
        // display attribute (field) names (1st time only)
        if (cnt == 1)
        {
          System.out.println("");
          for (j = 1; j <= numDataItems; j++)
          {
            String strDataName = p.getDataName(j);
            if (strDataName.equals(DK_CM_DKPARTS) == false &&
                strDataName.equals("DKResource") == false &&
                strDataName.equals("DKViews") == false &&
                strDataName.equals("DKLargeObject") == false &&
                strDataName.equals("DKFixedView") == false &&
                strDataName.equals("DKAnnotations") == false)
            {
              System.out.print("  \t\""+strDataName+"\"");
            }
          }
          System.out.println("");
        }
        // display attribute (field) values
        for (j = 1; j <= numDataItems; j++)
        {
          Object a = p.getData(j);
          String strDataName = p.getDataName(j);
          if (strDataName.equals(DK_CM_DKPARTS) == false &&
              strDataName.equals("DKResource") == false &&
              strDataName.equals("DKViews") == false &&
              strDataName.equals("DKLargeObject") == false &&
              strDataName.equals("DKFixedView") == false &&
              strDataName.equals("DKAnnotations") == false)
          {
            if (j == 1)
              System.out.print(" "+cnt+".\t");
            else
              System.out.print("\t");

            if ((a instanceof String)  ||
                (a instanceof Integer) ||
                (a instanceof Short)   ||
                (a instanceof Long)    ||
                (a instanceof Float)   ||
                (a instanceof Double)  ||
                (a instanceof DKDate)  ||
                (a instanceof DKTime)  ||
                (a instanceof DKTimestamp))
            {
              System.out.print("\"" + a + "\"");
            }
            else
              System.out.print("");
          }
        }
        System.out.println("");
      }
      catch (DKException exc)
      {
        System.out.println("Exception name " + exc.name());
        System.out.println("Exception message " + exc.getMessage());
        System.out.println("Exception error code " + exc.errorCode());
        System.out.println("Exception error state " + exc.errorState());
        exc.printStackTrace();
      }
      catch (Exception exc)
      {
        System.out.println("Exception message " + exc.getMessage());
        exc.printStackTrace();
      }
    }
  }

  // --------------------------------------------------------------------
  // display results
  // --------------------------------------------------------------------
  public void displayDDO(int           cnt,
                         DKDatastoreOD dsOD,
                         DKDDO         p)
  {
    String DocFile, DataFile, ResourceFile, TocFile, LogViewFile;

    FileOutputStream myLogViewFile, myResourceFile, myTocFile, myDataFile, myDocFile;
    String extension = new String("");
    String MimeType = new String("");
    String noteString = new String("");
    String segmentFile;
    String cnt_str;
    DKAnnotationOD annotation = null;
    int numNotes = 0;
    int numBytes = 0;

    DKDDO pDDO = null;
    dkXDO pXDO = null;
    int i = 0;
    int numDataItems = 0;
    short k = 0;
    short j = 0;
    String strDataName;
    dkCollection pCol = null;
    dkIterator pIter = null;
    Object a = null;
    dkDataObjectBase pDO = null;
    DKPid pid = null;

    try
    {
      // create the output directory if it does not exist
      String OutDir;
      if (File.separator.equals("\\"))
        OutDir = new String("c:\\cmbuild\\output\\");
      else
        OutDir = new String("/archive/bulldog/eip/samples/output/");

      File outputPath = new File(OutDir);
      if (outputPath.exists() == false)
      {
        // try this first
        if (outputPath.mkdirs() == false)
        {
          // If mkdirs did not work, try this
          outputPath.mkdir();
        }
      }

      if (p != null)
      {
        System.out.println("\n=====> Item " + cnt + " <=====");

        numDataItems = p.dataCount();
        pid = p.getPidObject();

        // Retrieve privileges
        DKDatastoreExtOD dsODExt = (DKDatastoreExtOD)dsOD.getExtension(DK_CM_DATASTORE_EXT);
        DKCommonPrivilege priv = dsODExt.getCommonItemPrivilege(p);
        if (priv != null)
        {
          System.out.println("privileges are:");
          if (priv.checkAuthorization(DK_CM_PRIV_PRINT) == DK_CM_PRIV_YES)
            System.out.println(" +++can print document");
          else
            System.out.println(" ---cannot print document");
          if (priv.checkAuthorization(DK_CM_PRIV_VIEW_ANNOTATION) == DK_CM_PRIV_YES)
            System.out.println(" +++can view annotations");
          else
            System.out.println(" ---cannot view annotations");
          if (priv.checkAuthorization(DK_CM_PRIV_CREATE_ANNOTATION) == DK_CM_PRIV_YES)
            System.out.println(" +++can create annotations");
          else
            System.out.println(" ---cannot create annotations");
          if (priv.checkAuthorization(DK_CM_PRIV_DELETE_ANNOTATION) == DK_CM_PRIV_YES)
            System.out.println(" +++can delete annotations");
          else
            System.out.println(" ---cannot delete annotations");
          if (priv.checkAuthorization(DK_CM_PRIV_MODIFY_MULTI_VIEWDATA) == DK_CM_PRIV_YES)
            System.out.println(" +++can change logical views");
          else
            System.out.println(" ---cannot change logical views");
        }
        System.out.println("object type is: "+pid.getObjectType());

        System.out.println("\npid string: " + pid.pidString());
        k = p.propertyId(DK_CM_PROPERTY_ITEM_TYPE);

        if (k > 0)
        {
          Short sVal = (Short)p.getProperty(k);
          j = sVal.shortValue();
          switch (j)
          {
            case DK_CM_DOCUMENT :
              System.out.println("  Item is a document ");
              break;
          }
        }

        cnt_str = new String(Integer.toString(cnt));
        DataFile     = OutDir + new String("docdata")  + cnt_str;
        ResourceFile = OutDir + new String("resource") + cnt_str + new String(".dat");
        TocFile = OutDir + new String("Toc") + cnt_str + new String(".dat");
        LogViewFile  = OutDir + new String("LogView")  + cnt_str + new String(".dat");

        System.out.println("  DataFile=>"+DataFile+"<=");
        System.out.println("  ResourceFile=>"+ResourceFile+"<=");
        System.out.println("  TocFile=>"+TocFile+"<=");
        System.out.println("  LogViewFile=>"+LogViewFile+"<=");

        // Removed existing output files
        File ResFile = new File(ResourceFile);
        if (ResFile.exists())
          ResFile.delete();
        File TOCFile = new File(TocFile);
        if (TOCFile.exists())
          TOCFile.delete();
        File ViewFile = new File(LogViewFile);
        if (ViewFile.exists())
          ViewFile.delete();

        System.out.println("  Number of Data Items: " + numDataItems);

        for (j = 1; j <= numDataItems; j++)
        {
          a = p.getData(j);
          strDataName = p.getDataName(j);
          System.out.println("    " + j + ". Attribute Name: " + strDataName );
          System.out.println("       type: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_TYPE));
          System.out.println("       nullable: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_NULLABLE));

          if (strDataName.equals(DK_CM_DKPARTS) == false &&
              strDataName.equals("DKResource") == false &&
              strDataName.equals("DKViews") == false &&
              strDataName.equals("DKLargeObject") == false &&
              strDataName.equals("DKFixedView") == false &&
              strDataName.equals("DKAnnotations") == false)
          {
            System.out.println("       attribute id: " + p.getDataPropertyByName(j,DK_CM_PROPERTY_ATTRIBUTE_ID));
          }

          if (a instanceof String)
          {
            System.out.println("       Attribute Value(String): =>" + a + "<=");
          }
          else if (a instanceof Integer)
          {
            System.out.println("       Attribute Value(Integer): " + a);
          }
          else if (a instanceof Short)
          {
            System.out.println("       Attribute Value(Short): " + a);
          }
          else if (a instanceof Long)
          {
            System.out.println("       Attribute Value(Long): " + a);
          }
          else if (a instanceof Float)
          {
            System.out.println("       Attribute Value(Float): " + a);
          }
          else if (a instanceof Double)
          {
            System.out.println("       Attribute Value(Double): " + a);
          }
          else if (a instanceof DKDate)
          {
            System.out.println("      Attribute Value(DKDate): " + a);
          }
          else if (a instanceof DKTime)
          {
            System.out.println("       Attribute Value(DKTime): " + a);
          }
          else if (a instanceof DKTimestamp)
          {
            System.out.println("       Attribute Value(DKTimestamp): " + a);
          }
          else if (a instanceof dkCollection)
          {
            System.out.println("       Attribute Value is collection");
            pCol = (dkCollection)a;
            pIter = pCol.createIterator();
            i = 0;

            while (pIter.more() == true)
            {
              i++;
              a = pIter.next();
              pDO = (dkDataObjectBase)a;

              if (pDO.protocol() == DK_CM_XDO)
              {
                System.out.println("         dkXDO object " + i + " in collection");
                pXDO = (dkXDO)pDO;
                DKPid pid2 = pXDO.getPidObject();
                System.out.println("           XDO pid string: " + pid2.pidString());

                // Retrieve and open instance handler for an XDO
                pXDO.retrieve();
                MimeType = pXDO.getMimeType();
                System.out.println("MimeType = " + MimeType);
                System.out.println("Content Begin:");

                byte[] DataBytes = pXDO.getContent();
                String DataString = new String(DataBytes);
                long DataSize = DataString.length();
                long item_type = ((DKPidXDOOD)pXDO.getPidObject()).getType();
                long part_id = ((DKPidXDOOD)pXDO.getPidObject()).getPartId();
                switch ((short)item_type)
                {
                  case DK_CM_ANNOTATION:
                    // binary
                    System.out.println("======DK_OD_ANNOTATION (Notes)======");
                    annotation = (DKAnnotationOD)((DKBlobOD)pXDO).getExtension("dkAnnotationExt");
                    //numNotes   = annotation.getNumNotes();
                    //noteString  = "NOTE " + new String(annotation.getNoteNumber);
                    //noteString += " of " + new String(annotation.numNotes);
                    noteString  = "NOTE: " + annotation.getTimestamp();
                    noteString += " by " + annotation.getUserID();
                    noteString += " for page " + Integer.toString(annotation.getPageNumber());
                    noteString += "\n";
                    noteString += annotation.isPublic() ? "Public" : "Private";
                    noteString += "--";
                    noteString += annotation.isOkToCopy() ? "Can" : "Cannot";
                    noteString += " be copied to another server";
                    noteString += new String("\n") + annotation.getText();
                    System.out.println( noteString );
                    break;
                  case DK_OD_VIEW:
                    // binary
                    System.out.println("======DK_OD_VIEW (Logical Views)======");
                    myLogViewFile = new FileOutputStream(LogViewFile, true);  // append
                    myLogViewFile.write(DataBytes);
                    myLogViewFile.close();
                    break;
                  case DK_OD_SEGMENT:
                    // binary
                    if ( extension.equals("") == true )
                    {
                      if ( MimeType.equals("application/afp") == true )
                        extension = new String("afp");
                      else if ( MimeType.equals("application/ondemand line") == true )
                        extension = new String("lin");
                      else if ( MimeType.equals("text/plain") == true )
                        extension = new String("txt");
                      else if ( MimeType.equals("image/bmp") == true )
                        extension = new String("bmp");
                      else if ( MimeType.equals("image/gif") == true )
                        extension = new String("gif");
                      else if ( MimeType.equals("image/jpeg") == true )
                        extension = new String("jpg");
                      else if ( MimeType.equals("image/pcx") == true )
                        extension = new String("pcx");
                      else if ( MimeType.equals("image/tif") == true )
                        extension = new String("tif");
                      else if ( MimeType.equals("application/pdf") == true )
                        extension = new String("pdf");
                      else
                        extension = new String("dat");
                    }
                    System.out.println("======DK_OD_SEGMENT (Doc Data) part_id=" + part_id + "======");
                    segmentFile = DataFile + new String(".") + extension;
                    if ( part_id == 0 )
                    {
                      File SegFile = new File(segmentFile);
                      if (SegFile.exists())
                        SegFile.delete();
                    }
                    // The first segment starts at part_id 0
                    if ( part_id >= 0)
                    {
                      myDataFile = new FileOutputStream(segmentFile, true);  // append
                      myDataFile.write(DataBytes);
                      myDataFile.close();
                    }
                    break;
                }
                System.out.println("\nContent End");
                // pXDO.open();
              }
            }
          }
          else if (a != null)
          {
            System.out.println("       Attribute Value: " + a.toString());
            if (strDataName.equals("DKResource")  ||
                strDataName.equals("DKFixedView") ||
                strDataName.equals("DKLargeObject"))
            {
              pDO = (dkDataObjectBase)a;

              if (pDO.protocol() == DK_CM_XDO)
              {
                System.out.println("         dkXDO object ");
                pXDO = (dkXDO)pDO;
                DKPid pid2 = pXDO.getPidObject();
                System.out.println("           XDO pid string: " + pid2.pidString());

                // Retrieve and open instance handler for an XDO
                pXDO.retrieve();
                // pXDO.open();

                byte[] DataBytes = pXDO.getContent();
                String DataString = new String(DataBytes);
                long DataSize = DataString.length();
                long item_type = ((DKPidXDOOD)pXDO.getPidObject()).getType();

                if (strDataName.equals("DKResource") )
                {
                  // binary
                  System.out.println("======DK_OD_RESOURCE (AFP Resource Group)======");
                  myResourceFile = new FileOutputStream(ResourceFile, true);  // append
                  myResourceFile.write(DataBytes);
                  myResourceFile.close();
                }
                else if (strDataName.equals("DKFixedView"))
                {
                  // string
                  System.out.println("======DK_OD_FIXED_VIEW (Fixed View)======");
                  System.out.println( DataString );
                }
                else if (item_type == DK_OD_TOC)
                {
                  // binary
                  System.out.println("======DK_OD_TOC (Large Object table of contents)======");
                  myTocFile = new FileOutputStream(TocFile, true);  // append
                  myTocFile.write(DataBytes);
                  myTocFile.close();
                  //System.out.println( DataString );
                }
              }
            }
          }
          else System.out.println("       Attribute Value is null");
        }

        DocFile = OutDir + new String("sample") + cnt_str + new String(".") + extension;
        File DocumentFile = new File(DocFile);
        if (DocumentFile.exists())
          DocumentFile.delete();

        // If the resource file exists, concatenate the resource with data
        ResFile = new File(ResourceFile);
        if (ResFile.exists())
        {
          int BytesTotal = 0;
          byte[] buffer = new byte[100000];
          myDocFile = new FileOutputStream(DocFile);
          FileInputStream myResFile = new FileInputStream(ResourceFile);
          if ( myResFile != null)
          {
            while ( myResFile.available() > 0 )
            {
              numBytes = myResFile.read( buffer );
              if ( numBytes == -1  )
              {
                System.err.println( "Resource read error" );
                break;
              }
              myDocFile.write(buffer, 0, numBytes);
              BytesTotal += numBytes;
            }
            myResFile.close();
            ResFile.delete();
          }
          segmentFile = DataFile + new String(".") + extension;
          FileInputStream myInDataFile = new FileInputStream(segmentFile);

          // copy data
          if ( myInDataFile != null)
          {
            int DateBytes = 0;
            while ( myInDataFile.available() > 0 )
            {
              numBytes = myInDataFile.read(buffer);
              if ( numBytes == -1  )
              {
                System.err.println( "Data read error" );
                break;
              }
              DateBytes += numBytes;
              myDocFile.write(buffer, 0, numBytes);
              BytesTotal += numBytes;
            }
            myInDataFile.close();
            File SegFile = new File(segmentFile);
            SegFile.delete();
          }
          myDocFile.close();

        } // if (ResFile.exists())

      }  // if (p != null)

    }
    catch (DKException exc)
    {
      System.out.println("Exception name " + exc.name());
      System.out.println("Exception message " + exc.getMessage());
      System.out.println("Exception error code " + exc.errorCode());
      System.out.println("Exception error state " + exc.errorState());
      exc.printStackTrace();
    }
    catch (Exception exc)
    {
      System.out.println("Exception message " + exc.getMessage());
      exc.printStackTrace();
    }
  }
}
